<?php


/**
 * class of helper functions which can be called statically
 * 
 * @license GPLv2
 * @license 2006 KBCMS LLC
 * @author  Jonathan Hendler
 * @package SONIA
 * @version 0.3 
 * */
class SONIA_H {
    //SONIA_H :: dbgtrc()
    public static function dbgtrc(){
        $bktrc = debug_backtrace();
        $bktrc_print = '';
        
        foreach($bktrc as $xxx){
          $bktrc_print .= "{$xxx['file']}{$xxx['line']}\n";
        };
        
        dbg($bktrc_print,'dbgbackgrace');
    }
    
    /**
     * function to check the standard SONIA tree array
     * for 
     * 
     * $array[IRI][name] //label
     *            [subclasses]
     *            [count]
     *            [count_in_search]
     *            [in_search]
     * @todo, log or throw errors
     * */
    public static function chk(& $array, $check_all = false, $property = false) {
        $this_checks = true;
        foreach ($array as $key => $value) {
            $this_checks = $this_checks && isset ($value['name']) && SONIA_H :: is_IRI($key);
            if (isset ($value['subclasses'])) {
                $this_checks = $this_checks && SONIA_H :: chk($value['subclasses'], $check_all, $property);
            }

            if (!$this_checks && (SONIA_DEBUG)) {
                trigger_error('SONIA_DEBUG:  helpers.sonia.inc chk -  died at key ' . $key . ' values ' . nl2br(htmlentities(print_r($value, 1))));
            }

        }
        return $this_checks;
    }

    /**
     * checks that the IRI is valid URI or SPARQL namespace abreviation
     * @todo could be much improved
     * */
    public static function is_IRI($uri) {
        return (strstr($uri, 'http'));
    }

    /**
     * parse out the base URI and get the id 
     * */
    public static function getIDFromURI($uri) {
        return str_replace('#', '', stristr($uri, '#'));
    }

    /**
     * do filters on data coming directly from forms
     * */
    public static function prep_input($form_input) {
        if (!is_array($form_input)) {
            return $form_input;
        }

    }

    /**
     * parses the return type for images 
     * */
    public static function getFileInfo(& $image_info) {
        $resturn_data = array ();

        foreach ($image_info as $data) {
            switch ($data['p']) {
                case SONIA_CREATE_FILENAME :
                    $resturn_data['filename'] = $data['o'];
                    break;
                case SONIA_CREATE_FILESIZE :
                    $resturn_data['filesize'] = $data['o'];
                    break;
                case SONIA_CREATE_FILEPATH :
                    $resturn_data['path'] = $data['o'];
                    break;
                case SONIA_RDFS_CREATE_LABEL :
                    $resturn_data['label'] = $data['o'];
                    break;
                case SONIA_RDFS_CREATE_COMMENT :
                    $resturn_data['comment'] = $data['o'];
                    break;

            }
        }

        return $resturn_data;
    }

    /**
     * given string content prompts the browser to trigger a download
     * 
     * */
    public static function SendContent($content, $filename, $mime_type = 'application/pdf') {
        header("Pragma: public");
        header("Expires: 0");
        //stuff for proxies
        header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
        header("Cache-Control: private", false);
        header('Content-type: ' . $mime_type);
        header('Content-Disposition: attachment; filename="' . $filename . '"');
        header("Content-Transfer-Encoding: binary");
        header("Content-Length: " . @ strlen($content));
        set_time_limit(0);
        echo $content;
        flush();
        die();
    }

    //TODO unfinished
    //   function SendFile($file_location, $filename, $mime_type = 'application/pdf')
    //   {
    //     if(!is_readable($file_location.'/'.$filename))
    //     {
    //       return false;
    //       
    //     }
    //     else
    //     {      
    //        header("Pragma: public");
    //        header("Expires: 0");
    //        //stuff for proxies
    //        header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
    //        header("Cache-Control: private",false);
    //        header('Content-type: '.$mime_type);
    //        header('Content-Disposition: attachment; filename="'.$filename.'"');
    //        header("Content-Transfer-Encoding: binary");
    //        header("Content-Length: ".@filesize($file_location.'/'.$filename));
    //        set_time_limit(0);
    //        readfile($file_location.'/'.$filename);
    //
    //        return true;
    //     }
    //   }
    //   
    //   
    /**
    * recursive function to find the hiearchy of classes
    * */
    static function buildHierarchy(& $temp_hiercharchy, $startclass) {
        $temp_array = array ();

        if (isset ($temp_hiercharchy[$startclass])) {
            $_startClass = $temp_hiercharchy[$startclass];

        } else {
            $_startClass = null;
        }

        $temp_array[$startclass] = ($_startClass != null && isset ($_startClass['name'])) ? $_startClass : array (
            'name' => $startclass
        );

        if (!empty ($_startClass['subclasses']) && is_array($_startClass['subclasses'])) {
            foreach ($_startClass['subclasses'] as $key) {
                $temp_array[$startclass]['subclasses'][] = SONIA_H :: buildHierarchy($temp_hiercharchy, $key);
            }
        }

        return $temp_array;
    }

    /**
     *  
     * */
    static function encodeURI($uri) {
        return rawurlencode(htmlentities($uri));
    }

    static function decodeURI($uri_encoded) {
        if (is_array($uri_encoded)){
          //dbg($uri_encoded,'$uri_encoded');
          //debug_print_backtrace();
          return;
        }
        return html_entity_decode(rawurldecode($uri_encoded));
    }

    /**
     * encodes and serializes arrays for use in URLS
     * @see sonia_decode_array
     * */
    public static function sonia_encode_array($array) {
        $return_array = array ();
        foreach ($array as $item) {
            $return_array[] = base64_encode($item);
        }

        return SONIA_H :: encodeURI(serialize($return_array));
    }

    /**
     * decodes and serializes arrays from in URLS
     * @see sonia_encode_array()
     * */
    public static function sonia_decode_array($array) {
        $return_array = array ();
        $temp_array = unserialize(SONIA_H :: decodeURI($array));

        foreach ($temp_array as $item) {
            $return_array[] = base64_decode($item);
        }

        return $return_array;
    }

    static function debug($content, $title = '', $echo = false) {
        //TODO backtrace
        //$backtrace = debug_backtrace();

        if (SONIA_DEBUG) {
            $html = '<div class="debug" style="text-align: left; margin-left: 30px;">';
            if ($title != '') {
                $html .= "<h3>$title</h3>";
            }
            if (is_array($content)) {
                $html .= '<pre>' . nl2br(htmlentities(print_r($content, 1))) . '</pre>';
            } else {
                $html .= $content;
            }
            $html .= '</div>';
        }

        if ($echo)
            echo $html;
        return $html;
    }

    /**
    * Given an IRI/URI (a string) of a class, the function parses a set of classes 
    * and recursively builds the set of the superclasses.
    * Searches for the superclasses of a category in cached values given an IRI
    * because there is not just one root class. Also preserves hierarchy rather than
    * returning a simple list of superclasses.
    * The data structure operated on follows the form:
    * $classes[URI]['subclasses']{...[URI]['subclasses']}
    *              ['name']
    * It returns an array of the same structure.
    * 
    * 
    * @param string iri the iri to find a name for
    * @param array classes a set of classes in a sonia standard associative array
    * @return array
    * @see SONIA->getSuperClassesRecursive() 
    * */
    public static function _getSuperClassesRecursive($iri, $classes) {
        $final_array = array ();

        foreach ($classes as $class => $info) {
            if (isset ($info['subclasses']) && is_array($info['subclasses']) && !empty ($info['subclasses'])) {
                $subclasses = SONIA_H :: _getSuperClassesRecursive($iri, $info['subclasses']);

                if (is_array($subclasses) && !empty ($subclasses)) {
                    $final_array[$class]['name'] = $info['name'];
                    $final_array[$class]['subclasses'] = $subclasses;
                }
            }

            //it may also be the case that the class is the one we are looking for 
            //although a class isn't going to be a subclass of itself
            if ($class == $iri) {
                $final_array[$class]['name'] = $info['name'];
            }
        } //end foreach

        return $final_array;
    }

    /**
     * converts the standard array hierarchy to a simple list/array
     * */
    public static function flattenHieararchy($classes) {
        $final_array = array ();
        foreach ($classes as $class => $info) {
            $final_array[$class] = $class;

            if (isset ($info['subclasses']) && is_array($info['subclasses']) && !empty ($info['subclasses'])) {
                $final_array = array_merge($final_array, SONIA_H :: flattenHieararchy($info['subclasses']));
            }
        }

        return $final_array;
    }
    
    /**
     * remove unwanted charactors
     * */
    public static function filter_input_text($string){
      $string = str_replace('"','',$string);
      //temp replaces space
      $string = str_replace(' ','88888888888',$string);
      $pattern = '/[\W]*/is';
      $string = preg_replace($pattern, '',$string);
      $string = str_replace('88888888888',' ',$string);
      //there should be at least one word char
      if (!preg_match('/[\w]+/is',$string)) $string = '';
      return $string;
    }
    
    public static function testSearchResultStructure(& $result, $type) {
        if ($type = 'class') {

        } else
            if ($result = 'property') {

            } else
                if ($result = 'object') {

                }

    }

    public static function word_between($needle, $word_one, $word_two) {
        return (($needle >= $word_one) && ($needle <= $word_two));
    }

    //TODO most of these below are for the local SONIA and should be moved

    //   /**
    //   * helper function wraps xmlTreeMaker
    //   * */
    //   static function getXMLarray($source_file)
    //    {
    //        assert(function_exists('xmlTreeMaker'));
    //        $fp = fopen($source_file, "r");
    //        assert($fp != null);
    //        $data = fread($fp, filesize($source_file));
    //        flush();
    //        return  xmlTreeMaker($data);
    //    }
    // 

    /**
      * 
      * Given a full URI separate the http from the name
      * 
      * @todo get namespace from a non abbreveiited URI
      * @param string $uri expects $uri in the abbreviated form ns1:name
      * @return array [ns],[v]
      * @see basicQuery
      * @see query
      * */
    //  static function getNamespaceFromURI($uri)
    //  {
    //    $len = strlen($uri);
    //    $array = array();
    //   
    //    if (strstr($uri,'#'))
    //    {
    //      //echo 1;
    //      $pos = strpos($uri,'#');
    //      $array['ns'] = substr($uri,0,abs($pos+1));
    //      $array['v'] = substr($uri,$pos+1);
    //    }
    //    else if (strstr($uri,':'))
    //    {
    //      //echo 2;
    //      $pos = strpos($uri,':');
    //      $array['ns'] = substr($uri,0,abs($pos));
    //      $array['v'] = substr($uri,$pos+1);
    //    }
    //    else 
    //    {
    //      //echo 3;
    //      trigger_error('"'.$uri.'" does not appear to be a valid URI', E_USER_NOTICE);
    //      $array['ns'] = NULL;
    //      $array['v'] = $uri;
    //    }
    //    
    //    return $array;
    //  }

    // static function getFullValue($abv_value)
    // {
    //    $result = SONIA_H :: getNamespaceFromURI($abv_value);
    //    return $this->ns_table[$result['ns']].$result['v'];
    // }

    ///////////////// belongs 
    // /**
    //  * 
    //  * */
    // static function getRDFNamespaces(&$namespaces)
    // {
    //    foreach($namespaces as $key=>$value)
    //    {
    //      $r = SONIA_H :: getNamespaceFromURI($key);
    //      $abv = $r['v'];
    //      $this->setFullNamespaceArray($abv, $value);
    //    }
    // }
    // 
    // 
    ///**
    //   * load a URI in the set of preset namespaces
    //   * */
    //  function setFullNamespaceArray($ns_abrev, $ns)
    //  {
    //    $this->ns_table[$ns_abrev] = $ns;
    //  }
    //  
    //  /**
    //   * do a quick lookup in a URI
    //   * */
    //  function getFullNamespaceFromAbrev($ns)
    //  {
    //    if(isset($this->ns_table[$ns])) 
    //    {
    //      return $this->ns_table[$ns];
    //    }
    //    else
    //    {
    //      return $this->default_ns_long;
    //    }
    //    
    //  }

    ///**
    //  * import an RDF file 
    //  * */
    // function importFromRDF($source_file)
    // {
    //    //load file
    //    $data = $this->getXMLarray($source_file);
    //    //get namespaces and aliases
    //    $namespaces = $data['rdf:RDF']['@'];
    //
    //    $this->getRDFNamespaces($namespaces);
    //    
    //    $data = $data['rdf:RDF']['#'];
    //    
    //    //loop through tree recursively and create statements
    //    $this->recursiveFind($data);
    // }
    // 
    // function exportToRDF()
    // {
    // 
    // }

} //end class
