<?php


//SONIA_IO
/**
 * 
 * STATE handling and IO
 * This implimentation may assume the avaialability of Drupal functions
 * Uses Drupal 4.7+
 * 
 * there are two levels of persistance in the cache. One is for all the pages a 
 * user traverses, another is for individual searches.
 * 
 * @see http://api.drupal.org/apis/4.7/cache
 * 
 * @license GPLv2
 * @license 2006 KBCMS LLC
 * @author  Jonathan Hendler
 * @package SONIA
 * @version 0.3 
 * */
class SONIA_IO extends SO_CA implements SONIA_IO_API {
  /**
   * $_REQUEST 
   * */
  public $request;

  /**
   * $_SESSION
   * */
  private $session;

  /**
   * search params
   * */
  private $ns;

  public function __construct() {
    $this->setRequest(null);
  }

  /**
   * In Drupal you may pass $_REQUEST or something retrieved from form
   * 
   * */
  public function setRequest($request) {
    if ($request != null && is_array($request)) {
      $this->request = $request;
    } else {
      $this->request = $_REQUEST;

      if (SONIA_FORM_REQUEST_PREFIX != null && isset ($_REQUEST[SONIA_FORM_REQUEST_PREFIX])) {
        $this->request_post = $_REQUEST[SONIA_FORM_REQUEST_PREFIX];
      }
    }
  }

  //---------------------------------------------------------------------------
  /**
   * helper function for retrieving the integer value of the pagination
   * @return int the begin page
   * */
  //---------------------------------------------------------------------------
  public static function getPaginationBegin() {
    //try drupal
    //http://api.drupal.org/api/4.7/function/variable_get
    if (function_exists('variable_get') && variable_get(SEMANTIC_SEARCH_PAGINATION_SIGNAL, 0) != 0) {
      $begin = variable_get(SEMANTIC_SEARCH_PAGINATION_SIGNAL, 0);
    } else
      if (isset ($_REQUEST[SONIA_FORM_REQUEST_PREFIX]) && isset ($_REQUEST[SONIA_FORM_REQUEST_PREFIX][SEMANTIC_SEARCH_PAGINATION_SIGNAL])) {

        $begin = intval(trim($_REQUEST[SONIA_FORM_REQUEST_PREFIX][SEMANTIC_SEARCH_PAGINATION_SIGNAL]));
      } else
        if (isset ($_REQUEST[SEMANTIC_SEARCH_PAGINATION_SIGNAL])) {

          $begin = intval(trim($_REQUEST[SEMANTIC_SEARCH_PAGINATION_SIGNAL]));
        } else {

          $begin = 0;
        }
    return $begin;
  }

  /**
   * simple get function
   * */
  public function getSearchParams() {
    return $this->search_params;
  }

  /**
   * handles the removal of elements from the search, via retreival of information
   * from the URL/POST
   * 
   * */
  private function unsetSearchFacet() {
    if (isset ($this->request['remove'])) {
      if ($this->request['remove'] == 'qstring') {
        //TODO , this mess is probably silly
        if (!is_array($this->search_params['qstring'])) {
          if (isset ($this->search_params['qstring'][SONIA_H :: decodeURI($this->request['vn'])]))
            unset ($this->search_params['qstring'][SONIA_H :: decodeURI($this->request['vn'])]);
        } else {
          foreach ($this->search_params['qstring'] as $qs => $val) {
            if (!is_array($val)) {

              if (isset ($this->search_params['qstring'][SONIA_H :: decodeURI($this->request['vn'])]) 
                && $val == SONIA_H :: decodeURI($this->request['vv'])) {
                unset ($this->search_params['qstring'][SONIA_H :: decodeURI($this->request['vn'])]);
              }else if (isset ($this->search_params['qstring'][SONIA_H :: decodeURI($this->request['vv'])]) 
                && $val == SONIA_H :: decodeURI($this->request['vn'])) {
                unset ($this->search_params['qstring'][SONIA_H :: decodeURI($this->request['vv'])]);
              }
              
           } else {
              if (isset ($val[SONIA_H :: decodeURI($this->request['vn'])]) 
                && $val[SONIA_H :: decodeURI($this->request['vn'])] == SONIA_H :: decodeURI($this->request['vv'])) {
                unset ($this->search_params['qstring'][$qs][SONIA_H :: decodeURI($this->request['vn'])]);
              }else if (isset ($val[SONIA_H :: decodeURI($this->request['vv'])]) 
                && $val[SONIA_H :: decodeURI($this->request['vv'])] == SONIA_H :: decodeURI($this->request['vn'])) {
                unset ($this->search_params['qstring'][$qs][SONIA_H :: decodeURI($this->request['vv'])]);
              }
           
           }
          }
        }
      } else {
        if ($this->request['remove'] != 'facet' && isset ($this->search_params[$this->request['remove']]))
          unset ($this->search_params[$this->request['remove']]);
        if (isset ($this->search_params['cat'][$this->request['remove']]))
          unset ($this->search_params['cat'][$this->request['remove']]);
        
        if ($this->request['remove']== 'facet' && isset ($this->search_params[$this->request['remove']] )){
          
          foreach($this->search_params['facet'] as $k =>$v){
            if (in_array(SONIA_H :: decodeURI( $this->request['vv']), $v)) {
                unset ($this->search_params['facet'][$k]);
            }
          }
        }
        
        if (is_array($this->search_params['p_value'])) {
          $remove = SONIA_H :: decodeURI($this->request['remove']);
          $remove_value = (isset($this->request['vv'])) ? SONIA_H :: decodeURI($this->request['vv']):'';
          foreach ($this->search_params['p_value'] as $k => $v) {
            if (!is_array($v)) {
              $k = SONIA_H :: decodeURI($k);
              $v = SONIA_H :: decodeURI($v);
              $this->_removePvalueSearchItem($remove, $k, $v,$remove_value);
            } else {
              foreach ($v as $kk => $vv) {
                $kk = SONIA_H :: decodeURI($kk);
                $vv = SONIA_H :: decodeURI($vv);
                $this->_removePvalueSearchItem($remove, $kk, $vv,$remove_value, $k);
              }
            }
          }
        }
      }
    }
  }

  /**
   * helper function to clean up unsetSearchFacet()
   * */
  private function _removePvalueSearchItem($remove, $k, $v, $remove_value,$k1 = null) {
    if ($remove == $k && $remove_value == $v){
         if ($k1 === null){
           if (isset($this->search_params['p_value'][$v]))  unset($this->search_params['p_value'][$v]);
           if (isset($this->search_params['p_value'][$k]))  unset($this->search_params['p_value'][$k]);
         }
         else{
           unset($this->search_params['p_value'][$k1]);
         }
    } else if ($remove == $v && $remove_value == $k){
         if ($k1 === null){
            if (isset($this->search_params['p_value'][$v]))  unset($this->search_params['p_value'][$v]);
            if (isset($this->search_params['p_value'][$k]))  unset($this->search_params['p_value'][$k]);
         }
         else{
           unset($this->search_params['p_value'][$k1]);
         }
    }
  }
  //--------------------------------------------------------------------------
  /**
  * This function provides a mechanism for searches to be preserved, updated and deleted
  * it is the major function which automates the retrieval of neccesary PARAMS
  * 
  * @param string $request_var the stand request var prefix
  * @return void
  * */
  //--------------------------------------------------------------------------
  public function setSearchParams($request_var) {
    //get new params
    $this->setNewSearchParams($request_var);
    //do removals
    $this->unsetSearchFacet();
    //check
    $this->setNewSearch( );
    
  }

  /**
   * handles the removal of session data to make for a clean, new search
   * */
  private function setNewSearch( ) {
    if (isset ($this->request[SONIA_NEWSEARCH_SIGNAL])) {
      $this->search_params = null;
    }
  }

  public function notReservedURLKey($key) {
    if (is_array($key))
      return false;

    $reserved_keys = array (
      '_s_',
      'sh',
      'v',
      'form_id',
      'form_token',
      SONIA_CRUD_ACTION_SIGNAL,
      SONIA_CRUD_SUBJECT,
      SONIA_CRUD_PREDICATE,
      SONIA_CRUD_OBJECT,
      SONIA_CRUD_OLD_OBJECT,
      SONIA_SEARCH_SUBMIT_SIGNAL,
      SONIA_COMMENCE_SEARCH,

      
    );

    return !in_array($key, $reserved_keys);

  }

  /***
  * assembles variables used to create search from the REQUEST instead of from SESSION
  * */
  private function setNewSearchParams() {
    //then check to see if there is a new search which overwrites this search
    if (isset ($this->request[SONIA_FORM_REQUEST_PREFIX])) {
      $temp_search_params = $this->request[SONIA_FORM_REQUEST_PREFIX];
    } else {
      $temp_search_params = array ();
    }
    
    /*
     * $this->search_params[0][0][0] = 0; is to instatiate the correct depth to the array
     * PHP otherwise will overwrite various levels of the array
     * the PHP folk call it a "feature"
     * */
    if (is_array($temp_search_params) && !empty ($temp_search_params)) {
      if (!is_array($this->search_params) || empty ($this->search_params))
        $this->search_params[0][0][0] = 0;
      foreach ($temp_search_params as $k => $v) {
        //ignore many params
        
        if (!is_array($v) && $this->notReservedURLKey($k) && $k != '') {
          
          if (preg_match('/^cat/i', $k)) {
            $this->search_params['cat'][$v] = 1;
          } else
            if (preg_match('/^facet/i', $k)) {
              $this->search_params['facet'][$v] = 1;
            }
          //            else if(preg_match('/^qstring/i',$k))
          //            {
          //              $this->search_params['qstring'][$v] = 1;
          //            }
          else {
            $this->search_params[$k] = $v;
          }

        } else
          if (!empty ($v) && is_array($v)) {
            //hack for one category
            if (false){
            //if ($k == 'cat') {
              //dbg('is cat?');
//              foreach ($v as $kk => $vv) {
//                unset ($v[$kk]);
//                $fixed = SONIA_H :: decodeURI($kk);
//                //$v[$fixed] = $vv;
//                if ($k != 0)
//                  $this->search_params[$k][$fixed] = $vv;
//              }

            } else  
              {
              $count = 0;
              foreach ($v as $k2 => $v2) {

                if ($k2 !== 'x5') {
                  if (!is_array($v2)) {
                    $fixed = SONIA_H :: decodeURI($k2);
                    if (!$this->dupe($k,$fixed, $v2)){
                      if ($v2 != '0'){
                        //doesn't allow multiple of "fixed"
                        $this->search_params[$k][$fixed] = $v2;
                        $temp_array = array ();
                        $count++;
                      }
                    }
                  } else {
                    foreach ($v2 as $kk2 => $vv2) {
                      $fixed = SONIA_H :: decodeURI($kk2);
                      $fixed2 = SONIA_H :: decodeURI($vv2);
                      if ($k == 'qstring') {
                        $fixed2 = SONIA_H :: filter_input_text($fixed2);
                      }
                      
                      if (!$this->dupe($k, $fixed, $fixed2)) {
                        if ($vv2 != '0'){ //might need to change this to a signal
                          //dbg ($temp_array,'$temp_array');
                          $this->search_params[$k][$count][$fixed] = $fixed2;
                          $temp_array = array ();
                          $count++;
                        }
                      }
                      else{
                       // dbg(null,"dupe($type,$key, $value)!!");
                      }
                    }
                  }
                } else {
                  $v2 = trim($v2);
                  if ($v2 != '' && $v2 != 0) {
                    $this->search_params[$k][$count] = $v2;
                    $count++;
                  }
                }
              }
            }
          }
      }
    }
   //dbg($temp_search_params, '$temp_search_params');
    //dbg($this->search_params, '$this->search_params');
  }
  
  private function dupe($type,$key, $value){
     
     if (isset($this->search_params[$type][$key] ) 
       && !is_array($this->search_params[$type][$key])
       && $this->search_params[$type][$key] === $value){
         return true;
       } else {
          
         if (is_array($this->search_params[$type])){
           foreach($this->search_params[$type] as $k =>$v){
             if (isset($v[$key]) && ($v[$key] === $value )){ 
               return true;
             }
           }
         }
       }
     
     return false;
     
  }

  //--------------------------------------------------------------------------
  /**
  * 
  * @return boolean should the search be exectued
  * */
  //--------------------------------------------------------------------------
  public function doSearch() {
    $reset_signal_set = !empty ($this->request[SONIA_NEWSEARCH_SIGNAL]);

    if (!$reset_signal_set) {
      if (isset ($this->do_search) && $this->do_search) {
        return true;
      } else
        if (isset ($this->request[SONIA_SEARCH_SUBMIT_SIGNAL]) && $this->request[SONIA_SEARCH_SUBMIT_SIGNAL]) {
          return true;
        } else
          if (isset ($this->request[SONIA_FORM_REQUEST_PREFIX][SONIA_SEARCH_SUBMIT_SIGNAL]) && $this->request[SONIA_FORM_REQUEST_PREFIX][SONIA_SEARCH_SUBMIT_SIGNAL]) {
            return true;
          }
    }

    return false;
  }

  public function setSearchOn() {
    $this->do_search = true;
  }
  //-------------------------------------------------------------------------- 
  /**
  * Signals if a request to update data has been submitted
  * 
  * USAGE:
  * @see parseCRUDFromREQUEST()
  * @return boolean
  * @todo add user permissions and managment
  * */
  //--------------------------------------------------------------------------
  public function doCRUD() {
    if (SONIA_FORM_REQUEST_PREFIX != null) {
      if (isset ($this->request[SONIA_FORM_REQUEST_PREFIX]) && isset ($this->request[SONIA_FORM_REQUEST_PREFIX][SONIA_CRUD_REQUEST_SIGNAL]))
        return true;
    }
    return isset ($this->request[SONIA_CRUD_REQUEST_SIGNAL]);
  }

  //----------------------------------------------------------------------------
  //----------------------------------------------------------------------------
  public function getRequestValues() {
    return $this->request;
  }

  ////////////////////////////////////////////////////////////////////////////
  // LOGGING error and file related
  ////////////////////////////////////////////////////////////////////////////
  //todo add an error check
  public function logError($message) {
    $fh = fopen(SONIA_ERROR_LOG, 'a');
    fwrite($fh, $message . "\n");
    fclose($fh);
  }

  public function throwError($message, $error_level = E_USER_ERROR) {
    trigger_error($message, $error_level);
  }

  ////////////////////////////////////////////////////////////////////////////
  // Import and Export
  ////////////////////////////////////////////////////////////////////////////
  /**
   * import RDF from a file, remote or local
   * 
   * @param string filename is the name of the file
   * @param string url the remote location of the file or path
   * */
  public function importRDF($filename, $url = null, $model_name = null) {

  }

  /**
   * export a portion of the SEMANTIC_SEARCH RDF data to a file
   * */
  public function exportRDF($filename, $path = null, $model_name = null) {

  }
}
