<?php

/**
 * semantic_search_get_arc_config() provides configuration items for ARC
 * which shouldn't need to change
 * for info on changing these configuration items 
 * @see http://arc.web-semantics.org/documentation/api
 * 
 * @package SONIA
 * @version 0.3
 * */
function semantic_search_get_arc_config() {

    $rdf = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
    $rdfs = "http://www.w3.org/2000/01/rdf-schema#";
    $owl = "http://www.w3.org/2002/07/owl#";

    $db_host = ARC_DB_HOST . ":" . ARC_DB_PORT;
    $db_user = ARC_DB_USER;
    $db_pwd = ARC_DB_PW;
    $db_name = ARC_DB_NAME;

    /*                                            
     *  http://arc.web-semantics.org/documentation/api
     
      
       
         "store_type"=>"basic+",            // (basic|basic+|split)
      "id_type"=>"hash_int",             // (hash_int|hash_md5|hash_sha1|incr_int)
      "reversible_consolidation"=>false, // adds additional columns
      "index_type"=>"advanced",          // (basic|advanced)
      "index_graph_iris"=>true,          // add graph columns to indexes
      "index_words"=>false,              // creates FULLTEXT index on values
      "charset"=>"utf8"                  // for MySQL, if supported
    
    
    $rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#";
    $foaf="http://xmlns.com/foaf/0.1/";
    $pim="http://example.com/pim/pim#";
    
    $config["prop_tables"] = array(
      array(
        "name"=>"type",
        "prop_type"=>"obj",
        "props"=>array($rdf."type")
      ),
      array(
        "name"=>"private",
        "prop_type"=>"obj",
        "props"=>array($foaf."mbox", $pim."fax")
      )
    );
    
    $conf = array_merge($conf, array(
      // split store query optimization 
      "obj_props"=>array(), // a list of object property IRIs
      "dt_props"=>array(), // a list of datatype property IRIs
    
      // add_data parameters 
      "loader_id2val_sql_buffer_size"=>200000,
      "loader_triple_sql_buffer_size"=>200000,
      "loader_id_cache_size"=>2000,
    
      // misc 
      "api_val2id_cache_size"=>2000, 
      "max_describe_depth"=>3, // DESCRIBE: bnode-path-depth 
    
    // proxy 
      "proxy_host"=>"192.168.0.2",
      "proxy_port"=>8080
    ));
    
    
     *  * 
     * */
    $conf = array (
        /* db */
        "db_host" => $db_host,
        "db_name" => $db_name,
        "db_user" => $db_user,
        "db_pwd" => $db_pwd,
        /* store prefix */
        "prefix" => ARC_DB_PREFIX_NAME,
        /* store */
        "store_type" => "basic+",
        "id_type" => "hash_int",
        "reversible_consolidation" => false,
        "encode_values" => false,
        "index_type" => "advanced",
        "index_graph_iris" => true,
        "index_words" => true,
        "charset" => "utf8",
        /* default prefixes */
        "prefixes" => array (
            "rdf" => $rdf,
            "rdfs" => $rdfs,
            "owl" => $owl,
            "semantic_search" => SONIA_DEFAULT_MODEL_IRI,
            "nina" => SONIA_DEFAULT_MODEL_IRI,
            "sonia" => SONIA_BASE_IRI
        )
    );

    /*    "dc"=>"http://purl.org/dc/elements/1.1/",
        "dct"=>"http://purl.org/dc/terms/",
        "foaf"=>"http://xmlns.com/foaf/0.1/",
        
    */

    //move to config
    define('SONIA_ARC_ADVANCED', false);

    define('SONIA_ARC_FASTER', true);

    if (SONIA_ARC_FASTER) {
        $conf = array_merge($conf, array (
                // split store query optimization 
        //"obj_props"=>array(), // a list of object property IRIs
        // "dt_props"=>array(), // a list of datatype property IRIs

        // add_data parameters 
        //  "loader_id2val_sql_buffer_size"=>2000000,
        //  "loader_triple_sql_buffer_size"=>2000000,
        //  "loader_id_cache_size"=>20000,

        // misc 
    "api_val2id_cache_size" => 20000,
                // "max_describe_depth"=>3, // DESCRIBE: bnode-path-depth 

    
        ));
    }

    if (SONIA_ARC_ADVANCED) {
        //get all the sonia consts for IRIs

        $conf["prop_tables"] = array (
            array (
                "name" => "type",
                "prop_type" => "obj",
                "props" => array (
                    $rdf . "type"
                )
            ),
            array (
                "name" => "object",
                "prop_type" => "obj",
                "props" => array (
                    SONIA_OBJECT_ID_URI
                )
            ),
            array (
                "name" => "files",
                "prop_type" => "obj",
                "props" => array (
                    SONIA_FILE_TYPE_URI
                )
            ),
            array (
                "name" => "ldata",
                "prop_type" => "obj",
                "props" => array (
                    SONIA_PROPERTY_ID_URI
                )
            ),
            array (
                "name" => "ldata",
                "prop_type" => "obj",
                "props" => array (
                    SONIA_RDFS_CREATE_CLASS_TYPE
                )
            ),
                //SONIA_RDFS_CREATE_CLASS_TYPE
        //      array(
        //        "name"=>"private",
        //        "prop_type"=>"obj",
        //        "props"=>array($foaf."mbox", $pim."fax")
        //      )
    
        );

        $conf = array_merge($conf, array (
                // split store query optimization 
    "obj_props" => array (
                SONIA_RDFS_CREATE_CLASS_TYPE,
                $rdf . "type"
                ), // a list of object property IRIs
                "dt_props" => array (), // a list of datatype property IRIs

        // add_data parameters 
    "loader_id2val_sql_buffer_size" => 2000000,
            "loader_triple_sql_buffer_size" => 2000000,
            "loader_id_cache_size" => 20000,

                // misc 
    "api_val2id_cache_size" => 20000,
                "max_describe_depth" => 3, // DESCRIBE: bnode-path-depth 

    
        ));
    }

    return $conf;
}
