<?php


/**
 * SONIA
 * Simple Ontology aNd Inference Algorithms
 * 
 * The SONIA Store API provides an internal API for SONIA API to interact with a 
 * an RDF store, Triple Store or other store
 * 
 * 
 * @todo full CRUD operations
 * 
 *
 * 
 * @author  Jonathan Hendler 
 * @package SONIA
 * @license GPLv2
 * @license copyright 2006-2008 Jonathan Hendler
 * @version .03 alpha
 * @date    4/19/2006
 * 
 * 
 * */
class SONIAARCStore implements SONIAStoreAPI {

  private $data_api;
  public $debug_text;

  //INITIALIZATION
  //--------------------------------------------------------------------------
  /**
   *  The conctructor provides a mechanism for instantiating the data store,
   * and loading data at runtime if necessary
   * 
   * @access public
   * @see instantiateBackend()
   * @see loadData
   * */
  //--------------------------------------------------------------------------  
  public function __construct() {
    $this->init(null);
    $this->debug_text = '';
    $this->cache = new SO_CA();
  }
  
  public function maintain($args) {
 
  }
  

  /**
   *  RDF into the rdf store 
   * 
   * TODO needs merging configuriation
   * 
   * @todo validate the RDF
   * @param string $iri_model the name of the model
   * @param string $file the location of the file, can be any uri PHP allows http:// file:/// or just a path
   * */
  public function importRDF($iri_model, $file) {
    $operation = 'add';

    //normally data is RDF, model is RDFS
    $api_name = (!strstr($file, 'rdfs')) ? 'data_api' : 'data_api';

    $rdfxml = file_get_contents($file);

    $cur_args = array (
      "graph_iri" => $iri_model,
      "add_rdfxml" => trim($rdfxml
    ), "result_type" => "array");

    $mthd = $operation . "_data";

    if (method_exists($this-> {
      $api_name }, $mthd)) {
      $q_result = $this-> {
        $api_name }
      -> $mthd ($cur_args);

      if ($q_result["error"]) {
        trigger_error('ARC: ' . $q_result["error"]);
      } else {
        //echo 'Success:<br /><pre>'.print_r($q_result["result"], 1).'</pre>';
      }
    } else {
      trigger_error("ARC: Undefined Operation");
    }
  }

  /**
   * @todo limit the export to a specific subgraph or model
   * */
  public function exportRDF($iri_model) {
    return $this->exportNative(true);
  }

  /**
   * Exports given data to rows.
   * */
  public function exportNative($rdf = false) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'DESCRIBE ?x ?y ?z
                                        WHERE 
                                        {
                                          ?x ?y ?z . 
                                        }';

    if ($rdf) {
      $q_result = $this->data_api->query(array (
        "query" => $query
      ));
    } else {
      $q_result = $this->data_api->query(array (
        "query" => $query,
        "result_type" => "rows"
      ));
    }

    if (empty ($q_result["result"]) && SEMANTIC_SEARCH_DEBUG) {
      // print_r($q_result);
    } else {
      return $q_result["result"];
    }
  }

  //--------------------------------------------------------------------------
  /**
   * Parse and check params set to create/ login to the store.
   * 
   *  //TODO following code should be in the import script
   *  // THese files should be moved to config and loaded only once.
   * @access private
   * @todo move all this to config
   *  TEMPORARY FUNCTION
   * 
   * 
   * //HACK
   * */
  //--------------------------------------------------------------------------  
  private function loadData() {
    //    $file1 = ARC_RDF_FILE_LOCATION . 'semantic_search_rdfs.rdf';
    //    $file2 = ARC_RDF_FILE_LOCATION . 'spacefinder_from_rss_2006_04_26.rdf';
    //    $file3 = ARC_RDF_FILE_LOCATION . 'spacefinder_from_rss_2006_04_26.rdfs';
    //
    //    $files = array ();
    //    $files[0]['iri'] = 'http://semantic_search..net/ns/semantic_search';
    //    $files[0]['file'] = $file1;
    //    $files[1]['iri'] = 'http://semantic_search..net/ns/car-spacefinder';
    //    $files[1]['file'] = $file2;
    //    $files[2]['iri'] = 'http://semantic_search..net/ns/car-spacefinder';
    //    $files[2]['file'] = $file3;
    //
    //    //loop and load files
    //    foreach ($files as $value) {
    //      importRDF($value['iri'], $value['file']);
    //    }

  }

  //--------------------------------------------------------------------------
  /**
   * Parse and check params set to create/ login to the store.
   * 
   * @param array $args an array of arguments needed for the store. 
   * eg. for mysql, db name, username, password etc 
   * 
   * */
  //--------------------------------------------------------------------------
  public function init($args) {
    //TODO this should be handled via a conf file, not a function
    $data_config = semantic_search_get_arc_config();
    $this->data_api = new ARC_api(array (
      "inc_path" => SONIA_ARC,
      "config" => $data_config,
      "log_path" => SONIA_ARC_LOG_PATH
    ));
    $this->data_api->db_connect();
  }

  /**
   * simple wrapper for ARC function
   * */
  public function removeTables($message_function_name = null) {
    $this->data_api->delete_store();
    $message = 'SEMANTIC_SEARCH: store REMOVED';
    $this->printMessage($message_function_name, $message);
  }

  public function truncateTables($message_function_name = null) {
    $this->data_api->reset_store();
    $message = 'SEMANTIC_SEARCH: store tables erased';
    $this->printMessage($message_function_name, $message);
  }
  //--------------------------------------------------------------------------
  /**
   * Installs the ARC backend
   * eg. used by Drupal installers
   * 
   * */
  //--------------------------------------------------------------------------
  public function installBackend($message_function_name = null) {

    if (!$this->data_api->store_exists()) {
      $this->data_api->create_store();
      //return the array of tablenames
      return array (
        SONIA_DB_PREFIX_NAME . 'store_var',
        SONIA_DB_PREFIX_NAME . 'id2val',
        SONIA_DB_PREFIX_NAME . 'triple',
        SONIA_DB_PREFIX_NAME . 'triple_dup',

        
      );
    } else {
      $message = 'SEMANTIC_SEARCH: ARC store was already installed.';
      $this->printMessage($message_function_name, $message);
    }
  }

  public function printMessage($message_function_name, $message) {
    if (($message_function_name != null) && (function_exists($message_function_name))) {
      $message_function_name ($message);
    }
  }
  
  /**
   * get data about an individual object
   * */
  private function getObjectData($object_uri){
    $sparql = ARC_DEFAULT_PREFIXES . "\n";
    $sparql .= 'SELECT DISTINCT ?p ?p_l ?o ?o_l ?s_l ?fname ?fpath
                                        WHERE { 
                                          "'.$object_uri.'" ?p ?o . 
                                          OPTIONAL {?p ' . SONIA_RDFS_LABEL . ' ?p_l }.
                                          OPTIONAL {?o ' . SONIA_RDFS_LABEL . ' ?o_l }.
                                          OPTIONAL {"'.$object_uri.'" ' . SONIA_RDFS_LABEL . ' ?s_l }. 
                                          OPTIONAL {?o <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_FILE_CLASS_URI . '> .  
                                                    ?o <' . SONIA_CREATE_FILENAME . '> ?fname . 
                                                    ?o <' . SONIA_CREATE_FILEPATH . '> ?fpath .       }   
                                        }';
                                        
    return $this->cachedQuery($sparql);
  }
  
  /**
   * get data about an individual object
   **/
  private function getObjectsOfParam($pro,$type = 'p_value'){
    if ($type == 'p_value'){
      $count_query = ARC_DEFAULT_PREFIXES;
      $count_query .= 'SELECT DISTINCT ?object WHERE {?object <' . $property . '> "' . $row['value'] . '" .}';
      $resulta = $this->cachedQuery($count_query);
    } else {
      $p = ' <'.$key.'> ';
      $o = ' ?o ';
    }
    $sparql = ARC_DEFAULT_PREFIXES . "\n";
    $sparql .= 'SELECT DISTINCT ?s  
                                        WHERE { 
                                          ?s '.$p.$o.'. 
                                          FILTER(REGEX(str(?s),"^'.SONIA_OBJECT_ID_URI.'")) .
                                        }';
                                        
    return $this->cachedQuery($sparql);
  }
  
  //RETRIEVAL
  //--------------------------------------------------------------------------
  /**
   * Parse the query and send a generalized query
   * In this case to sparql
   * 
   * for now, ?p and ?o get ignored for ?s searches and ?s is quite slow because it's
   * a filter match
   * 
   * @param array $query, the statement sent via SONIA
   * @todo for now just uses defaultns
   * @todo speed up the query of objects where an ID is given, no filter
   * @todo for now the subject based query only accepts URIs
   * 
   * @return string $convertedQuery 
   * @see SONIA :: UI_query()
   * @see query();
   * */
  //--------------------------------------------------------------------------
  public function convertQuery($query) {

    $user_id = $query['user_id'];
    $statement_sets = $query['statement_sets'];
    $limit = $query['limit'];
    $offset = $query['offset'];
    $any = $query['any'];
    $distinct = $query['distinct'];
    $describe = $query['describe'];

    //$sparql = ARC_DEFAULT_PREFIXES . "\n";
    $sparql .= 'SELECT DISTINCT ?s 
                  WHERE {
                    ?s ?p ?o .
                    FILTER(REGEX(str(?s),"^'.SONIA_OBJECT_ID_URI.'")) .
                    %s
                    %s
                  }  %s';
    $statement_blocks = '';
    //OPTIONAL {?o <'.SONIA_RDF_CREATE_TYPE.'> <'.SONIA_FILE_TYPE_URI.'> } as ?yes. // also title?
    $text_blocks = '';
    $matches = '';
    $text_block_count = 0;
    $order_by = '';

    //default used later for contructing 
    $text_block = '  %s %s ?x%s . FILTER(REGEX(str(?x%s), "%s")) . ' . "\n";
    $exact_block = '  %s %s "%s" . ' . "\n";
    $begin_block = '  %s %s ?x%s . FILTER(REGEX(str(?x%s), "^%s")) . ' . "\n";

    $statement_block = '  %s %s %s . ' . "\n";

    //TODO SLOW
    $subject_block = ' FILTER(REGEX(str(?s), "%s")) . ' . "\n";

    $lcount = 0;
    $vcount = 0;
    if (is_array($statement_sets)) {
      foreach ($statement_sets as $q) {
      	
      	$optional = (!empty($q['optional']));
      	
        if (!empty ($q['subject'])) {
          $s = '<' . $q['subject'] . '>';
          $p = ' ?p ';
          $o = ' ?o ';
        } else {
          $s = ' ?s ';

          if (!empty ($q['object'])) {
            $o = '<' . $q['object'] . '>';
            if ($q['literal_object']) {
              $lcount++;
              $o = $q['object'];
            }
          } else {
            $o = ' ?o' . $vcount . ' ';
            $vcount++;
          }

          if (!empty ($q['predicate'])) {
            $p = (isset ($q['category_query']) && ($q['category_query'])) ? '<' . SONIA_RDF_CREATE_TYPE . '>' : '<' . $q['predicate'] . '>  ';
          } else {
            $p = ' ?p' . $vcount . ' ';

            $vcount++;
          }

        } //end subject defined
        
        /// complete parsing of line
        if ($q['literal_object']) {

         if (empty($q['literal_type'])) $q['literal_type'] =    'exact';

          //check to see if there are non standard charactors, if so, change query type
          if ($q['literal_type'] == 'exact' && preg_match("/([\W]+)/", $q['object'], $match)) {
            $q['literal_type'] = 'begin';
          }

          switch ($q['literal_type']) {
          	
            case 'exact' :
               //AHIRC HACK
               if(strstr($p, SONIA_DEFAULT_MODEL_IRI .'/Property#field_ahirc_scope_0')){
                 
                 $result = strcasecmp('national',$q['object']);
                 
                 if ($result == 0) {
                    $sort = ' ASC';
                    $begin_block2 = '   %s %s ?scope .'; // FILTER(REGEX(str(?scope), "^(%s)")) . } ' . "\n";
                    $order_by =  $sort.'(?scope)'; // ASC (?s) 
                    $temp_block =  sprintf($begin_block2, $s, $p );//,'national') ;
                  } else {
                    $sort = ' DESC'; 
                    $begin_block2 = '  %s %s ?scope . FILTER(REGEX(str(?scope), "^(%s|%s)")) . ' . "\n";
                    $order_by = $sort.'(?scope)';// ASC (?s) 
                    $temp_block =  sprintf($begin_block2, $s, $p ,$q['object'],'national') ;
                 }
                 
               } else {
                 $temp_block =  sprintf($exact_block, $s, $p, $q['object']);
               }
            
              //$text_blocks .= sprintf($exact_block, $s, $p, $q['object']);
              break;
            case 'begin' :
              $temp_block =  sprintf($begin_block, $s, $p, $lcount, $lcount, $q['object']);
              break;
            case 'fulltext' :
            default :
              $temp_block = sprintf($text_block, $s, $p, $lcount, $lcount, $q['object']);
              break;
          }

        } else
          if (!empty ($q['subject'])) {
            //$subject_block
            $s = '' . $q['subject'] . '';
            $temp_block = sprintf($subject_block, $s);
            $s = '<' . $s . '>';
            $temp_block =  sprintf($statement_block, $s, $p, $o);
          } else {
            $temp_block =  sprintf($statement_block, $s, $p, $o);
          }
          
          $statement_blocks .= ($optional) ? ' OPTIONAL { '.$temp_block.' } ':$temp_block;
         
      }
    }
    
    //AHIRC HACK
   // $begin_block3 = ' OPTIONAL {  ?s <'.SONIA_DEFAULT_MODEL_IRI.'/Property#field_best_bet> ?bet . } ' . "\n";
	  $order_by = ' DESC(?bet) ' .$order_by; 
	//  $text_blocks .= sprintf($begin_block3 ) ;
	  $order_by = ' ORDER BY '.$order_by;
    
    //$sparql = sprintf($sparql, $statement_blocks, $text_blocks);
    $sparql = sprintf($sparql, $statement_blocks, $text_blocks,$order_by);
    //echo  '<hr><strong> kk' . nl2br(htmlentities($sparql)) . '</strong><hr>';
    $this->main_search_sparql = $sparql;
    return $sparql;
  }

  //--------------------------------------------------------------------------
  /**
   * Count the number of literals which have a give value
   * 
   * predicates and subjects may include the Namespace prefix or the entire
   * statement (in the case of SPARQL this should include "<" and ">"
   * 
   * @param string value - the literal value
   * @param string ns_predicate - the predicate (with the namespace)
   * @param string ns_subject - the subject (with the namespace)
   * @param array $args an array of arguments needed for the store. 
   * eg. for mysql, db name, username, password etc 
   * 
   * */
  //--------------------------------------------------------------------------

  public function getLiteralCount($ns_predicate, $ns_subject, $value) {

    $query = ARC_DEFAULT_PREFIXES;
    $query .= ' SELECT ?x 
                                       WHERE {
                                         ' . $ns_subject . ' ' . $ns_predicate . ' ?x .
                                          FILTER(REGEX(?x, "' . $value . '"))
                                       }
                                       ';
    $rows = $this->cachedQuery($query);
    return count($rows);
  }
  
  
  
  

  //--------------------------------------------------------------------------
  /**
   * Parse the query and send a generalized query to the store
   * 
   * @param string $ns_predicate use a ns from defaults like semantic_search:zip
   * @param string $ns_subject optional subject
   * 
   * @return array 'values' 'count'
   * 
   * */
  //--------------------------------------------------------------------------    
  public function getAllLiteralsAndCount($ns_predicate, $ns_subject) {
    if (empty ($ns_subject))
      $ns_subject = '?foo';

    $query = ARC_DEFAULT_PREFIXES;
    $query .= ' SELECT DISTINCT ?x 
                                       WHERE {
                                         ' . $ns_subject . ' ' . $ns_predicate . ' ?x .
                                       }
                                       ORDER BY ASC (?x)';

    $args = array (
      'query' => $query,
      'result_type' => ARC_DEFAULT_RESULT_TYPE,
        // "result_type_args"=>array("jsoni"=>$jsoni)

  
    );

    $q_result = $this->data_api->query($args);
    $results = $q_result["result"];
    $final_results = array ();
    $count = 0;

    if (is_array($results)) {
      foreach ($results as $result) {
        $final_results[$count]['value'] = $result['x'];
        $final_results[$count]['count'] = $this->getLiteralCount($ns_predicate, $ns_subject, $result['x']);
        $count++;
      }
    }

    return $final_results;
  }

  //--------------------------------------------------------------------------
  /**
   * takes a stores search results and parses it to sonia search results.
   * 
   * RESULTS:
   *     $new_record['o'] =  $object;
         $new_record['p'] =  $predicate;
         $new_record['p_l'] = $predicate_label;
         
         $search_result[$subject][SONIA_INSTANCE][$count]['t']   = $type;
         $search_result[$subject][SONIA_INSTANCE][$count]['t_l'] = $type_label;
         $search_result[$subject][$count] = $new_record;
   * 
   * @param array results from some format
   * @return array a standard SONIA results array
   * @todo shouldn't $subject be unique?
   * */
  //--------------------------------------------------------------------------
  public function parseStoreToSonia(&$results) {
    $search_result = array ();
    $count = 0;
    //parse the result into consistant form
    foreach ($results as $result) {
      $new_record = array ();

      $subject = $result['s'];
      $predicate = $result['p'];
      $object = $result['o'];

      $new_record['o'] = $object;
      if (isset ($result['o_l']))
        $new_record['o_l'] = $result['o_l'];
      $new_record['p'] = $predicate;
      if (isset ($result['p_l']))
        $new_record['p_l'] = $result['p_l'];
      if (isset ($result['s_l']))
        $new_record['s_l'] = $result['s_l'];
      if (!empty ($result['fname']))
        $new_record['fname'] = $result['fname'];
      if (!empty ($result['fpath']))
        $new_record['fpath'] = $result['fpath'];

      $search_result[$subject][$count] = $new_record;

      ++ $count;
    }

    return $this->cleanBadResults($search_result);
  }
  
  
  
  /**
   * encapsulation for caching a query
   * */
  public function cachedQuery($query, $return_type = 'rows') {
    $search_hash = 'searchHash' . md5($query.$return_type);
    global $sparql_execution_time;
    
    $rows = (SONIA_USE_CACHE) ? $this->cache->getCache($search_hash, true, true) : null;

    if (SONIA_USE_CACHE && $rows !== null) {
      return $rows;
    } else {
      $qr1 = $this->data_api->query(array (
        "query" => $query,
        "result_type" => $return_type
      ));
      
      $rows = $qr1["result"];
      $sparql_execution_time = $qr1["query_time"];
      
      if (SONIA_USE_CACHE){
        //save the result, if it is rows, then it is an array
        $this->cache->putCache($search_hash, $rows, true, ($return_type == 'rows'));
        //also cache the count
        if ($return_type == 'rows'){
          $this->addCachedCountQuery($query,$qr1["row_count"]);
        }
      }
      
      return $rows;
    }
  }

  /**
   * encapsulation for caching a query
   * */
  public function cachedCountQuery($query) {
    return $this->cachedQuery($query, 'row_count');
  }
  
  private function addCachedCountQuery($query,$count){
     $search_hash = 'searchHash' . md5($query.'row_count');
     $this->cache->putCache($search_hash, $count, true);
  }

  /**
   *  provide debugging output
   * */
  public function queryDebug($query, $print = true) {
    $args = array (
      'query' => $query,
      'result_type' => 'sql',
      
    );

    $q_result2 = $this->data_api->query($args);
    $this->debug_text .= '<hr>';
    //$this->debug_text .= nl2br(htmlentities(print_r($query, 1)));
    //$this->debug_text .= nl2br(htmlentities(print_r($q_result2, 1)));
    //$this->debug_text .= nl2br(htmlentities(print_r($this->data_api->get_errors(), 1)));
    //$this->debug_text .= '<hr>';
    $this->debug_text .= '<br> ' . $q_result2['result'] . '<br>';
    if ($print)
      dbg($this->debug_text, '$this->debug_text');
  }

  /**
   *  provide debugging output
   * */
  public function queryCountDebug($query) {
    $args = array (
      'query' => $query,
      'result_type' => 'row_count',

      
    );

    $q_result2 = $this->data_api->query($args);
    $this->debug_text .= '<hr>';
    $this->debug_text .= nl2br(htmlentities(print_r($query, 1)));
    $this->debug_text .= nl2br(htmlentities(print_r($q_result2, 1)));
    $this->debug_text .= nl2br(htmlentities(print_r($this->data_api->get_errors(), 1)));
    $this->debug_text .= '<hr>';
    $this->debug_text .= '<br> SQL ' . $q_result2['result'] . '<br>';
  }

  //--------------------------------------------------------------------------
  /**
   * Parse the query and send a generalized query to the store
   * 
   * @see convertQuery();
   * @see SONIA :: UI_query()
   * @see queryDebug()
   * 
   * @return array results and result codes to be used by SONIA
   * 
   * */
  //--------------------------------------------------------------------------
  public function query($query, $return_type = 'array', $convert_query = true) {
    //set default offset
    $_semantic_search_pagination_offset = (empty($_semantic_search_pagination_offset)) ? 0 : $_semantic_search_pagination_offset;
    $search_result = array ();
    if ($convert_query)
      $query = $this->convertQuery($query);
    //$this->queryDebug($query);
     
    //now I have results and count 
    $results = $this->cachedQuery($query);
    $this->object_search_result_ids = array();
    if (is_array($results)){
      foreach($results as $data){
        $this->object_search_result_ids[] = $data['s'];
      }
    }
    
    $this->object_search_result_count = count($this->object_search_result_ids);
    $index = 0;
    $count_completed = 0;
    $offset = SONIA_IO :: getPaginationBegin();
    $limit = SEMANTIC_SEARCH_DISPLAY_PAGINATION;
    $search_result = array();
    
    
    if(!empty($this->object_search_result_ids) && is_array($this->object_search_result_ids)){
      foreach($this->object_search_result_ids as $object_id){
        if ($index >= $offset && $count_completed < SEMANTIC_SEARCH_DISPLAY_PAGINATION ){
          $search_result[$object_id]  = $this->getObjectData($object_id);
          ++$count_completed;
        }
        ++$index;
       
        if ($count_completed >= SEMANTIC_SEARCH_DISPLAY_PAGINATION) break;
      }
    }
    return $search_result;
  }

  //--------------------------------------------------------------------------
  /**
   * TODO DEPRICATE
   * setup to clean results that may be undesirable
   * */
  //--------------------------------------------------------------------------
  public function cleanBadResults($search_result) {
    //TODO HACK
    //remove bad incomplete data
    foreach ($search_result as $key => $result) {
      //HACK fix for "bad" results where we would expect more than 3 fields of data
      if (count($result) < 2)
        unset ($search_result[$key]);
    }
    return $search_result;
  }

  //--------------------------------------------------------------------------
  /**
   * get all the classes which a given object belongs to
   * */
  //--------------------------------------------------------------------------
  public function getAllClassesOfObject($object_uri) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= '
                                    SELECT DISTINCT ?sc ?sc_label
                                      WHERE {
                                        <' . $object_uri . '> <' . SONIA_RDF_CREATE_TYPE . '> ?sc .
                                         ?sc   rdfs:label ?sc_label  .             
                                      }
                                  ';
    $rows = $this->cachedQuery($query);
    $row_count = count($rows);

  }

  /**
   * get all classes and their subclasses
   * @param boolean $root_given was the start class properly provided as a URI
   * 
   * @todo rename
   * */
  public function getAllCategoriesRecursive($startclass = null, $depth = 0, $startclass_label = '') {
    $query = ARC_DEFAULT_PREFIXES;

    if ($depth > 0 || ($startclass != null)) //get derived classes
      {
      //<'.$startclass.'> rdfs:label ?label .
      $query .= '
                                              SELECT DISTINCT ?sc ?sc_label 
                                                WHERE {
                                                  ?sc rdfs:subClassOf <' . $startclass . '> .
                                                  ?sc a rdfs:Class .    
                                                  ?sc rdfs:label ?sc_label  
                                       
                                                }
                                            ';

    } else //get root classes which are not subclasses of anything
      {
      $query .= '
                                              SELECT DISTINCT ?label  ?x ?y
                                                WHERE {
                                                  ?x a rdfs:Class .
                                                  ?x rdfs:label ?label .
                                                  OPTIONAL 
                                                  {
                                                    ?x  rdfs:subClassOf ?y .
                                                  }            
                                                }
                                            ';
    }

    $rows = $this->cachedQuery($query);
    $row_count = count($rows);
    $final_array = array ();
    $count = 0;

    if (($row_count > 0) && ($startclass != null)) {
      $final_array[$startclass]['label'] = $startclass_label;

      for ($i = 0; $i < $row_count; $i++) {
        if (isset ($rows[$i])) {
          $row = $rows[$i];

          $subclass = $row["sc"];
          //$label       = $row["label"];
          $sc_label = $row["sc_label"];

          $subclasses = $this->getAllCategoriesRecursive($subclass, $depth +1, $sc_label);

          if (!empty ($subclasses))
            $final_array[$startclass]['subclasses'][] = $subclasses;
        }
      }
    } else
      if ($row_count > 0) //parse root classes
        {
        for ($i = 0; $i < $row_count; $i++) {
          //must not be a subclass of anything else
          $row = (!empty ($rows[$i])) ? $rows[$i] : null;

          //echo '<br />$row["x"] '.$row["x"];

          if (empty ($row["y"])) {
            $_startclass = (isset ($row["x"])) ? $row["x"] : '';
            $label = (isset ($row["label"])) ? $row["label"] : ''; //$row["label"];
            $final_array[] = $this->getAllCategoriesRecursive($_startclass, $depth +1, $label);
          }
        }
      }
    //       //
    return $final_array;
  }

  //--------------------------------------------------------------------------
  /**
  * gets all the categories/classes and counts available for a search
  * 
  * @todo only works for 3 levels deep - should be a recursive function
  * @author 
  * @param 
  * */
  //--------------------------------------------------------------------------
  public function getAllCategories(& $count, $startclass = 'semantic_search:Space', $in = null) {
    //?x_label
    $query = ARC_DEFAULT_PREFIXES;
    $query .= '
                                  SELECT DISTINCT   ?c ?c_label ?sc ?sc_label ?x_label
                                    WHERE {
                                      ?sc rdfs:subClassOf ' . $startclass . ' .
                                      ' . $startclass . ' rdfs:label ?x_label .
                                      ?c a rdfs:Class .                
                                      ?c rdfs:subClassOf ?sc .
                                      ?c rdfs:label ?c_label .
                                      ?sc rdfs:label ?sc_label 
                                    }
                                ';

    $rows = $this->cachedQuery($query);
    $row_count = count($rows);

    $final_array = array ();

    $count = 0;

    if ($row_count > 0) {
      for ($i = 0, $i_max = $row_count; $i < min(20, $i_max); $i++) {
        $row = $rows[$i];
        $c_iri = $row["c"];
        // $id         = $this->parse_id_from_uri($c_iri);
        $c_label = $row["c_label"];
        $sc_iri = $row["sc"];
        $sc_label = $row["sc_label"];

        //HACK build hierarchy
        $final_array[$startclass]['name'] = $row['x_label'];
        $final_array[$startclass]['subclasses'][$sc_iri]['name'] = $sc_label;
        $final_array[$startclass]['subclasses'][$sc_iri]['subclasses'][$c_iri]['name'] = $c_label;

        //build classes
        //$final_array[$sc_iri]['name'] = $sc_label;
        //$final_array[$c_iri]['name'] = $c_label;
        $count++;
      }
    }
    return $final_array;
  }

  //--------------------------------------------------------------------------
  /**
  * add a property to the definistion of a class.
  * Data on specific objects are preserved
  * 
  * @param string class - the class being defined
  * @param string property - the property which belongs to the class
  * */
  //--------------------------------------------------------------------------   
  public function addPropertyToClass($class, $property) {
    return $this->createStatement($property, SONIA_RDF_CREATE_PROPERTY_OF, $class, false);
  }

  //--------------------------------------------------------------------------
  /**
  * add a property to the definistion of a class.
  * Data on specific objects are preserved
  * 
  * @todo check for duplicates
  * @param string class - the class being defined
  * @param string property - the property which belongs to the class
  * */
  //--------------------------------------------------------------------------   
  public function createProperty($property_name, $classes = null, $id = null) {
    if ($id === null) {
      //create ID //
      $id = $this->generatePropertyAutoIncrementField();
      $id_iri = SONIA_PROPERTY_ID_URI . $id;
    } else {
      $id_iri = $id;
    }

    //TODO check that it doesn' exist
    //create property
    //TODO not needed if duplicate
    $this->createStatement($id_iri, SONIA_RDF_CREATE_TYPE, SONIA_RDF_CREATE_PROPERTY_TYPE);

    //create label
    $this->createStatement($id_iri, SONIA_RDFS_CREATE_LABEL, $property_name, true);

    if (is_array($classes) && !empty ($classes)) {
      foreach ($classes as $class) {
        $this->addPropertyToClass($class, $id_iri);
      }
    }

    return $id_iri;
  }

  /**
   * generate an ID for a class itself
   * 
   * @see generateAutoIncrementField()
   * */
  public function generateClassAutoIncrementField() {
    return $this->generateAutoIncrementField(SONIA_CREATE_MAX_ID_CLASS_TYPE);
  }

  //--------------------------------------------------------------------------
  /**
  * generates an ID for a class being created and increments
  * 
  * @todo not ACID/ transactionally safe
  * @todo what if the object belongs to more than one class
  * @param string class MUST BE FULL URI
  * */
  //--------------------------------------------------------------------------
  public function generateAutoIncrementField($class) {
    $predicate = SONIA_AUTO_ID_PREFIX;
    $query = ARC_DEFAULT_PREFIXES;
    //get if exists
    $query .= 'SELECT ?s  
                                    WHERE {
                                      <' . $class . '> <' . SONIA_CREATE_MAX_ID_PROPERTY . '> ?s .
                                    }';
    //echo nl2br(htmlentities($query)).'<br>';      
    $qr1 = $this->data_api->query(array (
      "query" => $query,
      "result_type" => "rows"
    ));
    $rows = $qr1["result"];
    //$row_count       = () ?$qr1["row_count"] : '';
    $temp_facet_hiercharchy = array ();

    //if it does exist
    if (isset ($rows[0]['s'])) {
      $old_id = $rows[0]['s'];
      $id = $old_id +1;

      // echo "<br>update statement \$id $id \$old_id $old_id<br>";

      $this->updateStatement($class, SONIA_CREATE_MAX_ID_PROPERTY, null, $class, SONIA_CREATE_MAX_ID_PROPERTY, $id, true);

    } else {
      $id = 1;
      $this->createStatement($class, SONIA_CREATE_MAX_ID_PROPERTY, $id, true, 'int');
    }
    return SONIA_AUTO_ID_PREFIX . $id;
  }

  /**
   * get a list of properties (and their values) based on teh alphabetical 
   * index of their label
   * */
  public function getAllClassPropertiesRange(& $count, $startclass, $label_begin = null, $label_end = null) {
    //$text_block       = ' . ?s %s ?x%s . FILTER(REGEX(str(?x%s), "%s")) '."\n";
    $startclass_clause = '';
    if ($startclass != null)
      $startclass_clause = '?s ' . SONIA_RDF_PROPERTY_OF . ' <' . $startclass . '> .';
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?s ?little_literal
                                    WHERE {
                                      ?s a ' . SONIA_RDF_PROPERTY_TYPE . ' .
                                      ' . $startclass_clause . '
                                      ?s ' . SONIA_RDFS_LABEL . ' ?little_literal .';
    if ($label_begin != null && $label_end != null) {
      $regex = '^(' . $label_begin . ').*'; ///^(batty).*/i
      $query .= ' FILTER(REGEX(str(?little_literal), "' . $regex . '")) .';
    }
    $query .= '         }';
    $rows = $this->cachedQuery($query);
    $row_count = count($rows);
    $temp_facet_hiercharchy = array ();

    //parse into a tree
    foreach ($rows as $result) {
      $subject = $result['s'];
      $name = $result['little_literal'];

      $temp_facet_hiercharchy[$subject] = $name;
    }

    $count = count($temp_facet_hiercharchy);

    return $temp_facet_hiercharchy;

  }

  //--------------------------------------------------------------------------
  /**
  * gets all the categories/classes and counts available for a search
  * 
  * @param string in the list of subjects
  * */
  //--------------------------------------------------------------------------
  public function getAllProperties(& $count) {
    return $this->getAllClassProperties($count, null);
  }

  //--------------------------------------------------------------------------
  /**
  * gets all classes per properties
  * 
  * @todo does not handle inference! or subclasses
  * @param string $PROPERTY $URI
  * */
  //--------------------------------------------------------------------------
  public function getAllClassesOfProperty($property) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?s ?little_literal
                                    WHERE {
                                      <' . $property . '> a ' . SONIA_RDF_PROPERTY_TYPE . ' .
                                      ?s ' . SONIA_RDFS_LABEL . ' ?little_literal .
                                      ?s <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_RDFS_CREATE_CLASS_TYPE . '> .
                                      <' . $property . '> <' . SONIA_RDF_CREATE_PROPERTY_OF . '> ?s .
                                    }';

    $rows = $this->cachedQuery($query);
    $row_count = count($rows);
    $temp_facet_hiercharchy = array ();

    if (empty ($rows))
      return array ();
    //parse into a tree
    foreach ($rows as $result) {
      $subject = $result['s'];
      $name = $result['little_literal'];

      $temp_facet_hiercharchy[$subject]['name'] = $name;
    }

    $count = count($temp_facet_hiercharchy);
    $new_array = array ();

    $new_array[SONIA_RDFS_CREATE_CLASS_TYPE]['subclasses'] = $temp_facet_hiercharchy;
    $new_array[SONIA_RDFS_CREATE_CLASS_TYPE]['name'] = 'Classes';
    return $new_array;
  }

  //-------------------------------------------------------------------------- 
  /**
   * @see getAllClassProperties()
   * */
  //-------------------------------------------------------------------------- 
  public function getAllClassPropertiesMultiple(& $count, $startclasses) {
    $array = array ();
    if (is_array($startclasses)) {
      foreach ($startclasses as $startclass) {
        $array = array_merge_recursive($array, $this->getAllClassProperties($count, $startclass));
        //array_merge_recursive(
      }
    }
    return $array;
  }

  //--------------------------------------------------------------------------
  /**
  * gets all properties per class
  * 
  * @todo does not handle inference!
  * @param string in the list of subjects
  * */
  //--------------------------------------------------------------------------
  public function getAllClassProperties(& $count, $startclass) {
    $startclass_clause = '';
    if ($startclass != null)
      $startclass_clause = '?s ' . SONIA_RDF_PROPERTY_OF . ' <' . $startclass . '> .';
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?s ?little_literal
                                    WHERE {
                                      ?s a ' . SONIA_RDF_PROPERTY_TYPE . ' .
                                      ' . $startclass_clause . '
                                      ?s ' . SONIA_RDFS_LABEL . ' ?little_literal .
                                    }';
    $rows = $this->cachedQuery($query);
    $row_count = count($rows);
    $temp_facet_hiercharchy = array ();

    //parse into a tree

    if (is_array($rows)) {
      foreach ($rows as $result) {
        $subject = $result['s'];
        $name = $result['little_literal'];

        $temp_facet_hiercharchy[$subject]['name'] = $name;
      }
    }

    $count = count($temp_facet_hiercharchy);
    $new_array = array ();

    $new_array[SONIA_RDF_PROPERTY_OF]['subclasses'] = $temp_facet_hiercharchy;
    $new_array[SONIA_RDF_PROPERTY_OF]['name'] = 'Properties';
    return $new_array;
  }

  //-------------------------------------------------------------------------- 
  /**
   * get File info
   * */
  //-------------------------------------------------------------------------- 
  public function getFileInfo($image_uri) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?p ?o 
                                    WHERE {
                                      <' . $image_uri . '> ?p ?o .
                                    }';
    $rows = $this->cachedQuery($query);
    //$row_count       = count($rows);
    $count = 0;
    $return_object = array ();

    if (isset ($rows[0])) {
      foreach ($rows as $row) {
        $return_object[$count]['o'] = $row['o'];
        $return_object[$count]['p'] = $row['p'];
        $count++;
      }
    }

    return $return_object;
  }

  //-------------------------------------------------------------------------- 
  /**
   * get File info
   * */
  //-------------------------------------------------------------------------- 
  public function getFileInfoByFilename($filename, $iri_only = false) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?s ?p ?o 
                                    WHERE {
                                      ?s ?p ?o .
                                      ?s ?x "' . $filename . '" .
                                    }';
    $rows = $this->cachedQuery($query);
    //$row_count       = count($rows);
    $count = 0;
    $return_object = array ();

    if (isset ($rows[0])) {
      if ($iri_only)
        return $rows[0]['s'];
      foreach ($rows as $row) {
        $return_object[$count]['o'] = $row['o'];
        $return_object[$count]['p'] = $row['p'];
        $count++;
      }
    }

    return $return_object;
  }

  //-------------------------------------------------------------------------- 
  /**
   * gets an objects labels
   * assumes there is only one
   * */
  //-------------------------------------------------------------------------- 
  public function getLabel($object) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?little_literal 
                                    WHERE {
                                      <' . $object . '> ' . SONIA_RDFS_LABEL . ' ?little_literal .
                                    }';
    $rows = $this->cachedQuery($query);
    //$row_count       = count($rows);
    if (isset ($rows[0]))
      return $rows[0]['little_literal'];
  }

  //-------------------------------------------------------------------------- 
  /**
   * gets URI by lable
   * assumes there is only one
   * */
  //-------------------------------------------------------------------------- 
  public function getURIByLabel($label) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?uri 
                                    WHERE {
                                      ?uri ' . SONIA_RDFS_LABEL . ' "' . $label . '" .
                                    }';
    $rows = $this->cachedQuery($query);
    //$row_count       = count($rows);
    if (isset ($rows[0]))
      return $rows[0]['uri'];
  }

  //-------------------------------------------------------------------------- 
  /**
   * assumed cardinality of 0,1
   * */
  //-------------------------------------------------------------------------- 
  public function getComment($uri) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?little_literal 
                                    WHERE {
                                      <' . $uri . '> ' . SONIA_RDFS_COMMENT . ' ?little_literal .
                                    }';
    $rows = $this->cachedQuery($query);
    //$row_count       = count($rows);

    if (isset ($rows[0]))
      return $rows[0]['little_literal'];
  }

  //-------------------------------------------------------------------------- 
  /**
   * get's an instantiated objects "categories"
   * */
  //-------------------------------------------------------------------------- 
  public function getClasses($object) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?o 
                                    WHERE {
                                      <' . $object . '> ' . SONIA_RDF_PROPERTY_OF . ' ?o .
                                    }';

    return $this->cachedQuery($query);
  }

  /**
   * retrieve Classes,
   * their labels, 
   * */
  public function getClassesAndCounts() {
    //get class
    //get label
    //get object counts
    $results = array ();
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?class ?class_label ?object
                                    WHERE {
                                      ?class <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_RDFS_CREATE_CLASS_TYPE . '> .
                                      ?class ' . SONIA_RDFS_LABEL . ' ?class_label
                                      ?object <' . SONIA_RDF_CREATE_TYPE . '> ?class . 
                                    }';

    $rows = $this->cachedQuery($query);
    
    
    
    if (is_array($rows)) {
      foreach ($rows as $row) {
        $results[$row['class']]['label'] = $row['class_label'];
        $results[$row['class']]['objects'][] = $row['object'];

        if (!isset ($results[$row['class']]['count'])) {
          $results[$row['class']]['count'] = 1;
        } else {
          $results[$row['class']]['count'] = $results[$row['class']]['count'] + 1;
        }
      }
    }

    return $results;
  }

  /**
   * given a start class retrieve two layers of children eg
   * startclass = animals
   * layer1 = marsupials
   * layer2 = kangaroos
   * 
   * assumes that usually there aren't many classes and no circular references
   * */
  public function getTelescopingClassesAndCountsHierarchy( $startclass, $original_startclass, $with_objects = false, $climb = false) {
    if (empty ($startclass) && $startclass != '22')
      return;
    if (empty ($this->classes_and_counts_hiearararchy) || $startclass == '22'){
    	$nullarray = array();
      $this->classes_and_counts_hiearararchy = $this->getClassesAndCountsHierarchy($nullarray);
      $this->classes_and_counts_hiearararchy_counts = array();
    }
    
    $layer1 = array ();
    $classes = $this->classes_and_counts_hiearararchy;

   
    foreach ($classes as $class_uri => $class) {
      if (empty( $class['parents']))  $class['parents'] = array();
      if (in_array($startclass, $class['parents'])) {
        if (!$with_objects)
          unset ($classes[$class_uri]['objects']);
        $layer1[$class_uri] = $classes[$class_uri];
        $this->classes_and_counts_hiearararchy_counts[$class_uri]  = $classes[$class_uri]['count'];
        foreach ($classes as $class_uri2 => $class2) {
          if (empty( $class2['parents']))  $class2['parents'] = array();
          if (in_array($class_uri, $class2['parents'])) {
            $layer1[$class_uri]['children'][$class_uri2] = $class2;
            //indicator
            $layer1[$class_uri]['#level_two'] = true;
            if (!$with_objects)
              unset ($layer1[$class_uri]['children'][$class_uri2]['objects']);
          }
        }
      }
    }
    
    if ($climb && empty ($layer1)) {
      return $this->getTelescopingClassesAndCountsHierarchy( $original_startclass, null, $with_objects);
    }
    
    $final_array = array();
    //unset ($classes[$startclass]['objects']);
    $final_array[$startclass] = $classes[$startclass];
    $final_array[$startclass]['children'] = $layer1;
   
    return $final_array;
  }

  /**
   * retrieve Classes,
   * their labels, related objects, and potential parents
   * @param array gives you unique object and number of times they show
   * @return all classes, objects and parents 
   * */
  public function getClassesAndCountsHierarchy(& $unique_objects, $with_count = true) {
    //get class
    //get label
    if (!isset ($unique_objects))
      $unique_objects = array ();
    //get object counts
    $results = array ();
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?class ?class_label ?object ?parent ?description
                                    WHERE {
                                      ?class <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_RDFS_CREATE_CLASS_TYPE . '> .
                                      OPTIONAL {?class ' . SONIA_RDFS_LABEL . ' ?class_label} . 
                                      OPTIONAL {?class rdfs:subClassOf ?parent} .  
                                      OPTIONAL {?class  <' . SONIA_RDFS_CREATE_COMMENT . '> ?description}.  
                                    }';
    

    $rows = $this->cachedQuery($query);
    $unique_counts = array ();
    //dbg($this->debug_text);
    //$this->queryCountDebug($query) ;
     //dbg( $rows,'$rows');
    //?object <' . SONIA_RDF_CREATE_TYPE . '> ?class .

    if (is_array($rows)) {
      foreach ($rows as $row) {
        $results[$row['class']]['label'] = $row['class_label'];
        if (isset ($row['description']))
          $results[$row['class']]['description'] = $row['description'];
        if (!empty($row['parent'])) $results[$row['class']]['parents'][$row['parent']] = $row['parent'];
        if ($with_count && !isset($results[$row['class']]['count'])) $results[$row['class']]['count'] = $this->getObjectMatchesCount( $row['class'],SONIA_RDF_CREATE_TYPE, false, false);
      }
    }

    ///merge classes for for getting results
    return $results;
  }



  public function getPropertiesAndCounts($class = null, $with_objects = true) {
    $class_limit = null;
    //todo optionally get class
    if ($class != null) {
      $class_limit = '?object <' . SONIA_RDF_CREATE_TYPE . '> <' . $class . '> .';
    }
    
    $get_object =  ($with_objects)  ? ' ?object ' :''; 
    //get class
    //get label
    //get object counts
    $results = array ();
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?property ?property_label ?value '.$get_object.'
                                    WHERE {
                                      ?property <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_RDF_CREATE_PROPERTY_TYPE . '> .
                                      ?property ' . SONIA_RDFS_LABEL . ' ?property_label
                                      OPTIONAL { ?object ?property ?value . }
                                      ' . $class_limit . '
                                    }';

    $rows = $this->cachedQuery($query); 
    
   

    if (is_array($rows)) {
      foreach ($rows as $row) {
        if (!isset( $results[$row['property']]['count'])) $results[$row['property']]['count'] = 0;
        $results[$row['property']]['label'] = $row['property_label'];
        if ($with_objects) $results[$row['property']]['objects'][] = $row['object'];
        if ($with_objects && isset($row['object'])) {
          $results[$row['property']]['count'] = $results[$row['property']]['count'] + 1;
        }
      }
    }

    return $results;
  }

  public function getPropertyValuesAndCounts($property = null, $class = null) {
    $class_limit = null;
    //todo optionally get class
    if ($class !== null) {
      $class_limit = '?object <' . SONIA_RDF_CREATE_TYPE . '> <' . $class . '> .';
    }
    
    if ($property !== null){
     $pv2  = $pv = '<' . $property . '>';
     $pl = null;
     $normal = true;
    } else {
      $pv2 = $pl = ' ?property ';
      $pv =  '?property <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_RDF_CREATE_PROPERTY_TYPE . '> .    ?property '; 
      $normal = false;
    }
    
    
    $results = array ();
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?value ?property_label '.$pl.' 
                            WHERE {
                              ' . $pv . ' <' . SONIA_RDFS_CREATE_LABEL . '> ?property_label .  
                               ?object ' . $pv2 . ' ?value .
                              ' . $class_limit . '
                            }';

    $rows = $this->cachedQuery($query);
    
    
    
 //  echo  '<hr><strong> ' . nl2br(htmlentities($query)) . '</strong><hr>';
   // echo  '<hr><strong> ' . nl2br(htmlentities($rows)) . '</strong><hr>';
    
    //dbg(htmlentities($query));
    if (is_array($rows)) {
      foreach ($rows as $row) {
        $hash = trim(substr($row['value'], 0, SONIA_MAX_VALUE_SIZE_AS_KEY));
        $results[$hash]['label'] = trim($row['value']);
        $prop = ($normal)? $property: $row['property'];
        if(!isset($results[$hash]['count'])) $results[$hash]['count'] =  $this->getObjectMatchesCount( $row['value'],$prop);
        if (!$normal) {
           $results[$hash]['property'] = $row['property'];
           $results[$hash]['property_label'] = $row['property_label'];
        }
      }
    }
    return $results;
  }
  
  /**
   * helper function to retrieve a count of objects given an aribtrary facet
   * */
  public function getObjectMatchesCount($object,$property=null, $strict=false, $literal=true){
    $object =  ($literal) ?' "' . $object  . '" ':' <' . $object  . '> ';
    $_strict = ($strict) ? ' FILTER(REGEX(str(?s),"^'.SONIA_OBJECT_ID_URI.'")) .' :'';
    $property = ($property !== null) ? '<' . $property . '>': ' ?p ';
    $count_query = ARC_DEFAULT_PREFIXES;
    $count_query .= 'SELECT DISTINCT ?s WHERE {?s  ' . $property . ' '.$object.'. '.$_strict.'}';
    return $this->cachedCountQuery($count_query);
  }
  
  /**
   * helper function to retrieve a count of objects given an aribtrary facet
   * within the search
   * */
  public function getObjectsCountInSearch($object,$property=null, $strict=false, $literal=true){
    $query_addition = null;
    if ($object!==null){
      $object =  ($literal) ?' "' . $object  . '" ':' <' . $object  . '> ';
    } else {
      $object = ' ?object ';
    }
    
    $property = ($property !== null) ? '<' . $property . '>': ' ?p ';
    $query_addition = ' ?s  ' . $property . ' '.$object.'. ';
    
    
   
    
    $count_query = $this->main_search_sparql;
    $count_query = str_replace('?s ?p ?o .','?s ?p ?o . '."\n  ".$query_addition,$count_query) ;
    //doesn't like strict here
    $count_query = str_replace('FILTER(REGEX(str(?s),"^'.SONIA_OBJECT_ID_URI.'")) .','',$count_query);
    
    return  $this->cachedCountQuery($count_query);
  }
  
  //-------------------------------------------------------------------------- 
  /**
   * Update properties by changing their names and which classes they are assigned to
   * 
   * @todo a major issue with inheretance and other assoications
   * @todo should likely be moved to SONIA or one level up - doesn't need to be part of the store API
   * */
  //-------------------------------------------------------------------------- 
  public function updateProperty($property, $property_name, $classes) {
    if ($property == '')
      return;

    //perhaps these should be create statements
    //update label
    $this->updateStatement($property, SONIA_RDFS_CREATE_LABEL, null, $property, SONIA_RDFS_CREATE_LABEL, $property_name, true);

    foreach ($classes as $class) {
      //prevents duplicate statements 
      $this->updateStatement($property, SONIA_RDF_CREATE_PROPERTY_OF, $class, $property, SONIA_RDF_CREATE_PROPERTY_OF, $class);
    }
  }

  //-------------------------------------------------------------------------- 
  /**
  * get a namespace id for a namespace
  * 
  * 
  * @param string ns the namespace
  * @return int nid the primary key of the name space
  * 
  * @todo owner, group
  * 
  * 
  * */
  //--------------------------------------------------------------------------
  public function getNSID($ns) {

  }

  //--------------------------------------------------------------------------
  /**
   * 
   * Retrieves the number of objects in the category
   * 
   * USAGE:
   * 
   * @todo make IRIs constants or member variables instead of hard coded
   * @param string $category_name_or_id name or id of the start category 
   * 
   * @return int count
   * 
   * @see query()
   * */
  //--------------------------------------------------------------------------
  public function getCategoryCount($category_name_or_id) {
    $category_name_or_id = trim($category_name_or_id);

    //TODO check for validity
    $query = ARC_DEFAULT_PREFIXES;
    $query .= ' SELECT DISTINCT ?s
                                    WHERE {
                                      ?s a <' . $category_name_or_id . '> .
                                    }';

    $qr2 = $this->data_api->query(array (
      "query" => $query,
      "result_type" => "row_count"
    ));

    return (isset ($qr2["result"])) ? $qr2["result"] : 0;
  }

  //--------------------------------------------------------------------------
  /**
   * 
   * Retrieves the number of objects having a facet 
   * 
   * USAGE:
   * 
   * @param string $property_name_or_id name or idof the property 
   * 
   * @return int count
   * 
   * @see query()
   * */
  //--------------------------------------------------------------------------
  public function getPropertyCount($property_name_or_id) {
    $property_name_or_id = trim($property_name_or_id);

    //TODO check for validity
    $query = ARC_DEFAULT_PREFIXES;
    $query .= ' SELECT ?s
                                    WHERE {
                                      ?s  <' . $property_name_or_id . '> ?y.
                                    }';

    $qr2 = $this->data_api->query(array (
      "query" => $query,
      "result_type" => "row_count"
    ));
    return (isset ($qr2["result"])) ? $qr2["result"] : 0;
  }

  //DATA CREATION

  //--------------------------------------------------------------------------
  /**
  @param
  @return int sid the sid just created
  @todo add user and group permissions management
  @todo add model concept
  @todo add lang
  */
  //--------------------------------------------------------------------------
  function createStatement($subject, $predicate, $object, $object_literal = false, $dataType = 'string', $owner = null, $group = null, $default_ns = null) {
    $args = array ();
    $args['result_type'] = 'array';
    //if (strstr($object,'"')) str_replace('"','\'',$object);
    if (strstr($object,'"')){ 
         
    } else {
      if ($object_literal) {

        $args['add_triple'] = ' <' . $subject . '> <' . $predicate . '>  "' . $object . '" .';
      } else {
          if (strstr($object,' '))dbg($object,'NOTICE:  url conatains a space  '); 
          $args['add_triple'] = ' <' . $subject . '> <' . $predicate . '>  <' . $object . '> .';
      }  
    }
  
      if (!isset ($args['add_triple'])) {
      return;
    }

    $args['graph_iri'] = ARC_MAIN_DATA_IRI;
    $result = $this->data_api->add_data($args);

    // echo '<br/>'.'trying to add '.htmlentities($args['add_triple']).'<br/>';
    if (!empty ($result['error'])) {
      $statement = htmlentities($args['add_triple']);
      echo "<br />$statement <br /> Store error createStatement() \$object_literal = $object_literal <br/>";
      dbg($result);
    }
  }

  function updateStatement($subject, $predicate, $object, $new_subject, $new_predicate, $new_object, $object_literal = false, $dataType = 'string', $owner = null, $group = null, $default_ns = null) {
    $args = array ();
    $args['result_type'] = 'array';

    if ($subject != null) {
      $args['del_s'] = $subject;
    }

    if ($predicate != null) {
      $args['del_p'] = $predicate;
    }

    if ($object != null) {
      $args['del_o'] = $object;
    }

    //safety checks
    if (!isset ($args['del_s']) && !isset ($args['del_p']) && !isset ($args['del_o'])) {
      return;
    }

    if ($object_literal && $new_object != '') {
      $args['add_triple'] = '<' . $new_subject . '> <' . $new_predicate . '>  "' . $new_object . '" .';
    } else {
      $args['add_triple'] = '<' . $new_subject . '> <' . $new_predicate . '>  <' . $new_object . '> .';
    }

    if (!isset ($args['add_triple']))
      //warn user to use delete
      {
      die(!isset ($args['add_triple']));
      return;
    }

    $args['graph_iri'] = ARC_MAIN_DATA_IRI;
    $result = $this->data_api->update_data($args);

  }

  /**
   * 
   * 
   * */
  function deleteStatement($subject, $predicate, $object, $object_literal = false, $dataType = 'string', $owner = null, $group = null, $default_ns = null) {

    $args = array ();

    if ($subject != null) {
      $args['del_s'] = $subject;
    }

    if ($predicate != null) {
      $args['del_p'] = $predicate;
    }

    if ($object != null) {
      $args['del_o'] = $object;
    }

    //safety checks
    if (!isset ($args['del_s']) && !isset ($args['del_p']) && !isset ($args['del_o'])) {
      return;
    }
    $args['graph_iri'] = ARC_MAIN_DATA_IRI;
    $result = $this->data_api->delete_data($args);
  }

  //--------------------------------------------------------------------------
  /**
   * 
   * Creates an RDFS:Class
   * 
   * USAGE:
   * 
   * @param string class name is the name of the class
   * 
   * @return void
   * 
   * */
  //--------------------------------------------------------------------------
  public function createClass($classname, $superclasses, $id = null) {
    if ($id == null) {
      //create ID 
      //TODO check that it doesn' exist
      $id = $this->generateClassAutoIncrementField();
      $id_iri = SONIA_CLASS_ID_URI . $id;
    } else {
      //assume the id is a full IRI
      $id_iri = $id;
    }

    //todo make root class
    if (empty ($superclasses)) {
      //todo make this the root class
      $superclasses[] = SONIA_RDFS_CREATE_CLASS_TYPE;
    }

    //create property
    //TODO not needed if duplicate
    $this->createStatement($id_iri, SONIA_RDF_CREATE_TYPE, SONIA_RDFS_CREATE_CLASS_TYPE);

    //create label
    $this->createStatement($id_iri, SONIA_RDFS_CREATE_LABEL, $classname, true);

    foreach ($superclasses as $superclass) {
      $this->createStatement($id_iri, SONIA_RDFS_CREATE_SUBCLASS, $superclass);
    }

    return $id_iri;
  }

  //--------------------------------------------------------------------------
  /**
   * 
   * sets one RDFS:Class to be a subclass of another
   * 
   * USAGE:
   * 
   * @param string URI of the class
   * @param mixed URI of the superclasses
   * 
   * @return void
   * 
   * */
  //--------------------------------------------------------------------------
  public function setSubclass($class, $superclasses) {
    if (is_array($superclasses)) {
      foreach ($superclasses as $superclass) {
        if ($superclass == $class)
          continue;

        $this->createStatement($class, SONIA_RDFS_CREATE_SUBCLASS, $superclass);
      }
    } else
      if (!empty ($superclasses)) {
        if ($class == $superclasses)
          return;
        $this->createStatement($class, SONIA_RDFS_CREATE_SUBCLASS, $superclasses);
      }

  }

  public function generateObjectAutoIncrementField() {
    return $this->generateAutoIncrementField(SONIA_CREATE_OBJECT_COUNT_TYPE);
  }

  public function generatePropertyAutoIncrementField() {
    return $this->generateAutoIncrementField(SONIA_CREATE_MAX_ID_PROPERTY_TYPE);
  }

  public function generateFileAutoIncrementField() {
    return $this->generateAutoIncrementField(SONIA_CREATE_MAX_ID_FILE_TYPE);
  }

  //--------------------------------------------------------------------------
  /**
   * 
   * places facet and its value in an existing class. Same as an RDF Literal
   * 
   * USAGE:
   * 
   * @param string $object_name_or_id the reference to the object
   * @param string $facetType the identifier for the property
   * @param string $data the data
   * @param string $dataType int, string, etc , not enforced
   * 
   * 
   * @return array a multi-deminsional array expressing the hierarchy of 
   * categories
   * 
   * @see createObject()
   * */
  //--------------------------------------------------------------------------
  public function setObjectProperty($object_name_or_id, $property, $data, $dataType) {

  }

  /**
   * 
   * @return boolean
   * @see statementExists()
   * 
   * */
  public function URIExists($s) {
    return $this->statementExists($s, null, null, true);
  }

  /**
  * must be exact match - use FILTER through QUERY for regex matching
  * @return boolean
  * @see statementExists()
  * 
  * */
  public function LiteralExists($s) {
    return $this->statementExists(null, null, $s, false, true);
  }

  //--------------------------------------------------------------------------
  /**
  * basic function to confirm if a statement exists,
  * used to determine if a class, or paramter has already been created
  * 
  * @param string uri subject
  * @param string uri predicate
  * @param string uri object
  * @param boolean - see if the one URI exists anywhere in a statement
  * @param boolean - do an exact match on a literal
  * 
  * @return boolean does the statement exist
  * */
  //--------------------------------------------------------------------------
  public function statementExists($s, $p = null, $o = null, $any = false, $literal = false) {
    $query = ARC_DEFAULT_PREFIXES;
    $s = trim($s);

    if ($any && !empty ($s)) {
      /*
       * works but too slow
       * ARC recomendation is for 3 separate queries
       * 
       * SELECT DISTINCT * WHERE {
        ?x ?y ?z.
        FILTER (
        ?x = "http://www.xml.com/" || 
        ?y = "http://www.xml.com/" ||
        ?z = "http://www.xml.com/" 
        )
        }
       * */
      $s = ' <' . trim($s) . '>';
      $query1 = $query . " ASK WHERE { $s ?p ?o . } LIMIT 1";
      $qr1 = $this->data_api->query(array (
        "query" => $query1,
        "result_type" => "bool"
      ));
      if ($qr1['result'])
        return true;

      $query2 = $query . " ASK WHERE { ?s ?p $s . } LIMIT 1";
      $qr2 = $this->data_api->query(array (
        "query" => $query2,
        "result_type" => "bool"
      ));
      if ($qr2['result'])
        return true;

      $query3 = $query . " ASK WHERE { ?s $s ?o . } LIMIT 1";
      $qr3 = $this->data_api->query(array (
        "query" => $query3,
        "result_type" => "bool"
      ));
      if ($qr3['result'])
        return true;

    } else
      if ($literal) {
        $s = '?s';
        $p = '?p';
        $o = ' "' . trim($o) . '"';

        //TODO check for validity
        $query .= " ASK WHERE { $s $p $o . }";
        $qr = $this->data_api->query(array (
          "query" => $query,
          "result_type" => "bool"
        ));

        if ($qr['result'])
          return true;
      } else {
        $p = (!empty ($p)) ? ' <' . trim($p) . '>' : '?p';
        $o = (!empty ($o)) ? ' <' . trim($o) . '>' : '?o';
        $s = (!empty ($s)) ? ' <' . trim($s) . '>' : '?s';

        //TODO check for validity
        $query .= " ASK WHERE { $s $p $o . } LIMIT 1";
        $qr = $this->data_api->query(array (
          "query" => $query,
          "result_type" => "bool"
        ));

        if ($qr['result'])
          return true;
      }

    return false;

  }

} //end interface
