/*
  (c) 2007 - Jonathan Hendler All rights Reserved
  SONIA Sesame is a HTTP client for Sesame using the Zend Platform Library

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.semanticsearch.sonia;

import java.io.File;
import java.net.URL;
import java.util.*; // import java.util.Hashtable;
import java.nio.charset.*;

//openRDF
import org.openrdf.model.*;
import org.openrdf.sail.memory.*;
import org.openrdf.model.Literal;
import org.openrdf.repository.Repository;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.repository.sail.SailRepositoryConnection;
import org.openrdf.sail.memory.MemoryStore;
import org.openrdf.sail.nativerdf.NativeStore;
import org.openrdf.repository.http.HTTPRepository; // import
													// org.openrdf.model.impl.*;
import org.openrdf.OpenRDFException;
import org.openrdf.repository.RepositoryConnection;
import org.openrdf.query.TupleQuery;
import org.openrdf.query.TupleQueryResult;
import org.openrdf.query.BindingSet;
import org.openrdf.query.QueryLanguage;
import org.openrdf.model.vocabulary.RDF;
import org.openrdf.model.vocabulary.RDFS;
import org.openrdf.rio.RDFFormat;
import org.openrdf.repository.RepositoryResult;

// quercus
import com.caucho.quercus.env.Env;
import com.caucho.quercus.module.AbstractQuercusModule;

//logging
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.SimpleLayout;
import org.apache.log4j.FileAppender;
import org.apache.log4j.LogManager;

/**
 * class SONIA_Sesame is an HTTP wrapper for querying a SESAME store. The store
 * must already be set up, but data can be added, updated, removed, and queried.
 * 
 * This is only tested with Sesame 2.0 beta 5 as of this writing.
 * 
 * The main features of using this approach is
 * 
 * 1. speed, both in avoiding heavy rpc and in how the queries are executed 
 * 2.full features and access to the SESAMI API
 * 
 * @version 0.2
 * @package SONIA
 * @author Jonathan Hendler [jonathan dot hendler (at) gmail dot com]
 * @license GPLv3 or higher
 * 
 * @link http://www.openrdf.org sesame
 * @link http://quercus.caucho.com/quercus-3.1/doc/quercus-java-integration.xtp
 * @link http://www.openrdf.org/doc/sesame2/2.0-beta5/users/ch04.html
 * @link http://quercus.caucho.com/quercus-3.1/examples/quercus-module/index.xtp
 * @link http://java.sun.com/j2se/1.5.0/docs/api/java/util/Hashtable.html
 * 
 */
public class SesamePHPModule extends AbstractQuercusModule {
	protected String context;
	private Repository repo;
	private RepositoryConnection con;
	private static QueryLanguage lang;
	public boolean debug;
	//TODO cache these somewhere outside of the class, if possible or nec
	protected Hashtable<String,TupleQuery> preparedQueries;
	protected ValueFactory valueFactory;
	//a collection of hashtable buffers
	private int hashtableBufferCapacity = 20;
	private long totaltime;
	private Logger logger;
	//the PHP environment
	static Env env;
	
	/**
	 * constructor emulater - the constructor is left to Quercus
	 * also let's you reconfigure object
	 * 
	 * @todo add auth. For now, secure through only allowing localhost for the http server access
	 */
	public void SesamePHP_init(Env env, String context, String repo, String storetype,
			boolean debug, String logFileLocation) {
		
		String queryLanguage = "SPARQL";
		this.lang = new QueryLanguage(queryLanguage);
		this.debug = debug;
		this.context = context;
		this.preparedQueries = new Hashtable<String,TupleQuery>();
		this.env = env;
        this.totaltime = 0;
        
        //only outputs to console
        
        String start_message  = "Using org.apache.log4j.BasicConfigurator.configure(), otheriwse would be at "+logFileLocation+"SesamePHPmodule.log";
        
		//Logging
		if( LogManager.exists("SesamePHPmodule") == null){
			org.apache.log4j.BasicConfigurator.configure();
			this.logger = Logger.getLogger("SesamePHPmodule");
			try {
				  
		    	 FileAppender appender = (FileAppender) this.logger.getAppender(logFileLocation+"SesamePHPmodule.log");
		    	 if (appender == null){
		    		 SimpleLayout layout = new SimpleLayout();
		    		 appender = new FileAppender(layout,logFileLocation+"SesamePHPmodule.log");
		    		 this.logger.addAppender(appender);
		    		 this.logger.setLevel((Level) Level.DEBUG);
		    	 }
		         
		      } catch(Exception e) {}
		} else {
			this.logger = Logger.getLogger("SesamePHPmodule");
		}
		this.logger.error(start_message);
	      /*  logger.debug("Here is some DEBUG");
      logger.info("Here is some INFO");
      logger.warn("Here is some WARN");
      logger.error("Here is some ERROR");
      logger.fatal("Here is some FATAL"); */
		
        
		if (storetype.equals("memory")) {
			try {
				File dataDir = new File(repo);
				MemoryStore memStore = new MemoryStore(dataDir);
				memStore.setSyncDelay(0L);
				this.repo = new SailRepository(memStore);
			} catch (Exception e) {
				this.logger.error(e.toString());
			}
		} else if (storetype.equals("native")) {
			try {
				File dataDir = new File(repo);
				this.repo = new SailRepository(new NativeStore(dataDir));
			} catch (Exception e) {
				this.logger.error(e.toString());

			}
		} else if (storetype.equals("remote")) {
			try {
				int pos = repo.indexOf("::");
				String sesameServer = repo.substring(0, (pos-1));
				String repositoryID = repo.substring((pos+2) );
				//env.println("<br/>Server "+sesameServer+", Repoid = "+repositoryID);
				this.repo = new HTTPRepository(sesameServer.trim(), repositoryID.trim());
			} catch (Exception e) {
				this.logger.error(e.toString());
			}
		}

		try { 
			this.repo.initialize();
			this.valueFactory = this.repo.getValueFactory();
			this.con = this.repo.getConnection();
		} catch (Exception e) {
			 this.logger.error(e.toString());
		}
	}

	/**
	 * a simple store get
	 */
	public Hashtable[] SesamePHP_simpleGet(Env env, String sparql,
			String[] params ) {
		
		Hashtable[] r = null;
		if (this.debug) {
			this.logger.debug("Executing: " + sparql + " In context of " + this.context);
		}
		try {
			TupleQuery tupleQuery = this.con.prepareTupleQuery(this.lang, sparql);
			r = this.basicGet(tupleQuery, params);
		} catch (Exception e) {
			this.logger.error("SesamePHP_simpleGet(): "+e.toString());
		}
		return r;
	}
	
	/**
	 * a simple store get
	 */
	public int SesamePHP_simpleGetCount(Env env, String sparql) {
		TupleQuery tupleQuery = null; 
		try {
			 long start = System.currentTimeMillis();
			 tupleQuery = this.con.prepareTupleQuery(this.lang, sparql) ;
			 long end = System.currentTimeMillis();
		     //env.println("COMPLETED INNER prepare"+(end-start));
		} catch(Exception e){
			 this.logger.error("SesamePHP_simpleGetCount(): "+e.toString());
		}
		return this.basicGetCount( tupleQuery ); 
	}
	
	/**
	 * a prepared store get
	 */
	public int SesamePHP_preparedGetCount(
			Env env, 
			String preparedName, 
			String sparql,
			Hashtable<String,String>[] bindings) {
		
		TupleQuery tupleQuery = null;
		String key;
		String param;
		String resourceType;
		String resourceValue;
		
		try {
			if (this.preparedQueries.containsKey(preparedName)){
				tupleQuery = this.preparedQueries.get(preparedName);
			} else { // create the query
				tupleQuery = this.con.prepareTupleQuery(this.lang, sparql);
				this.preparedQueries.put(preparedName,tupleQuery);
			}
			
			//bind the variables
			for (int i=0;i<bindings.length;i++) {
				resourceType = bindings[i].get("type");
				resourceValue = bindings[i].get("value");
				param = bindings[i].get("key");
				
				if (resourceType.equals("literal")){
					tupleQuery.setBinding(param, this.valueFactory.createLiteral(resourceValue));
				} else {
					tupleQuery.setBinding(param, this.valueFactory.createURI(resourceValue));
				}
			}
			
	    } catch(Exception e){
	    	this.logger.error("SesamePHP_preparedGetCount(): "+e.toString());
	    }
	    
	    
	    
		return this.basicGetCount(tupleQuery);
	}
 	
	/**
	 * this private class speeds up the user experience for updates
	 * */
	private class UpdateStatement extends Thread{
		public Logger logger = null;
		public RepositoryConnection con = null;
		public ValueFactory valueFactory = null;
		public String s, p, o, is_literal,as, ap, ao, ais_literal;
		
		/**
		 * constructor of sorts
		 * */
		public synchronized void init(
				Logger logger,
				RepositoryConnection con,
				ValueFactory valueFactory,
				String s, 
				String p,
				String o,
				String is_literal,
				String as,
				String ap,
				String ao,
				String ais_literal
				
		){
			this.logger = logger;
			this.con = con;
			this.valueFactory = valueFactory;
			this.s = s;
			this.p = p;
			this.o = o;
			this.is_literal = is_literal;
			this.as = as;
			this.ap = ap;
			this.ao = ao;
			this.ais_literal = ais_literal;
		}
		
		/**
		 * the thread start
		 * */
		public synchronized void run(){
			this.doUpdate(
					this.s,
					this.p,
					this.o,
					this.is_literal,
					this.as,
					this.ap,
					this.ao,
					this.ais_literal
					);
		}
		
		public synchronized void doUpdate(
				String s, 
				String p,
				String o,
				String is_literal,
				String as,
				String ap,
				String ao,
				String ais_literal	){
			try {
				URI subject = null;
				URI predicate = null;
				Value object = null;
				
				if ((s != null) || (p != null) || (o != null)){
					try{
						if(s != null && (s != "")) subject = this.valueFactory.createURI(s);
					} catch (Exception e){
						this.logger.error("ERROR - s="+s+e.toString());
					}
					try{
						if(p != null && (p != "")) predicate = this.valueFactory.createURI(p);
					} catch (Exception e){
						this.logger.error("ERROR - p="+p+e.toString());
					}
					
					if(o != null && (o != "")){
						if (is_literal != null && is_literal !=""){
							object = this.valueFactory.createLiteral(o);
						} else {
							try {
							    object = this.valueFactory.createURI(o);
							} catch (Exception e){
								this.logger.error(""+e.toString());
							}
						}	
					}
					//safety check
					if ((subject != null && subject.toString().length() >0) || (predicate != null && predicate.toString().length() >0) || (object != null )){
					    try{
					    	this.con.setAutoCommit(false);
					    	this.con.remove(subject, predicate, object);
					    	this.logger.debug(" Removed "+s+" "+p+" "+o );
					    	this.con.commit();
						} catch (Exception e){
							this.con.rollback();
							this.logger.error("failed Removed -"+s+" "+p+" "+o+e.toString());
						}
					}
			    }
				
				if ((as != null && as.toString().length() > 0) &&  (ap != null && ap.toString().length() > 0) && (ao != null)) {
					Value new_object = null;
					URI new_subject = null;
					URI new_predicate = null;
					
					try{
						new_subject = this.valueFactory.createURI(as);
					} catch (Exception e){
						this.logger.error(" add subject=\""+as+"\" "+e.toString());
						this.logger.debug(" errored during  "+s+" "+p+" "+o );
					}
					
					try{
					    new_predicate = this.valueFactory.createURI(ap);
					} catch (Exception e){
						this.logger.error("add predicate=\""+ap+"\" "+e.toString());
					}
					if (ais_literal != null && ais_literal !=""){
						new_object = this.valueFactory.createLiteral(ao);
						this.logger.debug("created literal -"+ao);
					} else {
						try {
						    new_object = this.valueFactory.createURI(ao);
						} catch (Exception e){
							this.logger.error(" add object-"+e.toString());
						}
					}
					if ((new_subject != null) &&  (new_predicate != null) && (new_object != null)){
						try {
							this.con.setAutoCommit(false);
					        this.logger.debug(" Added -"+as+", "+ap+", "+ao);
							this.con.add(new_subject, new_predicate, new_object);
							this.con.commit();
						} catch (Exception e){
							this.con.rollback();
					        this.logger.error("error Added -"+as+" "+ap+" "+ao+e.toString());
						}
					}
				}
			}
			catch (Exception e) {
			   // handle exception
				this.logger.error("ais_literal="+ais_literal+", "+s+p+o+as+" "+ap+" "+ao+e.toString());
			}
			try {
				this.con.commit();
			} catch (Exception e) {
				   // handle exception
					this.logger.error(" final commit" + e.toString());
				}
		}//end do update
	}//end private class
	
	/**
	 * used for updates, deletes, and adds
	 * threaded and synchronized to create an appearance of speed
	 * transaction safe (not deeply tested)
	 * */
	public synchronized boolean SesamePHP_updateStatement(
			Env env, 
			String s, 
			String p,
			String o,
			String is_literal,
			String as,
			String ap,
			String ao,
			String ais_literal ){
		
		//http://java.sun.com/docs/books/tutorial/java/javaOO/nested.html
		SesamePHPModule.UpdateStatement u = this.new UpdateStatement();
		u.init(		this.logger,
					this.con,
					this.valueFactory,
					 s, 
					 p,
					 o,
					 is_literal,
					 as,
					 ap,
					 ao,
					 ais_literal);
		u.run();
		return true;
	}
	
	/**
	 * check to see if a statement exists
	 * */
	public boolean SesamePHP_statementExists(Env env, Hashtable<String,String> statement ){
		//might be null or BNODE
		boolean result = false;
		Value object;
		try{
			URI subject = this.valueFactory.createURI(statement.get("s"));
			URI predicate = this.valueFactory.createURI(statement.get("p"));
			if (statement.get("is_literal") != "true"){
				object = this.valueFactory.createURI(statement.get("o"));
			} else {
				object = this.valueFactory.createLiteral(statement.get("o"));
			}
			
			RepositoryResult<Statement> statements = this.con.getStatements(subject, predicate, object, false);
			
			try {
			   while (statements.hasNext()) {
				   result = true;
				   break;
			   }
			}
			finally {
			   statements.close(); // make sure the result object is closed properly
			}
		} catch(OpenRDFException e) {
			 
		}
		return result;
	} 
	
	/**
	 * a prepared get to use when many variations and iterations happen over a query
	 */
	public Hashtable[] SesamePHP_preparedGet(Env env, String preparedName, String sparql,
			String[] params, Hashtable<String,String>[] bindings ) {
		
		TupleQuery tupleQuery;
		Hashtable[] r = null;
		//Hashtable<String,String> temp;
		//Enumeration<String> keys;
		String key;
		String param;
		String resourceType;
		String resourceValue;
		
		try {
			//get the query
			// env.apc_fetch("$sonia_sesame_config");
			if (this.preparedQueries.containsKey(preparedName)){
				tupleQuery = this.preparedQueries.get(preparedName);
			} else { // create the query
				tupleQuery = this.con.prepareTupleQuery(this.lang, sparql);
				this.preparedQueries.put(preparedName,tupleQuery);
			}
			//bind the variables
			for (int i=0;i<bindings.length;i++) {
				resourceType = bindings[i].get("type");
				resourceValue = bindings[i].get("value");
				param = bindings[i].get("key");
				
				if (resourceType.equals("literal")){
					tupleQuery.setBinding(param, this.valueFactory.createLiteral(resourceValue));
				} else {
					tupleQuery.setBinding(param, this.valueFactory.createURI(resourceValue));
				}
			}
			
			//execute
			r = this.basicGet( tupleQuery, params);
		} catch (Exception ex){
			 this.logger.error(ex.toString());	
		}
		
		return r;
	}
	
	/**
	 * 
	 * @param query 1
	 * @param other querys (query, params, count,group,order)
	 * @param params q1
	 * @param params q2
	 * @param count by->
	 * @param q2 type
	 * @param order by
	 * @param 
	 * 
	 * @todo allow grouping to be done as a prepared get OR regular get to test speed
	 * @todo allow inferencing for subclasses
	 * */
//	
//	public  Hashtable[] SesamePHP_aggregateGet(
//			Env env, 
//			String preparedName, 
//			String preparedType,
//			String sparql1,
//			String sparql2,
//			Hashtable<String,String> params1, 
//			Hashtable<String,String> params2, 
//			Hashtable<String,String>[] preparedBindings, 
//			String[] order_by,
//			String[] sorting,
//			Boolean doindex){
//		
//		//get the result of the query
//		Hashtable[] result = SesamePHP_simpleGet( env, sparql1,params1, doindex);
//		
//		if (result.length>0){
//			//depending on the params grouping, counting, and ordering or other functions
//			//if preparedName = class
//			// do the prepared query for class using Sparql two
//			String bindingValue = "";
//			try{
//				if (preparedType.equals("count")){
//					int count = 0;
//				    for(int i=0;i<result.length;i++){
//				    	for(int j=0;j<preparedBindings.length;j++){
//				    		bindingValue = (String)result[i].get((String)preparedBindings[j].get("key"));
//				    		preparedBindings[j].put("value",bindingValue);
//				    	}
//				    	count = this.SesamePHP_preparedGetCount( env,  preparedName,  sparql2, params2, preparedBindings, doindex);
//						result[i].put("count",Integer.toString(count));
//					}
//				}
//			} catch(Exception ex){
//				
//			}
//			
//			if (order_by.length >0){
//				result = this.doOrderBy(result,order_by,sorting);
//			}
//			
//			return result;
//		} else {
//			return result;
//		}
//		
//	}
	
	/**
	 * sorting functions on the Hashtable result of the queries
	 * @todo saftey checks
	 * @todo add more of these object iniatiions to the construct
	 * 
	 * @link http://java.sun.com/docs/books/tutorial/collections/algorithms/index.html
	 * @link http://java.sun.com/javase/6/docs/api/java/util/Collections.html
	 * @link http://java.sun.com/javase/6/docs/api/java/util/Comparator.html
	 * */
	/*
//	private Hashtable[] doOrderBy( 
//			Hashtable[] results,
//			String[] order_by,
//			String[] sorting
//			){
//		
//		/**
//		 * a container for the return values
//		 * */
//		int resultlength = results.length;
//		Hashtable[]  final_result = new Hashtable[resultlength] ; //new Hashtable<Integer, Hashtable<String,String>>();
//		/**
//		 * a mapping for looking up keys of the provided results when the sorting is complete
//		 * */
//		Hashtable<String,Integer> mapTable = new Hashtable<String,Integer>(); 
//		
//		/**
//		 * the array of strings for sorting
//		 * */
//		ArrayList<String> sortingTable = new ArrayList<String>();
//		
//		/**
//		 * a container for the sort string
//		 * */
//		String sortstring;
//		String order_key;
//		String order_value;
//		Iterator it;
//		
//		Hashtable<String,String>  row = new Hashtable<String,String>();
//		int rownumber=0;
//		int or_num = order_by.length;
//		
//		//iterate over the results
//		for(int j=0;j<resultlength;j++){
//			sortstring = "";
//			row = results[j];
//			
//			//iterate over the orders
//			for(int i=0;i< or_num ; i++) {
//				order_key = order_by[i];
//				order_value = row.get(order_key);
//				for(int v=0;v< order_value.length();v++){
//					sortstring += order_value.charAt(v);
//				}
//			}
//			
//			//get the string from the row and addit to the sortstring
//			mapTable.put(sortstring,j);
//			sortingTable.add(sortstring);
//		}
//		
//		//sort the sortTable
//		Collections.sort(sortingTable); //add MIXED ASC DESC
//		it = sortingTable.iterator();
//		//iterate the sort table
//		//retrieve the result key from the sortable
//		//get the hashtable result row from the results
//		//place in the final results with the new sorted index
//		rownumber = 0;
//	    while (it.hasNext()) {
//	    	
//	       order_key =  (String)it.next();
//	       final_result[rownumber] = results[mapTable.get(order_key)];
//	       rownumber++;
//	    }
//		return final_result;
//	}
	
//	public Hashtable[] SesamePHP_testDoOrderBy(Env env){
//		Hashtable[] result = new Hashtable[3];
//		String[] order_by = new String[2];
//		String[] sorting = new String[2];
//		order_by[0] = "y";
//		order_by[1] = "z";
//		
//		Hashtable<String,String> result1 = new Hashtable<String,String>();
//		Hashtable<String,String> result2 = new Hashtable<String,String>();
//		Hashtable<String,String> result3 = new Hashtable<String,String>();
//		
//		//should be the second result
//		result1.put("x","second");
//		result1.put("y","b");
//		result1.put("z","a-second");
//		
//		//should be the first result
//		result2.put("x","first");
//		result2.put("y","a");
//		result2.put("z","z-first");
//		
//		//should be the last
//		result3.put("x","last");
//		result3.put("y","z");
//		result3.put("z","a-last");
//		
//		result[0]= result1;
//		result[1]= result2;
//		result[2]= result3;
//		
//		return this.doOrderBy(  result,  order_by ,sorting);
//	}

	/**
	 * the abstracted function for retrieveing data
	 * @todo use generics
	 */
	private int basicGetCount( TupleQuery tupleQuery ) {
		int count = 0;
		long start=0,end =0;
		
		if (this.debug){
		   start = System.currentTimeMillis();
		}
		try {
			TupleQueryResult result = tupleQuery.evaluate();
			BindingSet bindingSet;
			try {
				while (result.hasNext()) {
					bindingSet = result.next();
					++count;
				}
			} finally {
				result.close();
			}
		} catch (Exception ex) {
			this.logger.error(ex.toString());
		}
		
		if (this.debug){
			end = System.currentTimeMillis();
			this.logger.debug("Executed basicGetCount() in " + (end-start)+" seconds.");
			
		}  
		return  count;
	}
	
	
	/**
	 * allows the external PHP app to use this log
	 * @todo add debugging, info, and error levels
	 * */
	public void setlog(String message){
		this.logger.info(message);
	}
	
	/**
	 * the abstracted function for retrieveing data
	 * @todo use generics
	 */
	private Hashtable[] basicGet(
			TupleQuery tupleQuery, 
			String[] params 
			) {
		
		Hashtable<String,String> temp = new Hashtable<String,String>();
		ArrayList<Hashtable> resultHolder = new ArrayList<Hashtable>();
		Hashtable[] returnArray = new Hashtable[0];
		TupleQueryResult result = null;
		int index = 0;
		long start=0,end = 0;
		float hashtabletotal =0 ,bindingtotal=0,resultholderaddtotal =0,resultnexttotal=0,returnconversion=0,hasnexttotal=0;
		long hs=0, bs=0,rs=0,rn=0,ret=0,hn=0;
		float total =0,bigtotal = 0;
		boolean hasnext;
		
		try{
			if (this.debug) {
			  start = System.nanoTime();
			}
			result = tupleQuery.evaluate();
			if (this.debug){
			    end = System.nanoTime();
			    total = (float)(end-start)/1000000000;
				this.totaltime += (end-start);
				bigtotal = (float) this.totaltime /1000000000;
				this.logger.info("TIME OF EVAL QUERY: "+total +"seconds of "+bigtotal + " total.") ;
				
			}
		} catch (Exception ex) {
			this.logger.error("HERE "+ex.toString());
		}
		
		try {
			Enumeration<String> e;
			String param;
			Value value;
			String tempString = "";
			
			BindingSet bindingSet;
			int i;
			try {
				if (this.debug) {
					  start = System.nanoTime();
			    }
	            
				
				hasnext = result.hasNext();
				
				while (hasnext) {
					if (this.debug) hs = System.nanoTime();
					//long hashtabletotal =0 hs=0, bindingtotal=0,bs=0,resultholderaddtotal =0,rs=o;
					temp = new Hashtable<String,String>(this.hashtableBufferCapacity);
					if (this.debug) {
						hashtabletotal += (float)(System.nanoTime() -hs)/1000000000;
					}
					
					if (this.debug) rn = System.nanoTime();
					bindingSet = result.next();
					if (this.debug) {
						resultnexttotal += (float)(System.nanoTime() -rn)/1000000000;
					}
					
					for(i=0;i<params.length;++i){
						param = params[i];
						try{
							if (this.debug) bs = System.nanoTime();
							if (bindingSet.hasBinding(param)){
						        value = bindingSet.getValue(param);
						        //see http://www.openrdf.org/forum/mvnforum/viewthread?thread=1465
						        if (value instanceof URI) {
						        	tempString = value.toString();
					        	}
					        	else if (value instanceof Literal) {
					        		//	  tempString = value.toString().replace("@en","").replaceAll("\"$","").replaceAll("^\"","");
					        		tempString = ((Literal)value).getLabel();
					        	}
					        	else {
					        	   // handle the case of bnodes, for example by throwing an exception
					        	}
						        
						        temp.put(param,  tempString);
							}
							if (this.debug) {
								bindingtotal += (float)(System.nanoTime() -bs)/1000000000;
							}
						} catch (Exception ex) {
							this.logger.error("basicGet() temp.put(param, value.toString()  );"+ex.toString());
						}
					}
					if (this.debug) rs = System.nanoTime();
					resultHolder.add(index,temp);
					
					if (this.debug) {
						resultholderaddtotal += (float)(System.nanoTime() -rs)/1000000000;
					}
					index++;
					
					if (this.debug) hn = System.nanoTime();
					hasnext = result.hasNext();
					
					if (this.debug) {
						hasnexttotal += (float)(System.nanoTime() -hn)/1000000000;
					}
				}
				
				if (this.debug){
				    end = System.nanoTime();
				}
				
			} finally {
				try{
					result.close();
				} catch (Exception ex) {
					this.logger.error("result.close() "+ex.toString());
				}
			}
		} catch (Exception ex) {
			this.logger.error("Result size: " + index +"THERE " + params.length +" " +this.hashtableBufferCapacity+ " " + ex.toString());
		}
		
		if (this.debug) ret = System.nanoTime();
		returnArray =   resultHolder.toArray(returnArray);
		if (this.debug) {
			returnconversion += (float)(System.nanoTime() -ret)/1000000000;
		    total = (float)(end-start)/1000000000;
		    this.totaltime += (end-start);
			bigtotal = (float) this.totaltime /1000000000;
			this.logger.info("PARSING results: "+total +"seconds of "+bigtotal + " total with \n hasnexttotal="+hasnexttotal+", resultholderaddtotal="+resultholderaddtotal+",hashtabletotal="+hashtabletotal+",bindingtotal="+bindingtotal+", resultnexttotal="+resultnexttotal+", returnconversion="+returnconversion+".") ;
		}
		return returnArray;
	}

	/**
	 * @param env   provides Quercus environment resources.
	 * @param str
	 */
	public static void SesamePHP_test(Env env, String str) {
		env.println("Dude(tte) - it works. You said: " + str);
	}

	/**
	 * a cleanup function, called from the php object __destruct()
	 * 
	 * @todo not needed?
	 */
	public void finalize() {
		try {
			this.con.close();
		} catch (Exception e) {

		}
	}
	
	

}//end main class

/**
 * a singleton representing a SPARQL view
 * */
//	public class SPARQLViews {
//	    private static SPARQLViews instance = null;
////	    protected String context;
////	    private Repository repo;
////		private RepositoryConnection con;
////		private static QueryLanguage lang;
////		public boolean debug;
////		protected ValueFactory valueFactory;
////		//a collection of hashtable buffers
////		private int hashtableBufferCapacity = 20;
//	
//	  private SPARQLViews() {
//		  
//	  }
//	  
//	  public SPARQLViews createInstance(){
//		 instance = new SPARQLViews();
//	  }
//	
//	  public static SPARQLViews getInstance() {
//	    return instance;
//	  }
//	} //end private class
