<?php
/*
  (c) 2007 - Jonathan Hendler All rights Reserved
  SONIA Sesame is a HTTP client for Sesame using the Zend Platform Library

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

///////////////////////////////////////////////////
//WARNING ---------------------
//SPECIAL NON STANDARD PHP - RESIN Quercus integration
// see http://quercus.caucho.com/quercus-3.1/doc/quercus-java-integration.xtp
import  org.semanticsearch.sonia.*;
//END WARNING
///////////////////////////////////////////////////
 

/**
 * class SONIA_Sesame is an HTTP wrapper for querying a SESAME store. The store must 
 * already be set up, but data can be added, updated, removed, and queried.
 * 
 * This is only tested with Sesame 2.0 beta 5 as of this writing.
 * 
 * The main features of using this approach is 
 * 
 * 1. speed, both in avoiding heavy rpc and in how the queries are executed
 * 2. full features and access to the SESAMI API
 * 
 * There are some interesting things that can be done with 
 * 4.2.2.3. Preparing and Reusing Queries 
/* 
 * 
 * TODO list
 * 
 *  - refactor functions on client side
 *    - go through central interface
 *    - make this store an API/
 *      - public of get implemented
 *      - private abstract
 *      - private implemented check args
 *      
 *  - include a function to check the args
 *  - include a function to get the count of a feature within a search (class, class property, property, property, value)
 *  - implement the internals of the get()
 *     - order by
 *     - limit
 *     - recurse and/or inference
 *  - get objects data without optionals to increase speed
 * 
 *  - objects count in search
 *    - get objects without optionals 
 *    - get file attachments as subquery
 *    - get the objects with their classes
 *    - store the search results and get the search results from there not a new query
 * 
 *  - add support for other store types
 * 
 * @version 0.3 BROKEN
 * @package SONIA
 * @author  Jonathan Hendler [jonathan dot hendler (at) gmail dot com]
 * @license GPLv3 or higher
 *  
 * @link http://framework.zend.com the Zend Framework
 * @link http://www.openrdf.org sesame
 * @link http://quercus.caucho.com/quercus-3.1/doc/quercus-java-integration.xtp
 * @link http://www.openrdf.org/doc/sesame2/2.0-beta5/users/ch04.html 
 * 
 * 
 * @todo clear context function (erase all)
 * 
 * */

class SONIA_Sesame_Caucho{
	/**
	 * The store object
	 * */
	public static $store;
  
  private static $logfile;
	
	/**
	 * 
	 * */
	private static $query_language;
	
	/**
  * the log function
  * */
	private static $log_function;
	
	/**
	 * The store connection
	 * */
	private  $connection;
	
	/**
	 * boolean, to show debugging
	 * */
	private $debug;
	
	/**
	 * The response object
	 * */
	private $response;
	
	
	/**
	 * value factory for generating URI objects
	 * */
	private $value_factory;
	
	/**
	 * A string of the repository
	 * */
	private $repo;
	
	/**
	 * A string of the default context URI
	 * */
	private $context;

	/**
	 * an array of prepared statement objects
	 * */
	private $prepared_statements;
	
	/**
	 * an in memory cache of results
	 * */
	private $cache_store;
	
	/**
	 * a prepared statement objects
	 * */
	private $getObjectData;
	private $getObjectsCountInSearch;
	
	
	/**
	 * Internal value
	 * */
	//const _SONIA_RDF_TYPE = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type';
	
	/**
	 * constructor or setting up connection
	 * @todo support other types, only supports MemoryStore at the moment
	 * */
	public function __construct($context, $repo = 'SESAME_DATA', $store_type = 'memory', $debug = false ,$logfile_location = ''){
	  $this->debug = $debug;
	  $this->repo = $repo;
	  $this->context = $context;
	  $this->store = new Java('SesamePHPModule');
    $this->store->SesamePHP_init($context,$repo,$store_type,$debug,"SPARQL",$logfile_location);
	}
	
	/**
	 * an objects shutdown function
	 * */
	public function __destruct(){
		$this->store->finalize();
	}
  
	 
	/**
	 * debug a java object object by finding it's class and reflecting
	 * */
	public function debugObject(&$object){
		$classname = $object->getClass();
		return '<hr/><h1>'.$classname.'</h1><br/>' .
	  $this->debugReflect(trim(str_replace('class','',$classname)));
	}
	
	/**
	 * handy debugging function using java's reflect
	 * */
	public function debugReflect($classname){
		import java.lang.*;
	  $c = new Java("Class");
	  $c_ = $c->forName($classname);
	  return nl2br(print_r($c_->getDeclaredMethods()),1);
	}
	
	
	/**
	 * get instances of the class, property, or 
	 * */
	private function getCount($object,$property=null, $strict=false, $literal=true){
    $object =  ($literal) ?' "' . $object  . '" ':' <' . $object  . '> ';
    $_strict = ($strict) ? ' FILTER(REGEX(str(?s),"^'.SONIA_OBJECT_ID_URI.'")) . ' :'';
    $property = ($property !== null) ? '<' . $property . '>': ' ?p ';
    $count_query = 'SELECT DISTINCT ?s WHERE { ?s  ' . $property . ' '.$object.' . '.$_strict.' }';
    return $this->store->SesamePHP_simpleGetCount($count_query,array('s'));
	}
	
		/**
	 * get instances of the class, property, or 
	 * */
	private function getParents($class){
    $query = 'SELECT ?parent WHERE { ?class <http://www.w3.org/2000/01/rdf-schema#subClassOf> ?parent . } ';
    $bindings[1]['value'] = $class;
    $bindings[1]['type'] = 'resource';
    $bindings[1]['key'] = 'class';
    return $this->store->SesamePHP_preparedGet('getParents',$query, array('parent'),array(array('value')),false);
	}

  
	
	/**
	 * Function for doing a nested, aggregated set of queries.
	 * 
	 * 
	 * @param array args['query']
	 *                  ['queryname']
	 *                  ['params'] = array()
	 *                  ['order_by'][]['field'|'type'[NUMERIC|STRING|REGULAR]|order=ASC|DESC]
	 *                  ['count'] true|false
	 *                  ['limit'] 
	 *                  ['main_query'] the PHPSESSID indicating this is a main query
	 *                  ['offset'] 
	 *                  ['inference']
	 *                  ['hierarchy'] param name to merge hierarchy by
	 *                  ['children'][]['query']
	 *                                ['queryname']  
	 *                                ['params']
	 *                                ['bindings'][][value|type|key]
	 *                                ['binding_field_name'] name this result should have in the parent
	 *                                ['order_by']
	 *                                ['limit'] 
	 *                                ['offset'] 
	 *                                [recurse] depth//in place of inference?
	 *                                ['count'] true|false
	 *                                ['count_in_search'] a PHPSESSID indicating to use the main query to get COUNT IN SEARCH
	 *                                ['inference']
	 *                                ['prepare_view'] true|false 
	 *                                ['children']...
	 * 
	 * @return mixed result[rowindex][colname] = colvalue |
	 *                                           result[queryname] = colvalue
	 * 
	 * */
	public function get(&$args,&$parentresults){
	  $return_value = array();
	  $query = rawurldecode($args['query']);
	  try {
	  	$count = 0;
	  	$inner_count = 0;
	  	if (isset($args['limit'])) {
	  	  $limit = $args['limit'];
	  	  $offset = isset($args['offset']) ? $args['offset'] : 0;
	  	} else {
	  	  $limit =  intval('1000000000000');
	  	  $offset = 0;
	  	}
	  	
	  	if (!empty($args['prepare_view']) && !empty($parentresults)){
	  		  $bindings = array();
	  		  foreach($parentresults as $key=>$value){
	  		  	//apply limit
	  		  	if ($count>=$offset && ($inner_count <$limit)){
		  		  	//apply bindings
		  		  	
		  		  	foreach($args['bindings'] as $k=>$binding ){
		  		  		$bkey = $binding['key'];
		  		  		$binding['value'] = $value[$bkey];
		  		  	  $bindings[$k] = $binding;
		  		  	}
		  		  	
              
		  		  	if ($binding['value'] != ''){ //make sure the binding isn't empty
						  	if(!empty($args['count']) ){ // and not count_in_search
						  	  $parentresults[$key][$args['binding_field_name']] = $this->store->SesamePHP_preparedGetCount($args['queryname'],$query,$bindings);
						  	} else {
						  	  //return an array 
					  	    if(!empty($args['count_in_search'])) {
					  	    	$objects = $this->store->SesamePHP_preparedGet($args['queryname'],$query, $args['params'],$bindings);
					  	    	//refactor
					  	    	$object_array1 = array();
					  	    	foreach($objects as $object){
					  	    	  $object_array1[] = $object['object'];
					  	    	}
					  	    	//get value
					  	    	$object_array2 = apc_fetch($args['count_in_search']);
					  	    	//array_intersect
					  	    	$parentresults[$key]['count'] = count($object_array1);
					  	    	if (is_array($object_array1) && is_array($object_array2)){
					  	    		$parentresults[$key]['count_in_search'] = count(array_intersect($object_array1,$object_array2));
					  	    	} else {
					  	    	  $parentresults[$key]['count_in_search'] = 0;
					  	    	}
					  	    } else {
					  	       $parentresults[$key][$args['binding_field_name']] = $this->store->SesamePHP_preparedGet($args['queryname'],$query, $args['params'],$bindings);
					  	    }
					  	  }
		  		  	} else { //end empty binding
		  		  	}
				  	  ++$inner_count;
	  		  	}
			  	  ++$count;
	  		  } //end foreach
	  		  $return_value = $parentresults;
	  	} else {
			  	if(!empty($args['count']) ){
			  	  //return an integer 
			  	  return  $this->store->SesamePHP_simpleGetCount($query);
			  	} else {
			  	  //return an array 
			  	  $params = $args['params']; 
			  	  $return_value = $this->store->SesamePHP_simpleGet($query, $params);
            if (empty($return_value)) {
              //fix for http://drupal.org/node/174853
              return;
            }
			  	  if (!empty($args['main_query']) ){
			  	      $session = $args['main_query'];
			  	      $object_array = array();
			  	      $collapsed_array = array();
			  	      foreach($return_value as $key=> $object){
			  	        $object_array[] = $object['object'];
			  	        if (!isset($collapsed_array[$object['object']])){
			  	           $collapsed_array[$object['object']]['object'] = $object['object'];
			  	        }
			  	        
			  	        foreach($object as $name=>$val){
			  	        	if ($name != 'object'){
			  	        	  $collapsed_array[$object['object']]['other'][$name][]  = $val;
			  	        	}
			  	        }
			  	      }
			  	      
			  	      //collapse any extras for order by
			  	      //there is [key][object]
			  	      //				[key][data]
			  	      //        [key][x]
			  	      $return_value = $collapsed_array;
			  	      apc_store($session,$object_array,60);
			  	  }
		  	  }
	  	}
	  	//ORDERBY SHOULD BE FIRST OR THIS SHOULD BE FIRST?
	  	// order by first there is a limit in the child
	  	// order by second if there is an order by here using the child
		  if (!empty($args['children']) && is_array($args['children']) && empty($args['main_query'])){
		  	 foreach($args['children'] as $child){
		         $return_value = $this->get($child,$return_value);
		  	 }
		  }
      
		  //TODO, THIS SHOULD COME BEFORE CHILD QUERIES TO REDUCE COUNT
		  if (!empty($args['startclass'])){
		  	$os = (!empty($args['original_startclass'])) ? $args['original_startclass']:'';
		  	$return_value = $this->normalizeHierarchy($return_value,$args['startclass'],$os);
		  }
		  
      
		  //NOTE: order by of search form elements is done in semantic_search.search.inc
		  //THIS is set in ...
		  if (!empty($args['order_by']) && is_array($args['order_by'])){
		  	//['order_by'][]['field'|'type'[NUMERIC|STRING|REGULAR]|order=ASC|DESC]
		    //foreach orderby, do the order by
        // $collapsed_array[$object['object']]['other'][$name]  = $val;  
        $number_of_sort_arrays = count($args['order_by']); 
        $order_by_string = '';
      	$order_count = 0;
      	//create index->objectURI mapping for workaround to http://bugs.caucho.com/view.php?id=2014
      	$value_array = array();
      	$keymap_array = array();
      	$count = 0;
      	foreach($return_value as $key =>$val){
      		$keymap_array[$count] = $key;
      		$value_array[$count] = $val;
      		++$count;
      	}
        foreach ($args['order_by'] as $order_by){
        	  $array_name = 'sort'.$order_count;
          	if (empty(${$array_name})){
          		 ${$array_name} = array();
          		 $order_by_string .= ' $'.$array_name.',SORT_'.$order_by['order'];
          		 if (isset($order_by['type'])) $order_by_string .= ', SORT_'.$order_by['type'];
          	}
          	//build the array of sorting 
          	foreach($value_array as $object_uri => $data){
		          if (isset($data['other'][$order_by['field']])){
		          	  //print '<br> DATA OTHER <br/>'; print_r($data['other']);
		          	 if (!is_array($data['other'][$order_by['field']])){
		          	   ${$array_name}[$object_uri] = $data['other'][$order_by['field']];
		          	 } else {
		               ${$array_name}[$object_uri] = $data['other'][$order_by['field']][0];
		          	 }
		          } else {
		             ${$array_name}[$object_uri] = null;
		          }
	          }
	          $order_count++;
	          $order_by_string .= ', ';
        }
        try{
 				  eval(' array_multisort('.$order_by_string.' $value_array ); ');
 				  //now remap as workaround to http://bugs.caucho.com/view.php?id=2014
 				  $new_array = array();
 				  foreach($value_array as $tempkey => $values){
 				    $new_array[$keymap_array[$tempkey]] = $values;
 				  }
 				  
 				  unset($keymap_array);
 				  unset($value_array);
 				  $return_value = $new_array;
 				  
        } catch  (Exception $e){
 				  error_log(print_r($e,1));
 				} 
 				if (!empty($args['children']) && is_array($args['children'])){
			  	 foreach($args['children'] as $child){
			         $return_value = $this->get($child,$return_value);
			  	 }
			  }
 				
		  }
		  
		  //ONLY APPLY LIMIT HERE IF IT HASN"T BEEN ON PARENTS
		  if (!empty($args['limit']) && is_array($args['limit'])){
		    //it is innefficient 
        /* MOVE THIS TO get on server   
		    if(!empty($this->object_search_result_ids) && is_array($this->object_search_result_ids)){
		      foreach($this->object_search_result_ids as $object_id){
		        if ($index >= $offset && $count_completed < SEMANTIC_SEARCH_DISPLAY_PAGINATION ){
		          $search_result[$object_id]  = $this->getObjectData($object_id);
		          ++$count_completed;
		        }
		        ++$index;
		       
		        if ($count_completed >= SEMANTIC_SEARCH_DISPLAY_PAGINATION) break;
		      }
		    }
		    */
		  }
		  
		} catch (Exception $e){
			 error_log(print_r($e,1));
		   return;
		}
		 
		return $return_value;
	}
	
	private  function normalizeHierarchy(&$classes, $startclass, $original_startclass,$climb=false){
	   $results = array();
  		foreach ($classes as $row) {
  			if (!isset($results[$row['class']])){
	        $results[$row['class']]['label'] = $row['label'];
	        $results[$row['class']]['class'] = $row['class'];
	        if (isset($row['count'])) $results[$row['class']]['count'] = $row['count'];
	        if (isset($row['count_in_search'])) $results[$row['class']]['count_in_search'] = $row['count_in_search'];
	        if (isset ($row['description']))
	          $results[$row['class']]['description'] = $row['description'];
  			}
        if (!empty($row['parent'])) $results[$row['class']]['parents'][$row['parent']] = $row['parent'];
      }
       
    return $this->_normalizeHierarchy($results, $startclass,$original_startclass,$climb);
	}
	
	/**
	 * recursive helper function
	 * */
	private  function _normalizeHierarchy(&$classes, $startclass, $original_startclass,$climb=false){
	  $final_array = array();
	  $layer1 = array ();
	  
	  foreach ($classes as $class_uri => $class) {
      if (empty( $class['parents']))  $class['parents'] = array();
      if (in_array($startclass, $class['parents'])) {
        $layer1[$class_uri] = $classes[$class_uri];
        
        foreach ($classes as $class_uri2 => $class2) {
          if (empty( $class2['parents']))  $class2['parents'] = array();
          if (in_array($class_uri, $class2['parents'])) {
            $layer1[$class_uri]['children'][$class_uri2] = $class2;
            //indicator
            $layer1[$class_uri]['#level_two'] = true;
          }
        }
      }
    }
    //http://tools.arlut.utexas.edu/gash2/doc/javadoc/arlut/csd/Util/VectorUtils.html
    
	  if ($climb && empty ($layer1)) {
       return $this->normalizeHierarchy($classes, $original_startclass, null);
    }
	  
    $final_array[$startclass] = $classes[$startclass];
    $final_array[$startclass]['children'] = $layer1;
    $final_array[$startclass]['classes_count'] = count($classes);
   
    return $final_array;
	}
	
  /**
   * provides example usage
   * run as SONIA_Sesame :: test();
   * 
   * WARNGING: USE ONLY ON EMPTY STORES
   * 
   * @param String uri the path to the server
   * @param String repo the repository name
   * 
   * */
  public static function test($context,$repo,$store_type ='memory' ,$debug=true){
  	
	    $myobject = new Java('SesamePHPModule');
			$myobject->SesamePHP_init($context,$repo,$store_type,$debug,'/home/clients/websites/w_ahirc/');
			$myobject->SesamePHP_test('<br/> TEST WORKING  <br/>');
			$values = $myobject->SesamePHP_simpleGetCount("SELECT DISTINCT ?x WHERE {?y ?x ?z . }");
			echo 'COUNT of '.$values.' DISTINCT TOTAL PROPERTIES<br />';
			echo '<table style="font-size: 10px;">';
			$values = $myobject->SesamePHP_simpleGet("SELECT DISTINCT ?x WHERE {?y ?x ?z . }", array('x'));
		  
		  $bindings["type"] = 'resource'; //or resource
			$bindings["value"] = '';
			$bindings["key"] = 'x';
			$totalofall1 = 0;
			$totalofall2 = 0;
			$max1 = 0;
			$max2 = 0;
			
			foreach($values as $key=>$value){
			  $bindings["value"] = $distinct_property = $value['x'];
			  $start1 = _microtime();
			  $simplecount = $myobject->SesamePHP_simpleGetCount("SELECT DISTINCT ?z WHERE {?y <$distinct_property> ?z . }");
			  $end1 = _microtime();
			  
			  $start2 = _microtime();
			  //TODO fix this
			  $prepared_count = $myobject->SesamePHP_preparedGetCount('PropertiesDistinctCount',"SELECT DISTINCT ?z WHERE {?y ?x ?z . }", array('z'), array($bindings));
			  $end2 = _microtime();
			  
			  //stats
			  $total1 = $end1-$start1;
			  if ($max1<$total1)  $max1 = $total1;
			  $totalofall1 += $total1;
			  $total2 = $end2-$start2;
			  if ($max2<$total2)  $max2 = $total2;
			  $totalofall2 += $total2;
			  
			  echo "<tr><td>$distinct_property </td><td> Simple $simplecount ($total1 seconds) </td><td> Prepared  $prepared_count ($total2 seconds)</td></tr>";
			}
			
			echo '</table>';
			echo "<hr/ ><hr/ > RESULTS || Simple $totalofall1 Slowest $max1 || Prepared  $totalofall2 Slowest $max2 <hr/ >";
  }
} //end class


function _microtime()
{
   list($usec, $sec) = explode(" ", microtime());
   return ((float)$usec + (float)$sec);
}

?>