<?php
/*
  (c) 2007 - Jonathan Hendler All rights Reserved
  SONIA Sesame is a HTTP client for Sesame using the Zend Platform Library

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

require_once 'Zend/Http/Client.php';
require_once 'Zend/Json.php';

/**
 * class SONIA_Sesame is an HTTP wrapper for querying a SESAME store. The store must 
 * already be set up, but data can be added, updated, removed, and queried.
 * 
 * This is only tested with Sesame 2.0 beta 5 as of this writing.
 * 
 * @version  
 * @package SONIA
 * @author  Jonathan Hendler [jonathan dot hendler (at) gmail dot com]
 * @license GPLv3 or higher
 *  
 * @link http://framework.zend.com the Zend Framework
 * @link http://www.openrdf.org sesame
 * @link http://www.openrdf.org/doc/sesame2/system/ch08.html the sesame HTTP protocol
 * @link http://framework.zend.com/manual/en/zend.http.html the Zend Framework HTTP client library
 * @link http://www.openrdf.org/doc/sesame2/users/ch05.html#d0e1147 a SeRQL reference page
 * @link http://www.openrdf.org/forum/mvnforum/viewthread?thread=498 why no ORDER BY... yet
 * 
 * @todo clear context function (erase all)
 * 
 * */

class SONIA_Sesame{
  
  /**
   * the string representing the URI for the Sesame server
   * */
	private static $SESAME_SERVER_URI;
	
	/**
	 * The client object
	 * */
	private static $client;
	
	/**
	 * boolean, to show debugging
	 * */
	private static $debug;
	
	/**
	 * The response object
	 * */
	private $response;
	
	/**
	 * A string of the repository
	 * */
	private $repo;
	
	/**
	 * A string of the default context
	 * */
	private $context;
	
	/**
	 * the log function
	 * */
	private $log_function;
	
	/**
	 * constructor or setting up connection
	 * */
	public function __construct($sesame_server, $context, $repo = 'SYSTEM', $caucho_url= null, $debug = false, $log_function = 'error_log'){
	  $this->SESAME_SERVER_URI = $sesame_server;
	  $this->debug = $debug;
	  $this->repo = $repo;
	  $this->context = $context;
	  $this->log_function = $log_function;
	  
	  if (!class_exists('Zend_Http_Client')) {
	  	die ('You must add the <a href="http://framework.zend.com">Zend Framework library</a> to your php.ini include_path. ');
	  }
    
    if ($caucho_url != null){
    	 //a specialized webservice
    	 //print '??';
    	 $this->caucho_url = $caucho_url;
       $this->caucho_client = new Zend_Http_Client($caucho_url, array(
		    'maxredirects' => 0,
		    'timeout'      => 30,
		    'keepalive' => true,
		    'httpversion' => '1.1' ));
    }
	  
	  //print $this->SESAME_SERVER_URI ;
	  
	  $this->client = new Zend_Http_Client($this->SESAME_SERVER_URI , array(
	    'maxredirects' => 0,
	    'timeout'      => 30,
	    'keepalive' => true,
	    'httpversion' => '1.1' ));
	  
	  if ($this->debug){
	    if ($this->client ) print '$this->client created.';
	    if ($this->caucho_client)print '$this->caucho_client created.';
	  }
	  //session_start();
	   
	  if (isset($_SESSION['cookiejar']) && ( $_SESSION['cookiejar']  instanceof Zend_Http_CookieJar)) {
		  $this->client->setCookieJar($_SESSION['cookiejar']);
		} else {
	    $this->client->setCookieJar();
	  }
	}
	
	public function __destruct(){
	  // Store cookies in session, for next page
    $_SESSION['cookiejar'] = $this->client->getCookieJar();
	}
	
	
	public function getRepositoryList(){
	  $args['path'] = '/repositories';
	  return $this->get($args);
	}
	
	public function getProtocol(){
	  $args['path'] = '/protocol';
	  return $this->get($args);
	}
	
	public function getContexts(){
	  $args['path'] = '/repositories/'.$this->repo.'/contexts';
	  return $this->get($args);
	}
	
	public function getRepositoryStatements(){
	  $args['path'] = '/repositories/'.$this->repo.'/statements';
	  return $this->get($args);
	}
	
	public function getRepositorySize(){
	  $args['path'] = '/repositories/'.$this->repo.'/size';
	  return $this->get($args);
	}
	
	/**
	 * */
  public function store_exists(){
    return true;
  }	 
	/**
	 * purely for debugging
	 * */
	public function getQueryForm(){
	  $html = '<form action="'.$this->SESAME_SERVER_URI.'/repositories/'.$this->repo.'" method="post">';
	  $html .= '<textarea name="query" rows="10" cols="80">';
	  $html .= 'SELECT DISTINCT ?p WHERE {?s ?p ?o . }';
	  $html .= '</textarea>';
	  $html .= '<input type="hidden" name="queryLn" value="sparql" />';
	  $html .= '<input type="hidden" name="infer" value="false" />';
	  $html .= '<input type="submit" name="submit" value="submit" />';
	  $html .= '</form>';
	  
	  return $html;
	}
	
	/***
	 * 
	 * */
	private function createTransactionStatement($statement,$type = null){
	  $xml = "\n";
	  
	  //subject
	  if ($statement['bnode'] && isset($statement['s'])){
	    $xml .= '<bnode>'.$statement['s'].'</bnode> <!-- subject -->'."\n";
	  } else if (!isset($statement['s'])){
	    $xml .= '<null /> <!-- subject -->'."\n";
	  } else if(isset($statement['s'])){
	    $xml .= '<uri>'.$statement['s'].'</uri> <!-- subject -->'."\n";
	  }
	  
	  //predicate
    if (!isset($statement['p'])){
	    $xml .= "\n".'<null />  <!-- predicate -->'."\n";
	  } else if(isset($statement['p'])) {
	  	$xml .= '<uri>'.$statement['p'].'</uri> <!-- predicate -->'."\n";
	  }
	  
	  //object
	  if ($statement['is_literal'] && isset($statement['o'])){
	  	$extra = (isset($statement['o_extra'])) ? $statement['o_extra']:'xml:lang="en"';
	    $xml .= '<literal '.$extra.' >'.$statement['o'].'</literal> <!-- object -->'."\n";
	  } else if(isset($statement['o'])){
	    $xml .= '<uri>'.$statement['o'].'</uri> <!-- object -->'."\n";
	  } else if (!isset($statement['o'])){
	    $xml .= "\n".'<null />  <!-- object -->'."\n";
	    //dbg($statement,'NULL FOUND in '.$type);
	  }
	  
	  $xml .= '<contexts>'."\n";
	  $xml .= '<uri>'.$this->context.'</uri>'."\n";
	  $xml .= '</contexts>'."\n";
	  return $xml;
	}
	
	/**
	 * 
	 * */
	private function encodeStatement($statement){
	  return rawurlencode(base64_encode(serialize($statement)));
	}
	
	/**
	 * $statements['clear'], $statements['add']
	 * */
	public function updateStatement($statements){
    
		$this->caucho_client->resetParameters();
  	$this->caucho_client->setUri($this->caucho_url);
  	$this->caucho_client->setMethod( Zend_Http_Client::POST);
  	//$statements_encoded = $this->encodeStatement($statements);
  	
  	$get['updateStatement'] = 'true';
    
  	if (isset($statements['clear'][0]['s'])) {
  		$get['s'] = rawurlencode($statements['clear'][0]['s']);
  	}
  	if (isset($statements['clear'][0]['p'])) {
  	  $get['p'] = rawurlencode($statements['clear'][0]['p']);
  	}
  	if (isset($statements['clear'][0]['o'])) {
  	  $get['o'] = rawurlencode($statements['clear'][0]['o']);
  	}
  	if (isset($statements['clear'][0]['is_literal'])) {
    	$get['is_literal'] = rawurlencode($statements['clear'][0]['is_literal']);
  	}	
  	
  	if (isset($statements['add'][0]['s'])) {
  		$get['as'] = rawurlencode($statements['add'][0]['s']);
  	}
  	if (isset($statements['add'][0]['p'])) {
  	  $get['ap'] = rawurlencode($statements['add'][0]['p']);
  	}
  	if (isset($statements['add'][0]['o'])) {
  	  $get['ao'] = rawurlencode($statements['add'][0]['o']);
  	}
  	if (isset($statements['add'][0]['is_literal'])) {
    	$get['ais_literal'] = rawurlencode($statements['add'][0]['is_literal']);
  	}
  	
  	$this->caucho_client->setHeaders($this->getQueryHeaders('application/sparql-results+json') );
  	$this->caucho_client->setParameterGet($get);
  	//$start = _microtime();
    $response = $this->caucho_client->request('GET');
    //$total_time = _microtime()-$start;
    // dbg($response,'QUERY TIME NO JSON: '.$total_time);
    $_SESSION['cookiejar'] = ($this->client->getCookieJar());
    //dbg($this->caucho_client);
    //dbg($statements);
    //dbg($response);
    return $this->parse_response($response,$statements, $statements);
	}
	
	/**
	 * $statements['clear'], $statements['add']
	 * */
	  /*
	public function updateStatement($statements){
		if (empty($statements['clear']) && empty($statements['add'])){
			print 'bad Structure for Statements';
			print_r($statements);
			return null;
		}
		
		
		
		
		
		
		$xml = '<?xml version="1.0"?>'."\n";
		$xml .= '<transaction>'."\n";
		
		if (!empty($statements['clear']) && is_array($statements['clear'])){
		  foreach($statements['clear'] as $statement){
		  	if (!empty($statement['s']) ||!empty($statement['p'])||!empty($statement['o']) ){
				  $xml .= '<remove>'."\n";
				  $xml .= $this->createTransactionStatement($statement,'remove');
				  $xml .= '</remove>'."\n";
				  dbg($statement,'<hr />REMOVE <br/>'.htmlentities($xml),true);
		  	} else {
		  	}
			}
		}
		
		if (!empty($statements['add']) && is_array($statements['add'])){
			foreach($statements['add'] as $statement){
				if ( (isset($statement['o'])) &&
				    (!empty($statement['s'])) && 
				    (!empty($statement['p'])) && 
				    ((!empty($statement['o'])) || ($statement['o'] == 0)) ){
					
				  $xml .= '<add>'."\n";
				  $xml .= $this->createTransactionStatement($statement,'add');
				  $xml .= '</add>'."\n";
				  dbg($statement,'<hr />ADD <br/>'.htmlentities($xml),true);
				} else {
				  //dbg($statement, 'CAUGHT NULL STATEMENT ON ADD');
				}
			}
		}
    $xml .= '</transaction>'."\n";
    return $this->sendTransaction($xml);
	}   
	*/
	
	public function deleteStatement($statements){
	  return $this->updateStatement($statements);
	}
	
	public function insertStatement($statements){
	  return $this->updateStatement($statements);
	}
	
	/**
	 * Sesame's transaction format
	 * 
	 * @param string $xml the xml
	 * @param boolean $final, if the cache should be updated 
	 * @todo add auth
	 * */
	private function sendTransaction($xml, $final=false){
		if (empty($xml)){
			dbg($xml,'Error: EMPTY sendTransaction()',true);
			return;
		}
		if($this->debug){
		  print '<br />Sent<br>'.nl2br(htmlentities($xml));
		}
		
	  $args['path'] = '/repositories/'.$this->repo.'/statements?context='.$this->context;
	  
	  $args['headers'] = array(
	    'Accept' => 'application/rdf+xml,text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5',
	    'User-Agent'=>'SONIA Sesame API',
      'Accept-Charset'=>'ISO-8859-1,utf-8;q=0.7,*;q=0.7',
	    'Connection' => 'keep-alive',
	    'Keep-Alive' => '6000', //TODO make config
	    'Accept-Encoding' => 'gzip,deflate',
	    'Content-Type'=>'application/x-rdftransaction',
	  );
	  
	  $args['type'] = 'POST';
	 
    if (!empty($xml)){
    	 $results = $this->get($args,$xml,'application/x-rdftransaction');
    	 //the apc cache needs to be cleaned on data update
    	 if ($results){
    	    $clear_cache = file_get_contents($this->caucho_url.'?clear_cache=1');
    	 }
    	 return $results;
    }
	}
	
	/**
	 * simple abstraction for compatibility with previous work.
	 * @param $args, and array
   * @return SimpleXMLElement Object
	 * */
	 public function query($args ){
      $result = $this->getQueryResult($args['query']);
      return $result;
	 }
	
	private function getQueryHeaders($accept = 'application/sparql-results+json, application/rdf+xml, */*'){
	  return array(
	    'Accept' => $accept,
	    'User-Agent'=>'Semantic Search: SONIA Sesame API',
      'Accept-Charset'=>'ISO-8859-1,utf-8;q=0.7,*;q=0.7',
	    'Connection' => 'keep-alive',
	    'Keep-Alive' => '6000', //TODO make config
	    'Accept-Encoding' => 'gzip,deflate',
	    'Content-Type'=>'application/x-www-form-urlencoded',
	  );
	}
	
	/**
	 * This function sends a query to the Sesame Server
	 * @todo add auth
	 * */
	public function getQueryResult($query, $type = 'sparql'){
		if ($this->debug){
      print 'called getQueryResult() with query '. $query;
		}
    
	  $args['path'] = '/repositories/'.trim($this->repo);
	  $args['headers'] = $this->getQueryHeaders('application/sparql-results+json');
	  
	  $args['type'] = 'POST';
	  $args['params'] = array(
      'query'=> $query,
      'infer'=> 'false',
      'queryLn'=> $type,
      'accept'=> 'application/sparql-results+json',
      );
	  
	  $result = $this->get($args,null,null);
     
    return $result;
	}
	
	/**
	 * an example SPARQL query
	 * */
	public function getDistinctPredicates(){
	  $query =   'SELECT DISTINCT ?p WHERE {?s ?p ?o . }';  
	  return $this->getAggregateQuery('getDistinctPredicates',$query);
	}
	
	/**
	 * prepare the args for sending
	 * */
	private function encodeAggregateQuery($args){
		$new_args = array();
		foreach($args as $key=>$value){
			if ($key == 'query'){
			  $new_args['query'] = rawurlencode($value); 
			} else  if ($key == 'children' && !empty($value)){
				foreach($value as $k=>$v){
				  $new_args[$key][$k] = $this->encodeAggregateQuery($v);
				}
			} else {
			  $new_args[$key] = $value; 
			} 
		}
		return $new_args;
	}
	
	/**
	 * this function polls the server for the 
	 * $params = array(),$count = array(),$order_by = array()
	 * 
	 * @note requires using sesame_resin
	 * 
	 * */
	public function getAggregateQuery($queryname,$query,$args){
	  if (isset($this->caucho_client)){
	  	$args['query'] = ($query); 
	  	$args = $this->encodeAggregateQuery($args);
	  	
	  	
	  	
	  /*	
	  	 * @param array args['query']
	 *                  ['queryname']
	 *                  ['params'] = array()
	 *                  ['order_by'][]['field'|'type'[num|alpha|alphai]|order=ASC|DESC]
	 *                  ['count'] true|false
	 *                  ['limit'][offset|limitnumber]
	 *                  ['inference']
	 *                  ['children'][]['query']
	 *                                ['queryname'] //also the return name
	 *                                ['params']
	 *                                ['bindings'][][parentparam|type|thisparam]
	 *                                ['order_by']
	 *                                [recurse] depth//in place of inference?
	 *                                ['count'] true|false
	 *                                ['inference']
	 *                                ['children']...
	 * 
	 * @return mixed result[rowindex][colname] = colvalue |
	 *                                           result[queryname] = colvalue
	  	*/
	  	$encoded_args = $args;
	  	$this->caucho_client->resetParameters();
	  	$this->caucho_client->setUri($this->caucho_url);
	  	$this->caucho_client->setMethod( Zend_Http_Client::POST);
	  	$this->caucho_client->setParameterPost(array('args'=>$encoded_args));
	  	$this->caucho_client->setHeaders($this->getQueryHeaders() );
	  	//$start = _microtime();
	    $response = $this->caucho_client->request('POST');
	    //$total_time = _microtime()-$start;
	    // dbg($response,'QUERY TIME NO JSON: '.$total_time);
	    $_SESSION['cookiejar'] = ($this->client->getCookieJar());
	    return $this->parse_response($response,$args, $query);
	  } else {
	  	if ($this->debug) {
	  	  dbg($args,'$args');
	  	}
	    return $this->getQueryResult($query);
	  }
	}
	
	/**
	 * a general function for polling the server
	 * */
	private function get($args = null,$raw = null, $raw_type = null){
		
		 
		$this->client->resetParameters();
//		if (isset($_SESSION['cookiejar']) && (($_SESSION['cookiejar']) instanceof Zend_Http_CookieJar)) {
//		  $this->client->setCookieJar(($_SESSION['cookiejar']));
//		} else {
//	    $this->client->setCookieJar();
//	  }
	  
		if (!isset($args['path'])) $args['path'] = '';
		if (!isset($args['type'])) $args['type'] = 'GET';
		//dbg($this->SESAME_SERVER_URI.$args['path'], 'fullPATH');
	  $this->client->setUri($this->SESAME_SERVER_URI.$args['path']);
		
	  if (isset($args['headers']) && is_array($args['headers'])){
	    $this->client->setHeaders($args['headers'] );
	    if (isset($args['headers']['Content-Type']) ){
	      $this->client->setEncType($args['headers']['Content-Type']);
	    }
	  }
	  
	  if($this->debug) echo $this->SESAME_SERVER_URI.$args['path'].'  <br /> ';
	  
	  if (isset($args['params']) && is_array($args['params'])  && ($raw === null)) {
		  switch($args['type']){
		  	case 'POST':
		  	  $this->client->setMethod( Zend_Http_Client::POST);
		  	  $this->client->setParameterPost($args['params']);
		  	  break;
		  	case 'GET':
		    default:
		      $this->client->setMethod( Zend_Http_Client::GET);
				  $this->client->setParameterGet($args['params']);
				  break;
		    }
	  }
		
		if ($raw === null){
			//$start = _microtime();
			$response = $this->client->request($args['type']);
      //dbg($response);
      //dbg($this->client);
			//$end = _microtime();
      //print'<hr/><hr/>'. ($end-$start).'<hr/><hr/>';
		} else {
			//Zend_Http_Client::setRawData($raw,$raw_type);
			$this->client->setRawData(trim(str_replace("\n",' ',$raw)));
			$this->client->setEncType($raw_type);
			if($this->debug) $time_start = _microtime();
			$response = $this->client->request($args['type']);
			if($this->debug) $time_end = _microtime();
			if($this->debug) print '<hr /> Speed of RawData submit '.($time_end-$time_start ).'<br />';
		}
		// Store cookies in session, for next page
    //$_SESSION['cookiejar'] = ($this->client->getCookieJar());
		return $this->parse_response($response,$args,$raw);
	}
	
	/**
	 * since PHP 5.2.x doesn't handles this yet
	 * */
	private function _semantic_search_json_decode($string, $trim = true){
//		if ($trim){
//		  $string = str_replace("\0", '\u0000',$string);
//		  $string = preg_replace("@([\1-\037])@e",
//		   "sprintf('\\\\u%04X',ord('$1'))",$string);
//		}
    if (strlen($string) > 0 && !empty($string) && $string != 'null') {
  	  if (function_exists('json_decode')) {
  	    return json_decode($string,true);
  	  }else {
  	    return  Zend_Json::decode($string);
  	  }
    } else {
      return '';
    }
	}
	
	/**
	 * 
	 * */
	private function parse_response(&$response,&$args,&$xml){
   //dbg($response);
	  if ($response->getStatus() == 200) {
        $response_converted = (function_exists('iconv'))? iconv("ISO-8859-1","UTF-8", $response->getBody()):$response->getBody();
	  		//dbg($response_converted);
        try{
	  		   $result =	 $this->_semantic_search_json_decode($response_converted); //make sure it is an array
	  		   if ((($result==="") || ($result===null) ) && ($response_converted !== 'false') && !empty($response_converted)){
	  		     $result = $this->_semantic_search_json_decode($response_converted,false);
	  		     if ((($result==="") || ($result===null)) && ($response_converted !== 'false')  && !empty($response_converted)){
	  		     	 // throw new Exception('Search results were not empty, but decode failed '. $response_converted);
	  		     }
	  		   }
	  		   return $result;
	  		} catch(Exception $e){
	  		   dbg($response_converted, "ERROR: in json_decode");
	  		}
 
		} else if ($response->isError()) {
		  $message =  "Semantic Search - SESAME: transmitting data.<br />";
		  $message .= "Server reply was: " . $response->getStatus() . " " . $response->getMessage() . "\n";
		  $message .=  '<br />';
		  $message .=  $response->getHeadersAsString();
		  $message .=  '<br />';
		  $message .=  $response->getBody();  
		  $function = $this->log_function;
		  dbg($args,'ERROR<br />'.htmlentities($xml));
		  $function($message,'error');
		} else if ($response->getStatus() == 204) {
			//the expected Semame response for sending CUD operations
			//What is code response code 204?
			//http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html
		  return true;
		}
	}
  
  /**
   * provides example usage
   * run as SONIA_Sesame :: test();
   * 
   * @param String uri the path to the server
   * @param String repo the repository name
   * 
   * */
  public static function test($uri,$context, $repo,$caucho_url,$debug=true){
	  ////////////
		// TEST CODE
		$context = 'http://semanticsearch.example.com/';
		$context = 'http://ahirc.org/';
		$sonia_sesame = new SONIA_Sesame($uri,$context,$repo,null,$debug); //debugging is on
		
		print 'starting tests.<br />';
		
		 
		print '<br/>PROTOCOL='. $sonia_sesame->getProtocol();
		$list = $sonia_sesame->getRepositoryList();
		print '<br/>REPO LIST = ';
		foreach( $list as $repo){
			print_r($repo);
			foreach($repo as $k=>$v){
			   print '<br/>'.$k.' = '.$v;  
			}
		
		}
		
		//print '<br/>REPO LIST ='. print_r($sonia_sesame->getRepositoryList(),1);
		//lists all statements
		//$sonia_sesame->getRepositoryStatements( );
		
		//add data
		$add_statements = array();
		$add_statements['add'][0]['s'] = $context.'#testSubject';
		$add_statements['add'][0]['p'] = $context.'#testPredicate';
		$add_statements['add'][0]['o'] = $context.'#testObject';
		
		
	  $sonia_sesame->updateStatement($add_statements);
		$query = 'SELECT ?s ?o WHERE {?s <'.$context.'#testPredicate'.'> ?o}';
		echo '<hr />ADD RESULT<br />'.print_r($sonia_sesame->getQueryResult($query),1);
		
		//add a literal
		$add_statements = array();
		$add_statements['add'][0]['s'] = $context.'#testSubject';
		$add_statements['add'][0]['p'] = $context.'#testPredicateLiteral';
		$add_statements['add'][0]['o'] = "hello";
		$add_statements['add'][0]['is_literal'] = true;
		$sonia_sesame->updateStatement($add_statements);
		

		
		$add_statements = array();
		$add_statements['clear'][0]['p'] = $context.'#testPredicateLiteral';
		$add_statements['add'][0]['s'] = $context.'#testSubject';
		$add_statements['add'][0]['p'] = $context.'#testPredicateLiteral';
		$add_statements['add'][0]['o'] = "hello update successful";
		$add_statements['add'][0]['is_literal'] = true;
		$sonia_sesame->updateStatement($add_statements);
		
	  $add_statements = array();
		$add_statements['add'][0]['s'] = $context.'#testSubject';
		$add_statements['add'][0]['p'] = $context.'#testPredicateLiteral';
		$add_statements['add'][0]['o'] = "hello again duplicate";
		$add_statements['add'][0]['o_extra'] = '   ';
		$add_statements['add'][0]['is_literal'] = true;
		$sonia_sesame->updateStatement($add_statements);
		
		
		$add_statements = array();
		$add_statements['add'][0]['s'] = $context.'#testSubject';
		$add_statements['add'][0]['p'] = $context.'#testPredicateLiteral';
		$add_statements['add'][0]['o'] = "hello again duplicate";
		$add_statements['add'][0]['is_literal'] = true;
		$sonia_sesame->updateStatement($add_statements);
		
		
		$add_statements = array();
		$add_statements['add'][0]['s'] = $context.'#testSubject2';
		$add_statements['add'][0]['p'] = $context.'#testPredicate';
		$add_statements['add'][0]['o'] = $context.'#testObject2';
		
	  $sonia_sesame->updateStatement($add_statements);
		$query = 'SELECT ?s ?o WHERE {?s <'.$context.'#testPredicate'.'> ?o}';
		echo '<hr />ADD Subject 2<br />'.print_r($sonia_sesame->getQueryResult($query),1);
		
		//will clear all with the specific subject
		$update_statements = array();
		$update_statements['clear'][0]['s'] = $context.'#testSubject2';
		
	  $sonia_sesame->updateStatement($update_statements);
		$query = 'SELECT ?s ?o WHERE {?s <'.$context.'#testPredicate'.'> ?o}';
		echo '<hr />Removed Subject 2<br />'.print_r($sonia_sesame->getQueryResult($query),1);
	
	
	  //will clear all with the subject
		$update_statements = array();
		$update_statements['add'][0]['s'] = $context.'#testSubject';
		$update_statements['add'][0]['p'] = $context.'#testPredicate';
		$update_statements['add'][0]['o'] = $context.'#testObjectUPDATED';
		
		$update_statements['clear'][0]['p'] = $context.'#testPredicate';
		
	  $sonia_sesame->updateStatement($update_statements);
		$query = 'SELECT ?s ?o WHERE {?s <'.$context.'#testPredicate'.'> ?o}';
		echo '<hr />UPDATE RESULT<br />'.print_r($sonia_sesame->getQueryResult($query),1);	
		//print_r($sonia_sesame->getDistinctPredicates( ));
		print'<hr />';
		print_r($sonia_sesame->getAggregateQuery('SELECT DISTINCT ?p WHERE {?s ?p ?o . }'));
		print'<hr />';
		echo $sonia_sesame->getQueryForm( );
  }
} //end class

if (!function_exists('_microtime')){
	function _microtime()
	{
	   list($usec, $sec) = explode(" ", microtime());
	   return ((float)$usec + (float)$sec);
	}
}



?>