<?php

require 'sonia.sesame.class.inc';

/**
 * SONIA
 * Simple Ontology aNd Inference Algorithms
 * 
 * The SONIA Store API provides an internal API for SONIA API to interact with a 
 * an RDF store, Triple Store or other store
 * 
 * @author  Jonathan Hendler 
 * @package SONIA
 * @license GPLv2
 * @license copyright 2006 KBCMS LLC all other uses
 * @version .01 alpha
 * @date    8/12/2007
 * 
 * 
 * */
class SONIASesameStore implements SONIAStoreAPI {

  private $data_api;
  public $debug_text;
  private $dosearch;

  //INITIALIZATION
  //--------------------------------------------------------------------------
  /**
   *  The conctructor provides a mechanism for instantiating the data store,
   * and loading data at runtime if necessary
   * 
   * @access public
   * @see instantiateBackend()
   * @see loadData
   * 
   * @todo add $searchname as an optional prepend, to allow search by search configurations
   * */
  //--------------------------------------------------------------------------  
  public function __construct() {
  	$args = array();
  	$server = variable_get('sesame_store_url',null);
  	$repo = variable_get('sesame_repository_name',null);
  	
  	if (empty($server)){
  		$adminurl = url('admin/semantic_search/config');
  	  drupal_set_message('Please configure the Sesame store at <a href="'.$adminurl.'">'.$adminurl.'</a>','error');
  	}
  	
  	if (empty($repo)){
  		$adminurl = url('admin/semantic_search/config');
  	  drupal_set_message('Please configure the Sesame repository at <a href="'.$adminurl.'">'.$adminurl.'</a>','error');
  	}
  	
  	//TEMP, get from configs
  	$args['sesame_server'] = $server;
    $args['sesame_context'] = variable_get('_BASE_NAMESPACE_URL','');
    $args['sesame_repo'] = $repo;
    $args['sesame_caucho_url'] = variable_get('sesame_caucho_url',null);
    
    $this->init($args);
    $this->debug_text = '';
    $this->cache = new SO_CA();
    $this->dosearch = false;
  }
  
  //--------------------------------------------------------------------------
  /**
   * Parse and check params set to create/ login to the store.
   * 
   * @param array $args an array of arguments needed for the store. 
   * eg. for mysql, db name, username, password etc 
   * 
   * @todo login/auth
   * */
  //--------------------------------------------------------------------------
  public function init($args) {
     $this->data_api = new SONIA_Sesame(
       $args['sesame_server'], 
       $args['sesame_context'], 
       $args['sesame_repo'],
       $args['sesame_caucho_url'],
       false,
       'drupal_set_message' );
  }

  /**
   *  RDF into the rdf store 
   * 
   * TODO needs merging configuriation
   * 
   * @todo validate the RDF
   * @param string $iri_model the name of the model
   * @param string $file the location of the file, can be any uri PHP allows http:// file:/// or just a path
   * */
  public function importRDF($iri_model, $file) {
 
  }

  /**
   * @todo limit the export to a specific subgraph or model
   * */
  public function exportRDF($iri_model) {
    return $this->exportNative(true);
  }

  /**
   * Exports given data to rows.
   * */
  public function exportNative($rdf = false) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'DESCRIBE ?x ?y ?z
                                        WHERE 
                                        {
                                          ?x ?y ?z . 
                                        }';

    if ($rdf) {
      $q_result = $this->data_api->query(array (
        "query" => $query
      ));
    } else {
      $q_result = $this->data_api->query(array (
        "query" => $query,
        "result_type" => "rows"
      ));
    }

    if (empty ($q_result["result"]) && SEMANTIC_SEARCH_DEBUG) {
      // print_r($q_result);
    } else {
      return $q_result["result"];
    }
  }

  //--------------------------------------------------------------------------
  /**
   * Parse and check params set to create/ login to the store.
   * 
   *  //TODO following code should be in the import script
   *  // THese files should be moved to config and loaded only once.
   * @access private
   * @todo move all this to config
   *  TEMPORARY FUNCTION
   * 
   * 
   * //HACK
   * */
  //--------------------------------------------------------------------------  
  private function loadData() {
     

  }


  
  public function maintain($args) {
 
  }

  /**
   * simple wrapper for ARC function
   * */
  public function removeTables($message_function_name = null) {
    $message = 'SEMANTIC_SEARCH: store REMOVED';
    $this->printMessage($message_function_name, $message);
  }

  public function truncateTables($message_function_name = null) {
    $message = 'SEMANTIC_SEARCH: store tables erased';
    $this->printMessage($message_function_name, $message);
  }
  //--------------------------------------------------------------------------
  /**
   * Installs the ARC backend
   * eg. used by Drupal installers
   * 
   * */
  //--------------------------------------------------------------------------
  public function installBackend($message_function_name = null) {
    if ($this->data_api->store_exists()) {
     
    } else {
      $message = 'SEMANTIC_SEARCH: You will need to install Sesame independatly.';
      $this->printMessage($message_function_name, $message);
    }
  }

  public function printMessage($message_function_name, $message) {
    if (($message_function_name != null) && (function_exists($message_function_name))) {
      $message_function_name ($message);
    }
  }
  
  /**
   * get data about an individual object
   * */
  private function getObjectData($object_uri){
    $sparql = ARC_DEFAULT_PREFIXES . "\n";
    $sparql .= 'SELECT DISTINCT ?p ?p_l ?o ?o_l ?s_l ?fname ?fpath
                                        WHERE { 
                                          "'.$object_uri.'" ?p ?o . 
                                          OPTIONAL {?p ' . SONIA_RDFS_LABEL . ' ?p_l }.
                                          OPTIONAL {?o ' . SONIA_RDFS_LABEL . ' ?o_l }.
                                          OPTIONAL {"'.$object_uri.'" ' . SONIA_RDFS_LABEL . ' ?s_l }. 
                                          OPTIONAL {?o <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_FILE_CLASS_URI . '> .  
                                                    ?o <' . SONIA_CREATE_FILENAME . '> ?fname . 
                                                    ?o <' . SONIA_CREATE_FILEPATH . '> ?fpath .       }   
                                        }';
                                        
    return $this->cachedQuery($sparql);
  }
  
  /**
   * get data about an individual object
   **/
  private function getObjectsOfParam($pro,$type = 'p_value'){
    if ($type == 'p_value'){
      $count_query = ARC_DEFAULT_PREFIXES;
      $count_query .= 'SELECT DISTINCT ?object WHERE {?object <' . $property . '> "' . $row['value'] . '" .}';
      $resulta = $this->cachedQuery($count_query);
    } else {
      $p = ' <'.$key.'> ';
      $o = ' ?o ';
    }
    $sparql = ARC_DEFAULT_PREFIXES . "\n";
    $sparql .= 'SELECT DISTINCT ?s  
                                        WHERE { 
                                          ?s '.$p.$o.'. 
                                          FILTER(REGEX(str(?s),"^'.SONIA_OBJECT_ID_URI.'")) .
                                        }';
                                        
    return $this->cachedQuery($sparql);
  }
  
  //RETRIEVAL
  //--------------------------------------------------------------------------
  /**
   * Parse the query and send a generalized query
   * In this case to sparql
   * 
   * for now, ?p and ?o get ignored for ?s searches and ?s is quite slow because it's
   * a filter match
   *  ['order_by'][]['field'|'type'[NUMERIC|STRING|REGULAR]|order=ASC|DESC]
   * 
   * @param array $query, the statement sent via SONIA
   * @todo for now just uses defaultns
   * @todo speed up the query of objects where an ID is given, no filter
   * @todo for now the subject based query only accepts URIs
   * 
   * 
   * @todo pass in OPTIONAL flag
   *
   * 
   * @return string $convertedQuery 
   * @see SONIA :: UI_query()
   * @see query();
   * */
  //--------------------------------------------------------------------------
  public function convertQuery($query) {
    //dbg($query,'$query in convertQuery()');
    $user_id = $query['user_id'];
    $statement_sets = $query['statement_sets'];
    $limit = $query['limit'];
    $offset = $query['offset'];
    $any = $query['any'];
    $distinct = $query['distinct'];
    $describe = $query['describe'];
    $this->order_by = null; //['order_by'][]['field'|'type'[NUMERIC|STRING|REGULAR]|order=ASC|DESC]
   //add order by params needed for sorting
    
    
    //NOTE: made filters last for speed
    //$sparql = ARC_DEFAULT_PREFIXES . "  \n ";
    $sparql = ' SELECT DISTINCT ?object %s WHERE {
                    ?object ?p ?o .
                    
                    %s
                    %s  
                    FILTER(REGEX(str(?object),"^'.SONIA_OBJECT_ID_URI.'")) .
                  } ';
    $statement_blocks = '';
    $text_blocks = '';
    $matches = '';
    $text_block_count = 0;

    //default used later for contructing 
    $text_block = '  %s %s ?x%s . FILTER(REGEX(str(?x%s), "%s")) . ' . "   ";
    $exact_block = '  %s %s "%s" . ' . "  ";
    $begin_block = '  %s %s ?x%s . FILTER(REGEX(str(?x%s), "(^%s)")) . ' . "   ";

    $statement_block = '  %s %s %s . ' . "   ";

    //TODO SLOW
    $subject_block = ' FILTER(REGEX(str(?object), "%s")) . ' . "\n";
    $this->order_by = array();
    	
    $lcount = 0;
    $vcount = 0;
    if (is_array($statement_sets)) {
      foreach ($statement_sets as $q) {
      	
      	//if order by exists add it as a regular search, but include it in the results
      	//and add it to the params
      	$order_by = (!empty($q['order_by'])) ? $q['order_by'] : null;
       //['order_by'][]['field'|'type'[NUMERIC|STRING|REGULAR]|order=ASC|DESC]
      	$optional = (!empty($q['optional']));
      	
      	
      	
        if (!empty ($q['subject'])) {
          $s = '<' . $q['subject'] . '>';
          $p = ' ?p ';
          $o = ' ?o ';
        } else {
          $s = ' ?object ';

          if (!empty ($q['object'])) {
            $o = '<' . $q['object'] . '>';
            if ($q['literal_object']) {
              $lcount++;
              $o = $q['object'];
            }
          } else {
            $o = ' ?o' . $vcount . ' ';
            $vcount++;
          }

          if (!empty ($q['predicate'])) {
            $p = (isset ($q['category_query']) && ($q['category_query'])) ? '<'.SONIA_RDF_CREATE_TYPE.'>' : '<' . $q['predicate'] . '>  ';
          } else {
            $p = ' ?p' . $vcount . ' ';

            $vcount++;
          }

        } //end subject defined
        
        $temp_block = '';
        
        /// complete parsing of line
        if ($q['literal_object']) {

          if (!isset($q['literal_type'])) $q['literal_type'] = 'exact';
          

          //check to see if there are non standard charactors, if so, change query type
          if ($q['literal_type'] == 'exact' && preg_match("/([\W]+)/", $q['object'], $match)) {
            $q['literal_type'] = 'begin';
          }

          switch ($q['literal_type']) {
            case 'exact' :
             $temp_block = sprintf($exact_block, $s, $p, $q['object']);
            // $text_blocks .= sprintf($begin_block, $s, $p,$lcount, $lcount, $q['object']);
              break;
            case 'begin' :
              $temp_block = sprintf($begin_block, $s, $p, $lcount, $lcount, $q['object']);
              break;
            case 'fulltext' :
            default :
              $temp_block = sprintf($text_block, $s, $p, $lcount, $lcount, $q['object']);
              break;
          }
          $this->setOrderBy('x'.$lcount,$order_by);
          
        } else if (!empty ($q['subject'])) {
            //$subject_block
            $s = '' . $q['subject'] . '';
            $temp_block = sprintf($subject_block, $s);
            $s = '<' . $s . '>';
            $temp_block = sprintf($statement_block, $s, $p, $o);
            $this->setOrderBy($o,$order_by);
        } else {
        	$this->setOrderBy(str_replace('?','',$o),$order_by);
          $temp_block = sprintf($statement_block, $s, $p, $o);
        }
          
        $statement_blocks .= ($optional) ? ' OPTIONAL { '.$temp_block.' } ':$temp_block;
      }
    }
    
    $params = '';
    foreach($this->params as $pvalue){
      if ($pvalue !='object'){
        $params .= ' ?'.trim($pvalue);
      }
    }
     
    
    $sparql = sprintf($sparql, $params, $statement_blocks, $text_blocks);
//    dbg(  $this->order_by,'  $this->order_by');
//    dbg(  $this->params,'  $this->params');
//    dbg(htmlentities($sparql),'$sparql');
    return $sparql;
  }

   private function setOrderBy($param,$order_by){
      //prep order by for passing to sesame
      if ($order_by){
      	$order_by['field'] = trim($param);
    	  $this->params[] = trim($param);
    	  $this->order_by[] = $order_by;
    	}
    }

  //--------------------------------------------------------------------------
  /**
   * Count the number of literals which have a give value
   * 
   * predicates and subjects may include the Namespace prefix or the entire
   * statement (in the case of SPARQL this should include "<" and ">"
   * 
   * @param string value - the literal value
   * @param string ns_predicate - the predicate (with the namespace)
   * @param string ns_subject - the subject (with the namespace)
   * @param array $args an array of arguments needed for the store. 
   * eg. for mysql, db name, username, password etc 
   * 
   * */
  //--------------------------------------------------------------------------

  public function getLiteralCount($ns_predicate, $ns_subject, $value) {

    $query = ARC_DEFAULT_PREFIXES;
    $query .= ' SELECT ?x 
                                       WHERE {
                                         ' . $ns_subject . ' ' . $ns_predicate . ' ?x .
                                          FILTER(REGEX(?x, "' . $value . '"))
                                       }
                                       ';
    $rows = $this->cachedQuery($query);
    return count($rows);
  }
  
  
  
  

  //--------------------------------------------------------------------------
  /**
   * Parse the query and send a generalized query to the store
   * 
   * @param string $ns_predicate use a ns from defaults like semantic_search:zip
   * @param string $ns_subject optional subject
   * 
   * @return array 'values' 'count'
   * 
   * */
  //--------------------------------------------------------------------------    
  public function getAllLiteralsAndCount($ns_predicate, $ns_subject) {
    if (empty ($ns_subject))
      $ns_subject = '?foo';

    $query = ARC_DEFAULT_PREFIXES;
    $query .= ' SELECT DISTINCT ?x 
                                       WHERE {
                                         ' . $ns_subject . ' ' . $ns_predicate . ' ?x .
                                       }
                                       ORDER BY ASC (?x)';

    $args = array (
      'query' => $query,
      'result_type' => ARC_DEFAULT_RESULT_TYPE,
        // "result_type_args"=>array("jsoni"=>$jsoni)

  
    );

    $q_result = $this->data_api->query($args);
    $results = $q_result["result"];
    $final_results = array ();
    $count = 0;

    if (is_array($results)) {
      foreach ($results as $result) {
        $final_results[$count]['value'] = $result['x'];
        $final_results[$count]['count'] = $this->getLiteralCount($ns_predicate, $ns_subject, $result['x']);
        $count++;
      }
    }

    return $final_results;
  }

  //--------------------------------------------------------------------------
  /**
   * takes a stores search results and parses it to sonia search results.
   * 
   * RESULTS:
   *     $new_record['o'] =  $object;
         $new_record['p'] =  $predicate;
         $new_record['p_l'] = $predicate_label;
         
         $search_result[$subject][SONIA_INSTANCE][$count]['t']   = $type;
         $search_result[$subject][SONIA_INSTANCE][$count]['t_l'] = $type_label;
         $search_result[$subject][$count] = $new_record;
   * 
   * @param array results from some format
   * @return array a standard SONIA results array
   * @todo shouldn't $subject be unique?
   * */
  //--------------------------------------------------------------------------
  public function parseStoreToSonia(&$results) {
    $search_result = array ();
    $count = 0;
    //parse the result into consistant form
    foreach ($results as $result) {
      $new_record = array ();

      $subject = $result['s'];
      $predicate = $result['p'];
      $object = $result['o'];

      $new_record['o'] = $object;
      if (isset ($result['o_l']))
        $new_record['o_l'] = $result['o_l'];
      $new_record['p'] = $predicate;
      if (isset ($result['p_l']))
        $new_record['p_l'] = $result['p_l'];
      if (isset ($result['s_l']))
        $new_record['s_l'] = $result['s_l'];
      if (!empty ($result['fname']))
        $new_record['fname'] = $result['fname'];
      if (!empty ($result['fpath']))
        $new_record['fpath'] = $result['fpath'];

      $search_result[$subject][$count] = $new_record;

      ++$count;
    }

    return $this->cleanBadResults($search_result);
  }
  
  
  
  /**
   * encapsulation for caching a query
   * */
  public function cachedQuery($query, $return_type = 'rows',$queryname = '',$caucho= null) {
  	global $semantic_search_total_queries;
  	if (empty($semantic_search_total_queries))
  	  $semantic_search_total_queries = 0;
  	
    $search_hash = 'searchHash' . md5($query.$return_type);
    global $sparql_execution_time;
    
    $rows = (SONIA_USE_CACHE) ? $this->cache->getCache($search_hash, true, true) : null;
    
    if (SONIA_USE_CACHE && $rows !== null) {
        
      return $rows;
    } else {
    	if ($caucho && is_array($caucho)) {
    		//NOTE: set to true for stats
    		if (!isset($caucho['params'])) return null;
    		if (!isset($caucho['count'])) $caucho['count'] = false;
    		if (!isset($caucho['limit'])) $caucho['limit'] = null;
    		if (!isset($caucho['inference'])) $caucho['inference'] = false;
    		if (!isset($caucho['children'])) $caucho['children'] = null;
    		if (!isset($caucho['order_by'])) $caucho['order_by'] = null;
    		
        //set to true for debugging
    		if (false){
    			++$semantic_search_total_queries;
    			$start = _microtime();
    		  $results = $this->data_api->getAggregateQuery($queryname,$query,$caucho);
  	      $total_time = _microtime()-$start;
  	      dbg(htmlentities($query),'Query number '.$semantic_search_total_queries.' in '.$results['query_time'].' of '.$total_time);
  	      //dbg($results);
  	      //print nl2br(print_r( debug_backtrace(),1));
  	      return $results;
    		} else {
    	    return $this->data_api->getAggregateQuery($queryname,$query,$caucho);
    		}
    	} else {
    		//++$semantic_search_total_queries;
    		//dbg(htmlentities($query),'Query number '.$semantic_search_total_queries.' in '.$results['query_time'].' of '. $results['total_time']);
	      $sparql_xml_object = $this->data_api->query(array (
	        "query" => $query,
	        "result_type" => $return_type
	      ));
        
        
          /*
        if (!is_array($sparql_xml_object)){
          $text_str = '';
          $chars = explode("\n", chunk_split(str_replace("\n", '', $sparql_xml_object), 8));
          $_I = count($chars);
          for($i = 0; $i < $_I; $text_str .= chr(bindec($chars[$i])), $i  );
          $sparql_xml_object = json_decode($text_str);
        }
              
            $text_str = '';
      $chars = explode("\n", chunk_split(str_replace("\n", '', $bin_str), 8));
      $_I = count($chars);
      for($i = 0; $i < $_I; $text_str .= chr(bindec($chars[$i])), $i  );
    return $text_str; 
      
        //if this is a binary format, convert with iconv
        
           $sparql_xml_object  =  iconv  ( "ISO-8859-1"  , 'UTF-8' , $sparql_xml_object);
           
        }
        
        */ 
        
        //now changed to http://www.w3.org/2005/sparql-results#

//           
//  	      if (SONIA_USE_CACHE){
//  	        //save the result, if it is rows, then it is an array
//  	        $this->cache->putCache($search_hash, $rows, true, ($return_type == 'rows'));
//  	        //also cache the count
//  	        if ($return_type == 'rows'){
//  	          $this->addCachedCountQuery($query,count($sparql_xml_object->results->result->binding));
//  	        }
//  	      }
//        }
	              return $sparql_xml_object;
    	}
    }
  }

  /**
   * encapsulation for caching a query
   * */
  public function cachedCountQuery($query) {
    return $this->cachedQuery($query, 'row_count');
  }
  
  private function addCachedCountQuery($query,$count){
     $search_hash = 'searchHash' . md5($query.'row_count');
     $this->cache->putCache($search_hash, $count, true);
  }

  /**
   *  provide debugging output
   * */
  public function queryDebug($query, $print = true) {
    $args = array (
      'query' => $query,
      'result_type' => 'sql',
    );

    $q_result2 = $this->data_api->query($args);
    $this->debug_text .= '<hr>';
    //$this->debug_text .= nl2br(htmlentities(print_r($query, 1)));
    //$this->debug_text .= nl2br(htmlentities(print_r($q_result2, 1)));
    //$this->debug_text .= nl2br(htmlentities(print_r($this->data_api->get_errors(), 1)));
    //$this->debug_text .= '<hr>';
    $this->debug_text .= '<br> ' . $q_result2['result'] . '<br>';
    if ($print)
      dbg($this->debug_text, '$this->debug_text');
  }

  /**
   *  provide debugging output
   * */
  public function queryCountDebug($query) {
    $args = array (
      'query' => $query,
      'result_type' => 'row_count',

      
    );

    $q_result2 = $this->data_api->query($args);
    $this->debug_text .= '<hr>';
    $this->debug_text .= nl2br(htmlentities(print_r($query, 1)));
    $this->debug_text .= nl2br(htmlentities(print_r($q_result2, 1)));
    $this->debug_text .= nl2br(htmlentities(print_r($this->data_api->get_errors(), 1)));
    $this->debug_text .= '<hr>';
    $this->debug_text .= '<br> SQL ' . $q_result2['result'] . '<br>';
  }

  //--------------------------------------------------------------------------
  /**
   * Parse the query and send a generalized query to the store
   * 
   * @see convertQuery();
   * @see SONIA :: UI_query()
   * @see queryDebug()
   * 
   * @return array results and result codes to be used by SONIA
   * 
   * */
  //--------------------------------------------------------------------------
  public function query($query, $rdf = false, $convert_query = true) {
    //set default offset
    $_semantic_search_pagination_offset = (empty($_semantic_search_pagination_offset)) ? 0 : $_semantic_search_pagination_offset;
    $search_result = array ();
    
    
    $this->params =  array('object');
    
    if ($convert_query)
      $query = $this->convertQuery($query);
    
    $child = array();
    $child['params'] = array('p','p_l','o','o_l','s_l','fname','fpath');
    // bindings from the parent array and to the subquery
    $binding[0]['key'] = 'object';
    $binding[0]['type'] = 'resource';
    $binding[0]['value'] = '';
    $child['bindings'] = $binding;
    //$child['count'] = true;
    //the return value name
    $child['binding_field_name'] = 'data';
    $child['prepare_view'] = true;
    
    $_strict = (false) ? ' FILTER(REGEX(str(?s),"^'.SONIA_OBJECT_ID_URI.'")) .' :'';
   // $count_query = ARC_DEFAULT_PREFIXES;
    $object_query = 'SELECT DISTINCT ?p ?p_l ?o ?o_l ?s_l ?fname ?fpath
                                        WHERE { 
                                          ?object ?p ?o . 
                                          OPTIONAL {?p <' . SONIA_RDFS_CREATE_LABEL . '> ?p_l }.
                                          OPTIONAL {?o <' . SONIA_RDFS_CREATE_LABEL . '> ?o_l }.
                                          OPTIONAL { ?object <' . SONIA_RDFS_CREATE_LABEL . '> ?s_l }. 
                                          OPTIONAL {?o <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_FILE_CLASS_URI . '> .  
                                                    ?o <' . SONIA_CREATE_FILENAME . '> ?fname . 
                                                    ?o <' . SONIA_CREATE_FILEPATH . '> ?fpath .       }   
                                        }';
    $child['query'] = $object_query;
    $child['queryname'] = 'objectQueryChild';
    $child['offset'] = SONIA_IO :: getPaginationBegin();
    $child['limit'] = SEMANTIC_SEARCH_DISPLAY_PAGINATION;
    
//    $this->object_search_result_ids = array();
//    if (is_array($results)){
//      foreach($results as $data){
//        $this->object_search_result_ids[] = $data['s'];
//      }
//    }
//    
//    
 
    //set up get file info
    
    $args['params'] = $this->params;
    $args['order_by'] = $this->order_by;
    $args['main_query'] = session_id();
    $this->dosearch = true;
    $args['children'] = array($child);
    $results = ($query != '') ? $this->cachedQuery($query,'rows','objectQuery',$args) : array();
    //count here if needed
    //remap
    $final_results = array();
    $this->object_search_result_count = count($results);
    if (!empty($results)){
	    foreach($results as $result){
	      if (!empty($result['data'])){
	        $final_results[$result['object']]  = $result['data'];
	      }
	    }
    }
    return $final_results;
  }

  //--------------------------------------------------------------------------
  /**
   * TODO DEPRICATE
   * setup to clean results that may be undesirable
   * */
  //--------------------------------------------------------------------------
  public function cleanBadResults($search_result) {
    //TODO HACK
    //remove bad incomplete data
    foreach ($search_result as $key => $result) {
      //HACK fix for "bad" results where we would expect more than 3 fields of data
      if (count($result) < 2)
        unset ($search_result[$key]);
    }
    return $search_result;
  }

  //--------------------------------------------------------------------------
  /**
   * get all the classes which a given object belongs to
   * */
  //--------------------------------------------------------------------------
  public function getAllClassesOfObject($object_uri) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= '
                                    SELECT DISTINCT ?sc ?sc_label
                                      WHERE {
                                        <' . $object_uri . '> <' . SONIA_RDF_CREATE_TYPE . '> ?sc .
                                         ?sc   rdfs:label ?sc_label  .             
                                      }
                                  ';
    $rows = $this->cachedQuery($query);
    $row_count = count($rows);

  }

  /**
   * get all classes and their subclasses
   * @param boolean $root_given was the start class properly provided as a URI
   * 
   * @todo rename
   * */
  public function getAllCategoriesRecursive($startclass = null, $depth = 0, $startclass_label = '') {
    $query = ARC_DEFAULT_PREFIXES;

    if ($depth > 0 || ($startclass != null)) //get derived classes
      {
      //<'.$startclass.'> rdfs:label ?label .
      $query .= '
                                              SELECT DISTINCT ?sc ?sc_label 
                                                WHERE {
                                                  ?sc rdfs:subClassOf <' . $startclass . '> .
                                                  ?sc a rdfs:Class .    
                                                  ?sc rdfs:label ?sc_label  
                                       
                                                }
                                            ';

    } else //get root classes which are not subclasses of anything
      {
      $query .= '
                                              SELECT DISTINCT ?label  ?x ?y
                                                WHERE {
                                                  ?x a rdfs:Class .
                                                  ?x rdfs:label ?label .
                                                  OPTIONAL 
                                                  {
                                                    ?x  rdfs:subClassOf ?y .
                                                  }            
                                                }
                                            ';
    }

    $rows = $this->cachedQuery($query);
    $row_count = count($rows);
    $final_array = array ();
    $count = 0;

    if (($row_count > 0) && ($startclass != null)) {
      $final_array[$startclass]['label'] = $startclass_label;

      for ($i = 0; $i < $row_count; $i++) {
        if (isset ($rows[$i])) {
          $row = $rows[$i];

          $subclass = $row["sc"];
          //$label       = $row["label"];
          $sc_label = $row["sc_label"];

          $subclasses = $this->getAllCategoriesRecursive($subclass, $depth +1, $sc_label);

          if (!empty ($subclasses))
            $final_array[$startclass]['subclasses'][] = $subclasses;
        }
      }
    } else
      if ($row_count > 0) //parse root classes
        {
        for ($i = 0; $i < $row_count; $i++) {
          //must not be a subclass of anything else
          $row = (!empty ($rows[$i])) ? $rows[$i] : null;

          //echo '<br />$row["x"] '.$row["x"];

          if (empty ($row["y"])) {
            $_startclass = (isset ($row["x"])) ? $row["x"] : '';
            $label = (isset ($row["label"])) ? $row["label"] : ''; //$row["label"];
            $final_array[] = $this->getAllCategoriesRecursive($_startclass, $depth +1, $label);
          }
        }
      }
    //       //
    return $final_array;
  }

  //--------------------------------------------------------------------------
  /**
  * gets all the categories/classes and counts available for a search
  * 
  * @todo only works for 3 levels deep - should be a recursive function
  * @author 
  * @param 
  * */
  //--------------------------------------------------------------------------
  public function getAllCategories(& $count, $startclass = 'semantic_search:Space', $in = null) {
    //?x_label
    $query = ARC_DEFAULT_PREFIXES;
    $query .= '
                                  SELECT DISTINCT   ?c ?c_label ?sc ?sc_label ?x_label
                                    WHERE {
                                      ?sc rdfs:subClassOf ' . $startclass . ' .
                                      ' . $startclass . ' rdfs:label ?x_label .
                                      ?c a rdfs:Class .                
                                      ?c rdfs:subClassOf ?sc .
                                      ?c rdfs:label ?c_label .
                                      ?sc rdfs:label ?sc_label 
                                    }
                                ';

    $rows = $this->cachedQuery($query);
    $row_count = count($rows);

    $final_array = array ();

    $count = 0;

    if ($row_count > 0) {
      for ($i = 0, $i_max = $row_count; $i < min(20, $i_max); $i++) {
        $row = $rows[$i];
        $c_iri = $row["c"];
        // $id         = $this->parse_id_from_uri($c_iri);
        $c_label = $row["c_label"];
        $sc_iri = $row["sc"];
        $sc_label = $row["sc_label"];

        //HACK build hierarchy
        $final_array[$startclass]['name'] = $row['x_label'];
        $final_array[$startclass]['subclasses'][$sc_iri]['name'] = $sc_label;
        $final_array[$startclass]['subclasses'][$sc_iri]['subclasses'][$c_iri]['name'] = $c_label;

        //build classes
        //$final_array[$sc_iri]['name'] = $sc_label;
        //$final_array[$c_iri]['name'] = $c_label;
        $count++;
      }
    }
    return $final_array;
  }

  //--------------------------------------------------------------------------
  /**
  * add a property to the definistion of a class.
  * Data on specific objects are preserved
  * 
  * @param string class - the class being defined
  * @param string property - the property which belongs to the class
  * */
  //--------------------------------------------------------------------------   
  public function addPropertyToClass($class, $property) {
    return $this->createStatement($property, SONIA_RDF_CREATE_PROPERTY_OF, $class, false);
  }

  //--------------------------------------------------------------------------
  /**
  * add a property to the definistion of a class.
  * Data on specific objects are preserved
  * 
  * @todo check for duplicates
  * @param string class - the class being defined
  * @param string property - the property which belongs to the class
  * */
  //--------------------------------------------------------------------------   
  public function createProperty($property_name, $classes = null, $id = null) {
    if ($id === null) {
      //create ID //
      $id = $this->generatePropertyAutoIncrementField();
      $id_iri = SONIA_PROPERTY_ID_URI . $id;
    } else {
      $id_iri = $id;
    }

    //TODO check that it doesn' exist
    //create property
    //TODO not needed if duplicate
    $this->createStatement($id_iri, SONIA_RDF_CREATE_TYPE, SONIA_RDF_CREATE_PROPERTY_TYPE);

    //create label
    $this->createStatement($id_iri, SONIA_RDFS_CREATE_LABEL, $property_name, true);

    if (is_array($classes) && !empty ($classes)) {
      foreach ($classes as $class) {
        $this->addPropertyToClass($class, $id_iri);
      }
    }

    return $id_iri;
  }

  /**
   * generate an ID for a class itself
   * 
   * @see generateAutoIncrementField()
   * */
  public function generateClassAutoIncrementField() {
    return $this->generateAutoIncrementField(SONIA_CREATE_MAX_ID_CLASS_TYPE);
  }

  //--------------------------------------------------------------------------
  /**
  * generates an ID for a class being created and increments
  * 
  * @todo not ACID/ transactionally safe
  * @todo what if the object belongs to more than one class
  * @param string class MUST BE FULL URI
  * */
  //--------------------------------------------------------------------------
  public function generateAutoIncrementField($class) {
    $predicate = SONIA_AUTO_ID_PREFIX;
    $query = ARC_DEFAULT_PREFIXES;
    //get if exists
    $query .= 'SELECT ?s  
                                    WHERE {
                                      <' . $class . '> <' . SONIA_CREATE_MAX_ID_PROPERTY . '> ?s .
                                    }';
    //echo nl2br(htmlentities($query)).'<br>';      
    $qr1 = $this->data_api->query(array (
      "query" => $query,
      "result_type" => "rows"
    ));
    $rows = $qr1["result"];
    //$row_count       = () ?$qr1["row_count"] : '';
    $temp_facet_hiercharchy = array ();

    //if it does exist
    if (isset ($rows[0]['s'])) {
      $old_id = $rows[0]['s'];
      $id = $old_id +1;

      // echo "<br>update statement \$id $id \$old_id $old_id<br>";

      $this->updateStatement($class, SONIA_CREATE_MAX_ID_PROPERTY, null, $class, SONIA_CREATE_MAX_ID_PROPERTY, $id, true);

    } else {
      $id = 1;
      $this->createStatement($class, SONIA_CREATE_MAX_ID_PROPERTY, $id, true, 'int');
    }
    return SONIA_AUTO_ID_PREFIX . $id;
  }

  /**
   * get a list of properties (and their values) based on teh alphabetical 
   * index of their label
   * */
  public function getAllClassPropertiesRange(& $count, $startclass, $label_begin = null, $label_end = null) {
    //$text_block       = ' . ?s %s ?x%s . FILTER(REGEX(str(?x%s), "%s")) '."\n";
    $startclass_clause = '';
    if ($startclass != null)
      $startclass_clause = '?s ' . SONIA_RDF_PROPERTY_OF . ' <' . $startclass . '> .';
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?s ?little_literal
                                    WHERE {
                                      ?s a ' . SONIA_RDF_PROPERTY_TYPE . ' .
                                      ' . $startclass_clause . '
                                      ?s ' . SONIA_RDFS_LABEL . ' ?little_literal .';
    if ($label_begin != null && $label_end != null) {
      $regex = '^(' . $label_begin . ').*'; ///^(batty).*/i
      $query .= ' FILTER(REGEX(str(?little_literal), "' . $regex . '")) .';
    }
    $query .= '         }';
    $rows = $this->cachedQuery($query);
    $row_count = count($rows);
    $temp_facet_hiercharchy = array ();

    //parse into a tree
    foreach ($rows as $result) {
      $subject = $result['s'];
      $name = $result['little_literal'];

      $temp_facet_hiercharchy[$subject] = $name;
    }

    $count = count($temp_facet_hiercharchy);

    return $temp_facet_hiercharchy;

  }

  //--------------------------------------------------------------------------
  /**
  * gets all the categories/classes and counts available for a search
  * 
  * @param string in the list of subjects
  * */
  //--------------------------------------------------------------------------
  public function getAllProperties(& $count) {
    return $this->getAllClassProperties($count, null);
  }

  //--------------------------------------------------------------------------
  /**
  * gets all classes per properties
  * 
  * @todo does not handle inference! or subclasses
  * @param string $PROPERTY $URI
  * */
  //--------------------------------------------------------------------------
  public function getAllClassesOfProperty($property) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?s ?little_literal
                                    WHERE {
                                      <' . $property . '> a ' . SONIA_RDF_PROPERTY_TYPE . ' .
                                      ?s ' . SONIA_RDFS_LABEL . ' ?little_literal .
                                      ?s <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_RDFS_CREATE_CLASS_TYPE . '> .
                                      <' . $property . '> <' . SONIA_RDF_CREATE_PROPERTY_OF . '> ?s .
                                    }';

    $rows = $this->cachedQuery($query);
    $row_count = count($rows);
    $temp_facet_hiercharchy = array ();

    if (empty ($rows))
      return array ();
    //parse into a tree
    foreach ($rows as $result) {
      $subject = $result['s'];
      $name = $result['little_literal'];

      $temp_facet_hiercharchy[$subject]['name'] = $name;
    }

    $count = count($temp_facet_hiercharchy);
    $new_array = array ();

    $new_array[SONIA_RDFS_CREATE_CLASS_TYPE]['subclasses'] = $temp_facet_hiercharchy;
    $new_array[SONIA_RDFS_CREATE_CLASS_TYPE]['name'] = 'Classes';
    return $new_array;
  }

  //-------------------------------------------------------------------------- 
  /**
   * @see getAllClassProperties()
   * */
  //-------------------------------------------------------------------------- 
  public function getAllClassPropertiesMultiple(& $count, $startclasses) {
    $array = array ();
    if (is_array($startclasses)) {
      foreach ($startclasses as $startclass) {
        $array = array_merge_recursive($array, $this->getAllClassProperties($count, $startclass));
        //array_merge_recursive(
      }
    }
    return $array;
  }

  //--------------------------------------------------------------------------
  /**
  * gets all properties per class
  * 
  * @todo does not handle inference!
  * @param string in the list of subjects
  * */
  //--------------------------------------------------------------------------
  public function getAllClassProperties(& $count, $startclass) {
    $startclass_clause = '';
    if ($startclass != null)
      $startclass_clause = '?s ' . SONIA_RDF_PROPERTY_OF . ' <' . $startclass . '> .';
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?s ?little_literal
                                    WHERE {
                                      ?s a ' . SONIA_RDF_PROPERTY_TYPE . ' .
                                      ' . $startclass_clause . '
                                      ?s ' . SONIA_RDFS_LABEL . ' ?little_literal .
                                    }';
    $rows = $this->cachedQuery($query);
    $row_count = count($rows);
    $temp_facet_hiercharchy = array ();

    //parse into a tree

    if (is_array($rows)) {
      foreach ($rows as $result) {
        $subject = $result['s'];
        $name = $result['little_literal'];

        $temp_facet_hiercharchy[$subject]['name'] = $name;
      }
    }

    $count = count($temp_facet_hiercharchy);
    $new_array = array ();

    $new_array[SONIA_RDF_PROPERTY_OF]['subclasses'] = $temp_facet_hiercharchy;
    $new_array[SONIA_RDF_PROPERTY_OF]['name'] = 'Properties';
    return $new_array;
  }

  //-------------------------------------------------------------------------- 
  /**
   * get File info
   * */
  //-------------------------------------------------------------------------- 
  public function getFileInfo($image_uri) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?p ?o 
                                    WHERE {
                                      <' . $image_uri . '> ?p ?o .
                                    }';
    $rows = $this->cachedQuery($query);
    //$row_count       = count($rows);
    $count = 0;
    $return_object = array ();

    if (isset ($rows[0])) {
      foreach ($rows as $row) {
        $return_object[$count]['o'] = $row['o'];
        $return_object[$count]['p'] = $row['p'];
        $count++;
      }
    }

    return $return_object;
  }

  //-------------------------------------------------------------------------- 
  /**
   * get File info
   * */
  //-------------------------------------------------------------------------- 
  public function getFileInfoByFilename($filename, $iri_only = false) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?s ?p ?o 
                                    WHERE {
                                      ?s ?p ?o .
                                      ?s ?x "' . $filename . '" .
                                    }';
    $rows = $this->cachedQuery($query);
    //$row_count       = count($rows);
    $count = 0;
    $return_object = array ();

    if (isset ($rows[0])) {
      if ($iri_only)
        return $rows[0]['s'];
      foreach ($rows as $row) {
        $return_object[$count]['o'] = $row['o'];
        $return_object[$count]['p'] = $row['p'];
        $count++;
      }
    }

    return $return_object;
  }

  //-------------------------------------------------------------------------- 
  /**
   * gets an objects labels
   * 
   * 
   * @fixes http://drupal.org/node/217291
   * */
  //-------------------------------------------------------------------------- 
  public function getLabel($object) {
    $query = 'SELECT DISTINCT ?little_literal
                                    WHERE {
                                      <' . $object . '> <' . SONIA_RDFS_CREATE_LABEL . '> ?little_literal .
                                    } LIMIT 1';
    $rows = $this->cachedQuery($query);
    
    if (!empty($rows) && is_array($rows) && isset($rows['results']['bindings'][0])){
      return $rows['results']['bindings'][0]['little_literal']['value'];
    } else {
      return '';
    }
  }

  //-------------------------------------------------------------------------- 
  /**
   * gets URI by lable
   * assumes there is only one
   * */
  //-------------------------------------------------------------------------- 
  public function getURIByLabel($label) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?uri 
                                    WHERE {
                                      ?uri ' . SONIA_RDFS_LABEL . ' "' . $label . '" .
                                    }';
    $rows = $this->cachedQuery($query);
    //$row_count       = count($rows);
    if (isset ($rows[0]))
      return $rows[0]['uri'];
  }

  //-------------------------------------------------------------------------- 
  /**
   * assumed cardinality of 0,1
   * */
  //-------------------------------------------------------------------------- 
  public function getComment($uri) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?little_literal 
                                    WHERE {
                                      <' . $uri . '> ' . SONIA_RDFS_COMMENT . ' ?little_literal .
                                    }';
    $rows = $this->cachedQuery($query);
    //$row_count       = count($rows);

    if (isset ($rows[0]))
      return $rows[0]['little_literal'];
  }

  //-------------------------------------------------------------------------- 
  /**
   * get's an instantiated objects "categories"
   * */
  //-------------------------------------------------------------------------- 
  public function getClasses($object) {
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?o 
                                    WHERE {
                                      <' . $object . '> ' . SONIA_RDF_PROPERTY_OF . ' ?o .
                                    }';

    return $this->cachedQuery($query);
  }

  /**
   * retrieve Classes,
   * their labels, 
   * */
  public function getClassesAndCounts($sort = false) {
    //get class
    //get label
    //get object counts
    $results = array ();
    $args = array();
    //?object //?object <' . SONIA_RDF_CREATE_TYPE . '> ?class . 
    //$query = ARC_DEFAULT_PREFIXES; 
    $query = 'SELECT DISTINCT ?class ?class_label 
                                    WHERE {
                                      ?class <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_RDFS_CREATE_CLASS_TYPE . '> .
                                      ?class <' . SONIA_RDFS_CREATE_LABEL . '> ?class_label .
                                      
                                    }';
    
    $args['params'] = array('class','class_label');
    $child = array();
    $child['params'] = array('object');
    // bindings from the parent array and to the subquery
    $binding[0]['key'] = 'class';
    $binding[0]['type'] = 'resource';
    $binding[0]['value'] = '';
    $child['bindings'] = $binding;
    $child['count'] = true;
    //the return value name
    $child['binding_field_name'] = 'count';
    $child['prepare_view'] = true;
    
    $count_query = 'SELECT DISTINCT ?object WHERE { ?object   <' . SONIA_RDF_CREATE_TYPE . '>  ?class .  }';
    
    $child['query'] = $count_query;
    $child['queryname'] = 'getClassesAndCountsChild';
    
    $args['children'] = array($child)
    ;
    $rows = $this->cachedQuery($query,'rows','getClassesAndCounts',$args);
    $_sort = array();
    if (is_array($rows)) {
      foreach ($rows as $row) {
        $results[$row['class']]['label'] = $row['class_label'];
       // $results[$row['class']]['objects'][] = $row['object'];
        $results[$row['class']]['count'] = $row['count'];
        if ($sort)$_sort[$row['class']] = $row['count'];
//        if (!isset ($results[$row['class']]['count'])) {
//          $results[$row['class']]['count'] = 1;
//        } else {
//          $results[$row['class']]['count'] = $results[$row['class']]['count'] + 1;
//        }
      }
      if ($sort) array_multisort($_sort,SORT_NUMERIC, SORT_ASC,$results);
    }

    return $results;
  }

  /**
   * given a start class retrieve two layers of children eg
   * startclass = animals
   * layer1 = marsupials
   * layer2 = kangaroos
   * 
   * assumes that usually there aren't many classes and no circular references
   * */
  public function getTelescopingClassesAndCountsHierarchy( $startclass, $original_startclass ) {
    return $this->getClassesAndCountsHierarchy($startclass, $original_startclass);
  }

  /**
   * retrieve Classes,
   * their labels, related objects, and potential parents
   * 
   * gets all classes
   * 
   * @param array gives you unique object and number of times they show
   * @return all classes, objects and parents 
   * */
  public function getClassesAndCountsHierarchy($startclass,$original_startclass= null) {
    //get object counts
    $results = array ();
   // $query = ARC_DEFAULT_PREFIXES; //?object 
    $query = 'SELECT DISTINCT ?class ?label ?parent ?description  WHERE { ' .
    		'?class <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_RDFS_CREATE_CLASS_TYPE . '> .
         OPTIONAL {?class <' . SONIA_RDFS_CREATE_LABEL . '> ?label    }   .
         OPTIONAL {?class  <' . SONIA_RDFS_CREATE_COMMENT . '> ?description  }  .
         OPTIONAL {?class <'.SONIA_RDFS_CREATE_SUBCLASS.'> ?parent  } .
         		 }';
    $params = array('class','label', 'description','parent');
    
    /**
     *
                                      
     * */
    $count_field = 'class';
    $args['params'] = $params;
    $args['startclass'] = $startclass;
    
     $children = array();
    $children['params'] = array('object');
    // bindings from the parent array and to the subquery
    $binding[0]['key'] = 'class';
    $binding[0]['type'] = 'resource';
    $binding[0]['value'] = '';
    $children['bindings'] = $binding;
    if (!$this->dosearch){
      $children['count'] = true;
    } else {
      $children['count_in_search'] = session_id();
    }
    //the return value name
    $children['binding_field_name'] = 'count';
    $children['prepare_view'] = true;
    
    $_strict = (false) ? ' FILTER(REGEX(str(?object),"^'.SONIA_OBJECT_ID_URI.'")) .' :'';
    $count_query = 'SELECT DISTINCT ?object WHERE {?object   <' . SONIA_RDF_CREATE_TYPE . '>  ?class . '.$_strict.'}';
    
    $children['query'] = $count_query;
    $children['queryname'] = 'getClassesAndCountsHierarchyChild';
    
    $args['children'] = array($children) ;
    
    if ($original_startclass !== null) $args['original_startclass'] = $original_startclass; 
    return $this->cachedQuery($query,'rows','getClassesAndCountsHierarchy',$args); //use the aggregate caucho function
     
  }



  public function getPropertiesAndCounts($class = null, $with_objects = true) {
    $class_limit = null;
    //todo optionally get class
    if ($class != null) {
      $class_limit = '?object <' . SONIA_RDF_CREATE_TYPE . '> <' . $class . '> .';
    }
    
    $get_object =  ($with_objects)  ? ' ?object ' :''; 
    //get class
    //get label
    //get object counts
    $results = array ();
    $query = ARC_DEFAULT_PREFIXES;
    $query .= 'SELECT DISTINCT ?property ?property_label ?value '.$get_object.'
                                    WHERE {
                                      ?property <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_RDF_CREATE_PROPERTY_TYPE . '> .
                                      ?property <' . SONIA_RDFS_CREATE_LABEL . '> ?property_label
                                      OPTIONAL { ?object ?property ?value . }
                                      ' . $class_limit . '
                                    }';
    
    $args['params'] = array('property','property_label','value');
    $rows = $this->cachedQuery($query,'getPropertiesAndCounts',$args); 
    
   

    if (is_array($rows)) {
      foreach ($rows as $row) {
        if (!isset( $results[$row['property']]['count'])) $results[$row['property']]['count'] = 0;
        $results[$row['property']]['label'] = $row['property_label'];
        if ($with_objects) $results[$row['property']]['objects'][] = $row['object'];
        if ($with_objects && isset($row['object'])) {
          $results[$row['property']]['count'] = $results[$row['property']]['count'] + 1;
        }
      }
    }

    return $results;
  }

  public function getPropertyValuesAndCounts($property = null, $class = null) {
    
    $class_limit = null;
    //todo optionally get class
    if ($class !== null) {
      $class_limit = '?object <' . SONIA_RDF_CREATE_TYPE . '> <' . $class . '> .';
    }
    
    if ($property !== null){
     $pv2  = $pv = '<' . $property . '>';
     $pl = null;
     $normal = true;
    } else {
      $pv2 = $pl = ' ?property  ';
      $pv =  '?property <' . SONIA_RDF_CREATE_TYPE . '> <' . SONIA_RDF_CREATE_PROPERTY_TYPE . '> .    ?property '; 
      $normal = false;
    }
    
    
    $results = array ();
    //$query = ARC_DEFAULT_PREFIXES;
    $params =  array('label','property_label');//,'property'
    if($pl) $params[] = str_replace('?','',$pl);
    $count_field = 'label' ;
    $count_field = null;
    
    //
    $query .= ' SELECT DISTINCT  ?label  ?property_label  '.$pl.'  
                            WHERE {
                              ' . $pv . ' <' . SONIA_RDFS_CREATE_LABEL . '> ?property_label .  
                               ?object ' . $pv2 . ' ?label .
                              ' . $class_limit . '
                            }';
                            
    $children = array();
    $children['params'] = array('object');
    // bindings from the parent array and to the subquery
    $binding[0]['key'] = 'label';
    $binding[0]['type'] = 'literal';
    $binding[0]['value'] = '';
    $children['bindings'] = $binding;
    if (!$this->dosearch){
      $children['count'] = true;
    } else {
      $children['count_in_search'] = session_id();
    }
    //the return value name
    $children['binding_field_name'] = 'count';
    $children['prepare_view'] = true;
    
    $_strict = (false) ? ' FILTER(REGEX(str(?object),"^'.SONIA_OBJECT_ID_URI.'")) .' :'';
   // $count_query = ARC_DEFAULT_PREFIXES;
    $count_query = 'SELECT DISTINCT ?object WHERE {?object   <' . $property . '>  ?label . '.$_strict.'}';
    
    $children['query'] = $count_query;
    $children['queryname'] = 'getPropertyValuesAndCountsChild';
    
    $args = array('params'=>$params,'count'=>$count_field,'children'=>array($children));
    $rows = $this->cachedQuery($query,'rows','getPropertyValuesAndCounts',$args);
    $new_array = array();
    //remapping for compliance with other libs
    if (!empty($rows)) {
	    if (is_array($rows)){
        foreach($rows as$k=>$v ){
	        $new_array[$v['label']] = $v;
	      } 
      } else {
         $new_array = array($rows);
       }
    }
    return $new_array;
  }
  
  /**
   * helper function to retrieve a count of objects given an aribtrary facet
   * */
  public function getObjectMatchesCount($object,$property=null, $strict=false, $literal=true){
    $object =  ($literal) ?' "' . $object  . '" ':' <' . $object  . '> ';
    $_strict = ($strict) ? ' FILTER(REGEX(str(?s),"^'.SONIA_OBJECT_ID_URI.'")) .' :'';
    $property = ($property !== null) ? '<' . $property . '>': ' ?p ';
    $count_query = ARC_DEFAULT_PREFIXES;
    $count_query .= 'SELECT DISTINCT ?s WHERE {?s  ' . $property . ' '.$object.'. '.$_strict.'}';
    return $this->cachedCountQuery($count_query);
  }
  
  /**
   * helper function to retrieve a count of objects given an aribtrary facet
   * within the search
   * */
  public function getObjectsCountInSearch($object,$property=null, $strict=false, $literal=true){
    $query_addition = null;
    if ($object!==null){
      $object =  ($literal) ?' "' . $object  . '" ':' <' . $object  . '> ';
    } else {
      $object = ' ?object ';
    }
    
    $property = ($property !== null) ? '<' . $property . '>': ' ?p ';
    $query_addition = ' ?object  ' . $property . ' '.$object.'. ';
   
    
    $count_query = $this->main_search_sparql;
    $count_query = str_replace('?object ?p ?o .','?object ?p ?o . '."\n  ".$query_addition,$count_query) ;
    //doesn't like strict here
    $count_query = str_replace('FILTER(REGEX(str(?object),"^'.SONIA_OBJECT_ID_URI.'")) .','',$count_query);
    dbg(htmlentities($object.' '.$property), $strict.' '.$literal);
    return  $this->cachedCountQuery($count_query);
  }
  
  //-------------------------------------------------------------------------- 
  /**
   * Update properties by changing their names and which classes they are assigned to
   * 
   * @todo a major issue with inheretance and other assoications
   * @todo should likely be moved to SONIA or one level up - doesn't need to be part of the store API
   * */
  //-------------------------------------------------------------------------- 
  public function updateProperty($property, $property_name, $classes) {
    if ($property == '')
      return;

    //perhaps these should be create statements
    //update label
    $this->updateStatement($property, SONIA_RDFS_CREATE_LABEL, null, $property, SONIA_RDFS_CREATE_LABEL, $property_name, true);

    foreach ($classes as $class) {
      //prevents duplicate statements 
      $this->updateStatement($property, SONIA_RDF_CREATE_PROPERTY_OF, $class, $property, SONIA_RDF_CREATE_PROPERTY_OF, $class);
    }
  }

  //-------------------------------------------------------------------------- 
  /**
  * get a namespace id for a namespace
  * 
  * 
  * @param string ns the namespace
  * @return int nid the primary key of the name space
  * 
  * @todo owner, group
  * 
  * 
  * */
  //--------------------------------------------------------------------------
  public function getNSID($ns) {

  }

  //--------------------------------------------------------------------------
  /**
   * 
   * Retrieves the number of objects in the category
   * 
   * USAGE:
   * 
   * @todo make IRIs constants or member variables instead of hard coded
   * @param string $category_name_or_id name or id of the start category 
   * 
   * @return int count
   * 
   * @see query()
   * */
  //--------------------------------------------------------------------------
  public function getCategoryCount($category_name_or_id) {
    $category_name_or_id = trim($category_name_or_id);

    //TODO check for validity
    $query = ARC_DEFAULT_PREFIXES;
    $query .= ' SELECT DISTINCT ?s
                                    WHERE {
                                      ?s a <' . $category_name_or_id . '> .
                                    }';

    $qr2 = $this->data_api->query(array (
      "query" => $query,
      "result_type" => "row_count"
    ));

    return (isset ($qr2["result"])) ? $qr2["result"] : 0;
  }

  //--------------------------------------------------------------------------
  /**
   * 
   * Retrieves the number of objects having a facet 
   * 
   * USAGE:
   * 
   * @param string $property_name_or_id name or idof the property 
   * 
   * @return int count
   * 
   * @see query()
   * */
  //--------------------------------------------------------------------------
  public function getPropertyCount($property_name_or_id) {
    $property_name_or_id = trim($property_name_or_id);

    //TODO check for validity
    $query = ARC_DEFAULT_PREFIXES;
    $query .= ' SELECT ?s
                                    WHERE {
                                      ?s  <' . $property_name_or_id . '> ?y.
                                    }';

    $qr2 = $this->data_api->query(array (
      "query" => $query,
      "result_type" => "row_count"
    ));
    return (isset ($qr2["result"])) ? $qr2["result"] : 0;
  }

  //DATA CREATION

  //--------------------------------------------------------------------------
  /**
  @param
  @return int sid the sid just created
  @todo add user and group permissions management
  @todo add model concept
  @todo add lang
  */
  //--------------------------------------------------------------------------
  function createStatement($subject, $predicate, $object, $object_literal = false, $dataType = 'string', $owner = null, $group = null, $default_ns = null) {
    $statements = array ();
    if ($subject !== null) 
      $statements['add'][0]['s'] = $subject;
    if ($predicate !== null) 
      $statements['add'][0]['p'] = $predicate;
    if ($object !== null) 
      $statements['add'][0]['o'] = $object;
    if ($object_literal && $object != '') 
      $statements['add'][0]['is_literal'] =  true;
  
    $result = $this->data_api->insertStatement($statements);
  }

  function updateStatement($subject, $predicate, $object, $new_subject, $new_predicate, $new_object, $object_literal = false, $dataType = 'string', $owner = null, $group = null, $default_ns = null) {
    $statements = array ();
    if ($subject !== null) 
      $statements['clear'][0]['s'] = $subject;
    if ($predicate !== null) 
      $statements['clear'][0]['p'] = $predicate;
    if ($object !== null) 
      $statements['clear'][0]['o'] = $object;
    if ($object_literal && $object != '') 
      $statements['clear'][0]['is_literal'] =  true;
      
    if (($new_subject !== null) && ($new_predicate !== null) && ($object_literal && $new_object != '') ) 
    {
	      $statements['add'][0]['s'] = $new_subject;
	    if ($new_predicate !== null) 
	      $statements['add'][0]['p'] = $new_predicate;
	    if ($new_object !== null) 
	      $statements['add'][0]['o'] = $new_object;
	    if ($object_literal && $new_object != '') 
	      $statements['add'][0]['is_literal'] =  true;
    }
    
    $result = $this->data_api->updateStatement($statements);

  }

  /**
   * 
   * 
   * */
  function deleteStatement($subject, $predicate, $object, $object_literal = false, $dataType = 'string', $owner = null, $group = null, $default_ns = null) {

    $statements = array ();
    if ($subject !== null) 
      $statements['clear'][0]['s'] = $subject;
    if ($predicate !== null) 
      $statements['clear'][0]['p'] = $predicate;
    if ($object !== null) 
      $statements['clear'][0]['o'] = $object;
    if ($object_literal && $object != '') 
      $statements['clear'][0]['is_literal'] =  true;
  
    //safety checks
    if (($subject == null) && ($predicate == null) && ($object == null)) {
      return;
    }
    $result = $this->data_api->deleteStatement($statements);
  }

  //--------------------------------------------------------------------------
  /**
   * 
   * Creates an RDFS:Class
   * 
   * USAGE:
   * 
   * @param string class name is the name of the class
   * 
   * @return void
   * 
   * */
  //--------------------------------------------------------------------------
  public function createClass($classname, $superclasses, $id = null) {
    if ($id == null) {
      //create ID 
      //TODO check that it doesn' exist
      $id = $this->generateClassAutoIncrementField();
      $id_iri = SONIA_CLASS_ID_URI . $id;
    } else {
      //assume the id is a full IRI
      $id_iri = $id;
    }

    //todo make root class
    if (empty ($superclasses)) {
      //todo make this the root class
      $superclasses[] = SONIA_RDFS_CREATE_CLASS_TYPE;
    }

    //create property
    //TODO not needed if duplicate
    $this->createStatement($id_iri, SONIA_RDF_CREATE_TYPE, SONIA_RDFS_CREATE_CLASS_TYPE);

    //create label
    $this->createStatement($id_iri, SONIA_RDFS_CREATE_LABEL, $classname, true);

    foreach ($superclasses as $superclass) {
      $this->createStatement($id_iri, SONIA_RDFS_CREATE_SUBCLASS, $superclass);
    }

    return $id_iri;
  }

  //--------------------------------------------------------------------------
  /**
   * 
   * sets one RDFS:Class to be a subclass of another
   * 
   * USAGE:
   * 
   * @param string URI of the class
   * @param mixed URI of the superclasses
   * 
   * @return void
   * 
   * */
  //--------------------------------------------------------------------------
  public function setSubclass($class, $superclasses) {
    if (is_array($superclasses)) {
      foreach ($superclasses as $superclass) {
        if ($superclass == $class)
          continue;

        $this->createStatement($class, SONIA_RDFS_CREATE_SUBCLASS, $superclass);
      }
    } else
      if (!empty ($superclasses)) {
        if ($class == $superclasses)
          return;
        $this->createStatement($class, SONIA_RDFS_CREATE_SUBCLASS, $superclasses);
      }

  }

  public function generateObjectAutoIncrementField() {
    return $this->generateAutoIncrementField(SONIA_CREATE_OBJECT_COUNT_TYPE);
  }

  public function generatePropertyAutoIncrementField() {
    return $this->generateAutoIncrementField(SONIA_CREATE_MAX_ID_PROPERTY_TYPE);
  }

  public function generateFileAutoIncrementField() {
    return $this->generateAutoIncrementField(SONIA_CREATE_MAX_ID_FILE_TYPE);
  }

  //--------------------------------------------------------------------------
  /**
   * 
   * places facet and its value in an existing class. Same as an RDF Literal
   * 
   * USAGE:
   * 
   * @param string $object_name_or_id the reference to the object
   * @param string $facetType the identifier for the property
   * @param string $data the data
   * @param string $dataType int, string, etc , not enforced
   * 
   * 
   * @return array a multi-deminsional array expressing the hierarchy of 
   * categories
   * 
   * @see createObject()
   * */
  //--------------------------------------------------------------------------
  public function setObjectProperty($object_name_or_id, $property, $data, $dataType) {

  }

  /**
   * 
   * @return boolean
   * @see statementExists()
   * 
   * */
  public function URIExists($s) {
    return $this->statementExists($s, null, null, true);
  }

  /**
  * must be exact match - use FILTER through QUERY for regex matching
  * @return boolean
  * @see statementExists()
  * 
  * */
  public function LiteralExists($s) {
    return $this->statementExists(null, null, $s, false, true);
  }

  //--------------------------------------------------------------------------
  /**
  * basic function to confirm if a statement exists,
  * used to determine if a class, or paramter has already been created
  * 
  * @param string uri subject
  * @param string uri predicate
  * @param string uri object
  * @param boolean - see if the one URI exists anywhere in a statement
  * @param boolean - do an exact match on a literal
  * 
  * @return boolean does the statement exist
  * */
  //--------------------------------------------------------------------------
  public function statementExists($s, $p = null, $o = null, $any = false, $literal = false) {
   
   //dbg($s.$p.$o,'AA');
   
   $query = '';
    $s = trim($s);

    if ($any && !empty ($s)) {

      $s = ' <' . trim($s) . '>';
      $query1 = $query . " ASK WHERE { $s ?p ?o . }";
      $qr1 = $this->data_api->query(array (
        "query" => $query1,
        "result_type" => "bool"
      ));
      
      
      if ($qr1['result'])
        return true;

      $query2 = $query . " ASK WHERE { ?s ?p $s . } ";
      $qr2 = $this->data_api->query(array (
        "query" => $query2,
        "result_type" => "bool"
      ));
      if ($qr2['result'])
        return true;

      $query3 = $query . " ASK WHERE { ?s $s ?o . } ";
      $qr3 = $this->data_api->query(array (
        "query" => $query3,
        "result_type" => "bool"
      ));
      if ($qr3['result'])
        return true;

    } 
    
//    else
//      if ($literal) {
//        $s = '?s';
//        $p = '?p';
//        $o = ' "' . trim($o) . '"';
//
//        //TODO check for validity
//        $query .= " ASK WHERE { $s $p $o . }";
//        $qr = $this->data_api->query(array (
//          "query" => $query,
//          "result_type" => "bool"
//        ));
//
//        if ($qr['result'])
//          return true;
//      } else {
//        $p = (!empty ($p)) ? ' <' . trim($p) . '>' : '?p';
//        $o = (!empty ($o)) ? ' <' . trim($o) . '>' : '?o';
//        $s = (!empty ($s)) ? ' <' . trim($s) . '>' : '?s';
//
//        //TODO check for validity
//        $query .= " ASK WHERE { $s $p $o . } ";
//        $qr = $this->data_api->query(array (
//          "query" => $query,
//          "result_type" => "bool"
//        ));
//        
//        //dbg($qr,"statementExists($s, $p = null, $o = null, $any = false, $literal = false)");
//
//        if ($qr['result'])
//          return true;
//      }

    return false;
   
  }

} //end interface
