<?php

/**
 * Static Functions related to URL parsing and generation
 * 
 * @license GPLv2
 * @license 2006 KBCMS LLC
 * @author  Jonathan Hendler
 * @package SONIA
 * @version 0.3 
 * */

class SONIA_URL_H {

    public static function sfsdf($fpath) {
        $fsize = filesize($fpath);
        $bufsize = 20000;

        if (isset ($_SERVER['HTTP_RANGE'])) //Partial download
            {
            if (preg_match("/^bytes=(\\d+)-(\\d*)$/", $_SERVER['HTTP_RANGE'], $matches)) { //parsing Range header
                $from = $matches[1];
                $to = $matches[2];
                if (empty ($to)) {
                    $to = $fsize -1; // -1  because end byte is included
                    //(From HTTP protocol:
                    // 'The last-byte-pos value gives the byte-offset of the last byte in the range; that is, the byte positions specified are inclusive')
                }
                $content_size = $to - $from +1;

                header("HTTP/1.1 206 Partial Content");
                header("Content-Range: $from-$to/$fsize");
                header("Content-Length: $content_size");
                header("Content-Type: application/force-download");
                header("Content-Disposition: attachment; filename=$fname");
                header("Content-Transfer-Encoding: binary");

                if (file_exists($fpath) && $fh = fopen($fpath, "rb")) {
                    fseek($fh, $from);
                    $cur_pos = ftell($fh);
                    while ($cur_pos !== FALSE && ftell($fh) + $bufsize < $to +1) {
                        $buffer = fread($fh, $bufsize);
                        print $buffer;
                        $cur_pos = ftell($fh);
                    }

                    $buffer = fread($fh, $to +1 - $cur_pos);
                    print $buffer;

                    fclose($fh);
                } else {
                    header("HTTP/1.1 404 Not Found");
                    exit;
                }
            } else {
                header("HTTP/1.1 500 Internal Server Error");
                exit;
            }
        } else // Usual download
            {
            header("HTTP/1.1 200 OK");
            header("Content-Length: $fsize");
            header("Content-Type: application/force-download");
            header("Content-Disposition: attachment; filename=$fname");
            header("Content-Transfer-Encoding: binary");

            if (file_exists($fpath) && $fh = fopen($fpath, "rb")) {
                while ($buf = fread($fh, $bufsize))
                    print $buf;
                fclose($fh);
            } else {
                header("HTTP/1.1 404 Not Found");
            }
        }
    }

    public static function restartSearchURL() {
        return SONIA_URL_H :: makeHashURLForSearch() . '&' . SONIA_NEWSEARCH_SIGNAL . '=y';
    }


    public static function makeHashURLForSearch() {
        $url = '';
        $url .= '&' . SONIA_FORM_REQUEST_PREFIX . '[' . SONIA_SEARCH_SUBMIT_SIGNAL . ']=1';
        $url .= '&' . SONIA_FORM_REQUEST_PREFIX . '[' . SONIA_COMMENCE_SEARCH . ']=1';
        return $url;
    }
   
    /**
    * allows entire search to placed in a URL to allow for bookmarking or 
    * sending in an email, etc
    * 
    * @param int $count = when to not add a url to the list 
    * 
    * */
    public static function makeFullURLFromSearchWithExclude(& $ns, $count = 0) {
        $url = '?';
        $i = 1;
        
        if (is_array($ns)) {
            foreach ($ns as $key => $value) {
            	  if ($count == 0 || ($count >= $i)){
            	  	
	                if (!is_array($value)) {
	                    if (SONIA_URL_H :: notSignal($key)) {
	                         $url .= '&' . SONIA_FORM_REQUEST_PREFIX . '[' . $key . ']=' . SONIA_H :: encodeURI($value);
	                         $i++;
	                    }
	                } else {
	                    foreach ($value as $k => $v) {
	                      if (!is_array($v)){
	                        $k2 = SONIA_H :: encodeURI($k);
	                        $v2 = SONIA_H :: encodeURI($v);
	                        if (SONIA_URL_H :: notSignal($key) && ($count == 0 || ($count >= $i))) {
	                          $url .= '&' . SONIA_FORM_REQUEST_PREFIX . '[' . $key . '][' . $k2 . ']=' . $v2;
	                          $i++;
	                        }
	                      }
	                      else{
	                        foreach($v as $kk=>$vv){
	                          $k2 = SONIA_H :: encodeURI($kk);
	                          $v2 = SONIA_H :: encodeURI($vv);
	                          if (($count == 0 || ($count >= $i))) $url .= '&' . SONIA_FORM_REQUEST_PREFIX . '[' . $key . '][][' . $k2 . ']=' . $v2;
	                          $i++;
	                        }
	                      }
	                    }
	                }
	            }
              
            }
        }
        
        $url .= '&' . SONIA_FORM_REQUEST_PREFIX . '[' . SONIA_SEARCH_SUBMIT_SIGNAL . ']=1';
        $url .= '&' . SONIA_FORM_REQUEST_PREFIX . '[' . SONIA_COMMENCE_SEARCH . ']=1';
        $url .= '&' .  SEMANTIC_SEARCH_PAGINATION_SIGNAL . '='. SONIA_IO :: getPaginationBegin();
        return $url . '';
    }
   
    /**
      * allows entire search to placed in a URL to allow for bookmarking or 
      * sending in an email, etc
      * Does not allow CRUD options
      * */
    public static function makeFullURLFromSearch(& $ns, $prefix = null) {
        $url = $prefix;
        return $url . SONIA_URL_H :: makeFullURLFromSearchWithExclude($ns, 0);
    }

    /**
    * check that a string value is not a CRUD url option
    * used to prevent CRUD commands from being exectuted twice on page form redirects
    * 
    * */
    public static function notSignal($key) {
        $crud_keys = array (
            SONIA_SEARCH_SUBMIT_SIGNAL,
            SONIA_COMMENCE_SEARCH,
            SONIA_NEWSEARCH_SIGNAL,
        );
        return !in_array($key, $crud_keys);
    }

} // end class
