<?php


/*
 * SONIA
 * Simple Ontology and Inference Algorithms
 * 
 * The SONIA API provides a standardized API for the use of RDF stores and other RDBMS
 * solutions to provided faceted search. 
 * */
/**
 * SONIA
 * Simple Ontology and Inference Algorithms
 * 
 * The SONIA API provides a standardized API for the use of RDF stores and other RDBMS
 * solutions to provided faceted search. 
 * 
 * 
 * @license GPL-V2 FSF
 * @license 2006 KBCMS LLC
 * @author  Jonathan Hendler <[hendlerman (at) yahoo DOT com]>
 * @package SONIA
 * @version 0.4
 * 
 * */
interface SONIAAPI {
    /**
    * import RDF into the rdf store 
    * 
    * @todo validate the RDF
    * @param string $iri_model the name of the model
    * @param string $file the location of the file, can be any uri PHP allows http:// file:/// or just a path
    * */
    public function importRDF($iri_model, $file);

    //--------------------------------------------------------------------------
    /**
    * export the rdf store 
    * 
    * @todo validate the RDF
    * @param string $iri_model the name of the model
    * */
    //--------------------------------------------------------------------------
    public function exportRDF($iri_model);

    //--------------------------------------------------------------------------
    /**
     * Export data to a file and a native format
     * 
     * @param array query given a query, export the results to a
     * @return string flat file conents
     * 
     * */
    //--------------------------------------------------------------------------
    public function exportNative();

    //--------------------------------------------------------------------------
    /**
     * Import data from a SONIA native triple format given one of four methods.
     * 
     * @param string method 1. overwrite old - no duplicates 2. preserve old - not duplicates 3. duplicates allowed 4. clear out ALL old
     * @param string file, filecontents uplodaded
     * @return array verbose output string succes or failure
     * */
    //--------------------------------------------------------------------------
    public function importNative($method, $file);

    //--------------------------------------------------------------------------
    /**
     * Helper function to provide a list of available stores. For exampleone
     * may choose to query multiple stores, be they an RDF, DB, or triple
     * backend implementation
     * @return array a list of available implemented stores
     * 
     * */
    //--------------------------------------------------------------------------
    public function listStores();

    /**
    * A simplification of getCustomSearchParamsData().
    * Given a category/class, parameter, and an optional FILTER (see SPARQL)
    * return an array of IDs(IRIs), labels, and counts (total in search, total) which can be used
    * to construct dynamic search forms. For ranges on return values or labels use
    * getTelescopingSearchParamsData().
    * 
    * SONIA depends on a PHP memory hashtable (array) to store _all_ classes and
    * params and their labels, so this _may_ be restrictive for very large 
    * data sets.  
    * 
    * The function is basically a search operation. Return values are cached 
    * by the function unless otherwise expressed.
    * 
    * @param string startclass the startclass for the category
    * @param string fsearch_type "category", "parameter"
    * @param int depth how far down in a hierarchy to traverse from "startclass"
    * 
    * 
    * @todo tests and handling for large datasets for categories, classes
    * @todo more complex category arrangment based on RDF graphs rather than a 
    *   faceted search hiearchy only
    *  
    * @see getCustomSearchParamsData()
    * */
    public function getSearchParamsData($start_class, $fsearch_type, $depth = 3);

    /**
    * A custom paramter value or place in a hiearchy that remembers either it's 
    * depth in the hiearchy or range of values within the total search results. 
    * 
    * @param string $query_type 'class', 'property', 'object'
    * @param mixed $uris the set of uris
    * @param string $return_type 
    *   'list_objects' 'list_properties' 'list_values'
    * @param array $values['min'], $values['max'] range search 
    *   null for all $values['regex']
    * @param string $order_by eg. 'label ASC'
    * @param boolean $return_counts include count information, can be a performance hit
    * 
    * @param boolean cache whether or not to cache the return value.
    * @param int $break_custom the in the telescoping search, how many breaks between 
    *  MIN and MAX should be set. MIN and Max can be integer or alphabetical
    * 
    * @return array ''
    *  
    * @see getSearchParamsData()
    * */
    public function getCustomSearchParamsData($query_type, $uris, $return_type, $values, $order_by = null, $return_counts = true, $depth = 3, $break_custom = 5, $cache = true);

    //--------------------------------------------------------------------------
    /**
     * 
     * Creates an RDFS:Class
     * 
     * USAGE:
     * @param string $id is the ID or URI of the class
     * @param string class name is the name of the class
     * @param array $superclasses are the set of superclasses
     * @todo replace IDs
     * 
     * @return void
     * 
     * */
    //--------------------------------------------------------------------------
    public function updateClass($id, $classname, $superclasses = null);

    //--------------------------------------------------------------------------
    /**
     * 
     * creates an instance of an RDFS:Class 
     * 
     * USAGE:
     * 
     * @param string $object_name_or_id the reference to the object
     * @param array $object_category_list list of categories to which the object belongs
     * @param array $object_param_list list of parameters and there values,datatypes, literal or not
     * 
     * @return void
     * 
     * @see setObjectFacet()
     * */
    //--------------------------------------------------------------------------
    public function createObject($label, //TODO not needed?
    $classes, $parameters);

} //end interface
