<?php

/**
 * 
 * */
abstract class SONIASearch extends SONIABase {
    //--------------------------------------------------------------------------
    /**
     * 
     * Constructs and exectutes queries over a triple store, RDF store, RDBMS etc.
     * 
     * a statement set is a set of parameters to join nested queries into a 
     * single complex query.
     * It is also to be used over an RPC.
     * 
     * USAGE:
     * 
     * @param int user_id for ACL management
     * @param array statement_sets is the array of statements to build the query
     * @param int limit the LIMIT on the number of results to retrieve - 
     *          unlike standard SQL this is number of statements not number of Rows
     * @param int offset where to begin the LIMIT
     * @param boolean any whether to construct the statements to match any or all of the parametrs
     * @param boolean distinct remove repeate results
     * @param boolean whole_class - get all info relating/contained (SPARQL DESCRIBE)
     * 
     * @return array results
     * 
     * @see makeSearchStatement()
     * */
    //--------------------------------------------------------------------------
    function query($user_id, $statement_sets, $limit, $offset, $distinct, $whole_class, $any, $strict = false) {

        //$this->io->logError ('QUERY SENT'.print_r($statement_sets,1));

        ///////////////////////////////////////////////////////////////////////////
        //HACK
        //returns only objects and class types
        //prevents file types
        //in the future, people may want results of another type, like files
        //     if ($strict)
        //     {
        //       $statement_sets[] = array (
        //        'subject'=>null,
        //        'predicate'=> SONIA_RDF_CREATE_TYPE,
        //        'object'=>SONIA_CLASS_ID_URI,
        //        'literal_object'=>true,
        //        'category_query'=>false,
        //        'order_by' = 'type'|'order'|'key'  );
        //     }
        //clean statement set
        //should default to false
        $query = array ();
        $query['user_id'] = $user_id;
        $query['statement_sets'] = $statement_sets;
        $query['limit'] = $limit;
        $query['offset'] = $offset;
        $query['any'] = $any;
        $query['distinct'] = $distinct;
        $query['describe'] = $whole_class;
        //dbg($statement_sets,'$statement_sets');
        return $this->store->query($query);
    }

    public function classQuery() {
        return $this->store->getClassesAndCounts();
    }

    /**
     * @todo limit to a class
     * */
    public function propertyQuery() {
        return $this->store->getPropertiesAndCounts();
    }

    public function propertyValuesQuery($property_uri) {
        if (is_array($property_uri))
            $property_uri = $property_uri[0];
        return $this->store->getPropertyValuesAndCounts($property_uri);

    }

    public function objectQuery() {
        //parse input

        //get results

        //parse results
    }
    
    /**
     * generate the search hash
     * */
    private function getSearchHash(){

      if (empty($this->sonia_search_array)){
        $key = 'empty_';
      } else{
        $key = array();
        foreach($this->sonia_search_array as $q){
          $k = md5($q['predicate'].$q['subject'].$q['object']);
          $key[$k] = $q['predicate'].$q['subject'].$q['object'];
        }
      }
      return SO_CA :: gh($key, 'base', true);
   }
    
    /**
     * retrieve the results of a query and set it as a member variable of the
     * object
     * 
     * uses caching
     * 
     * */
    public function setSearchResults() {
        $search_hash = $this->getSearchHash();
        //dbg($this->sonia_search_array,'$this->sonia_search_array');
        $this->search_results = (SONIA_USE_CACHE) ? $this->io->getCache($search_hash,true,true) : null;
        
       if (SONIA_USE_CACHE && $this->search_results !==  null ) {
            //dbg($this->sonia_search_array,'y? ' .$search_hash);

             return;
        } else
            if (!empty ($this->sonia_search_array) && is_array($this->sonia_search_array)) {
            	 //TODO move drupal specific stuff out
            	 foreach (module_list() as $module) {
						      $result = module_invoke($module, 'semantic_search_params_extra', $this->sonia_search_array );
						      if (!empty($result)) {
						        $this->sonia_search_array = $result;
						      }
						   }
						   //dbg($this->sonia_search_array,'$this->sonia_search_array');
                $expand = (isset ($this->search_params['expand'])) ? true : false;
                 //__nsp(' SONIA q');
                $this->search_results = $this->query(0, $this->sonia_search_array, null, null, $expand, '', '', true);
                 // __nsp(' SONIA q end');
                 //dbg(count($this->search_results),"cached search array of depth ".count($this->sonia_search_array));
                 if (SONIA_USE_CACHE) $this->io->putCache($search_hash, $this->search_results,true,true);
                
            } 
           
    }
    
    function mt()
    {
       list($usec, $sec) = explode(" ", microtime());
       return ((float)$usec + (float)$sec);
    }
    
    //--------------------------------------------------------------------------
    /**
     * will carry out needed functions if the search has been commenced
     * */
    //--------------------------------------------------------------------------
    public function _doSearch() {
        $this->parseRequest();
        $this->setSearchResults();
    }

    /**
     * check to see if the key exists in the property heiarchy
     * 
     * */
    public function propertyExists($key, $array = null) {
        //     if ($array == null)
        //     {
        //       $array = $this->temp_facet_hiercharchy;
        //       $key = SONIA_H :: decodeURI($key);
        //     }
        //     
        //     if (isset($array[$key]))
        //     { 
        //        return true;
        //     }
        //     
        //     foreach ($array as $k => $value)
        //     {
        //        if (isset($value['subclasses']))
        //        {
        //            $result = $this->propertyExists($key,$value['subclasses']);
        //            
        //            if ($result) return true;
        //        }
        //     }
        //     
        return $this->store->statementExists($key, SONIA_RDF_CREATE_TYPE, SONIA_RDF_CREATE_PROPERTY_TYPE);
    }

    /**
     * Determine if a category was used in a search
    * */
    public function categoryInSearch($key) {
        if (empty ($this->search_params) || !(is_array($this->search_params)))
            return false;
        return ($key != '' && (isset ($this->search_params[SONIA_FORM_REQUEST_PREFIX]['cat'][$key]) || isset ($this->search_params['cat'][$key])));
    }

    /**
    * Determine if a property was used in a search
    * */
    public function propertyInSearch($key) {
        if (empty ($this->search_params) || !(is_array($this->search_params)))
            return false;
        return ($key != '' && (isset ($this->search_params[SONIA_FORM_REQUEST_PREFIX]['facet'][$key]) || isset ($this->search_params['facet'][$key])));
    }

    //--------------------------------------------------------------------------
    /**
     * 
     * Retrieves the number of objects in the category
     * 
     * USAGE:
     * 
     * @param string $category_name_or_id name or id of the start category 
     * 
     * @return int count
     * 
     * @see query()
     * */
    //--------------------------------------------------------------------------
    public function getCategoryCount($category_name_or_id) {
        return $this->store->getCategoryCount($category_name_or_id);
    }

    /**
    * check that it is time to run the search
    * */
    public function searchSubmitted() {
        return isset ($this->all_params[SONIA_FORM_REQUEST_PREFIX][SONIA_COMMENCE_SEARCH]);
    }

    //--------------------------------------------------------------------------   
    /**
     * 
     * 
     * @see getObject($subject_uri)
     * */
    //--------------------------------------------------------------------------
    public function getObjectWithFiles($subject_uri) {
        $object = $this->getObject($subject_uri);

        if (is_array($object)) {
            foreach ($object as $data) {
                if ($data['p'] == SONIA_CREATE_HAS_IMAGE) {
                    $object['images'][] = $this->store->getFileInfo($data['o']);
                } else
                    if ($data['p'] == SONIA_CREATE_HAS_FILE) {
                        $object['files'][] = $this->store->getFileInfo($data['o']);
                    }
            }
        }

        return $object;
    }

    //--------------------------------------------------------------------------
    /**
     * Get information about a single object
     * If it doesn't exist return false
     * 
     * @todo add filters etc
     * @return mixed - object data or false
     * 
     * */
    //--------------------------------------------------------------------------
    public function getObject($subject_uri) {
        //then try DB  
        $result = $this->store->statementExists($subject_uri);
        if (!$result)
            return false;

        $sonia_search_array[] = array (
            'subject' => $subject_uri,
            'predicate' => null,
            'object' => null,
            'literal_object' => false,
            'category_query' => true
        );

        $result = $this->query(0, $sonia_search_array, null, null, false, '', '');
        if (isset ($result[$subject_uri]))
            $result = $result[$subject_uri];
        return $result;
    }

    /**
     * get the classes an object belongs to directly
     * */
    public function getObjectsDirectClasses($subject_uri) {
        $sonia_search_array[] = array (
            'subject' => $subject_uri,
            'predicate' => SONIA_RDF_CREATE_TYPE,
            'object' => null,
            'literal_object' => false,
            'category_query' => true
        );

        $result = $this->query(0, $sonia_search_array, null, null, false, '', '');
        if (isset ($result[$subject_uri]))
            $result = $result[$subject_uri];
        return $result;
    }

    //--------------------------------------------------------------------------
    /**
     * Get information about a single object
     * If it doesn't exist return false
     * 
     * Note:
     * There can be duplicate properties.
     * 
     * 
     * @param array $search_results
     * @param string uri $subject
     * @param string uri $property
     * @todo add filters etc
     * @return mixed
     * 
     * */
    //--------------------------------------------------------------------------
    public function getLiteralValueOfSearchResult(& $search_results, $subject, $property) {
        $results = array ();
        if (isset ($search_results[$subject])) {
            foreach ($search_results[$subject] as $item) {
                if (isset ($item['p']) && $item['p'] == $property) {
                    $results[] = $item['o'];
                }
            }
        } else {
            foreach ($search_results as $item) {
                if (isset ($item['p']) && $item['p'] == $property) {
                    $results[] = $item['o'];
                }
            }
        }
        return $results;
    }

    //--------------------------------------------------------------------------
    /**
     * 
     * */
    //--------------------------------------------------------------------------
    public function getAllClassesOfObject($object_uri) {
        return $this->store->getAllClassesOfObject($object_uri);
    }

    //--------------------------------------------------------------------------
    /**
     * 
     * Retrieves a facets search forms
     * 
     * USAGE:
     * 
     * @param string start name or id of the start category 
     * @todo clean up formatting of standard arrays
     * 
     * @return array a multi-deminsional array expressing the hierarchy of 
     * categories
     * 
     * @see query()
     * */
    //--------------------------------------------------------------------------
    public function getProperties($start) {
        $varname = SONIA_CACHE_PROPERTIES_NAME;

        if (($start == null) || ($start == '')) {
            $start = SONIA_DEFAULT_FACET_START_ID;
        }

        //cache or DB retrieval
        if ($this->io->checkCache($varname, true)) {
            $this->temp_facet_hiercharchy = $this->io->getCache($varname, true);
        } else //store in SESSION cache
            {
            $this->temp_facet_hiercharchy = $this->store->getAllProperties($this->total_facet_count, $start);
            $this->getPropertyCount($this->temp_facet_hiercharchy);
            $this->io->putCache($varname, $this->temp_facet_hiercharchy, true);
        }

        $this->getPropertyCounts($this->temp_facet_hiercharchy);

        // this will need to be cleaned up.. properties can also have subclasses
        //$this->all_facets = SONIA_H :: buildHierarchy($this->temp_facet_hiercharchy, $start);
        $this->all_facets = $this->temp_facet_hiercharchy;
        return $this->all_facets;
    }

    //----------------------------------------------------------------------------   
    /**
     * recursive function fo find occurences of properties in search
     * 
     * 
     * @todo do quick sums of superclasses
     * */
    //----------------------------------------------------------------------------
    function getPropertyCounts(& $temp_facet_hiercharchy) {
        //count in search - can not be cached
        if (empty ($temp_facet_hiercharchy) || !is_array($temp_facet_hiercharchy))
            return;
        foreach ($temp_facet_hiercharchy as $key => $category) {
            if (isset ($category['subclasses'])) {
                //not very memory efficient
                $tmp = $category['subclasses'];
                $this->getPropertyCounts($tmp);
                $temp_facet_hiercharchy[$key]['subclasses'] = $tmp;
            } else {

                $temp_facet_hiercharchy[$key]['count_in_search'] = ($temp_facet_hiercharchy[$key]['count'] > 0) ? $this->getFacetSearchCount($key) : 0;

                $temp_facet_hiercharchy[$key]['in_search'] = $this->propertyInSearch($key);
            }
        }
    }


    //--------------------------------------------------------------------------
    /**
     * Simple accessor method used to retrieve the search params.
     * Can be used by the UI to preserve and parse the search params
     * 
     * @param void
     * @return array the search_params array which represents the search params used internally
     * */
    //--------------------------------------------------------------------------
    public function getSearchParams() {
        return (isset ($this->search_params)) ? $this->search_params : null;
    }

    /**
     * 
     * @todo should be recursive goes 3 levels
     * */
    public function getCategoryName($iri) {
        $decodedkey = SONIA_H :: decodeURI($iri);

        if (isset ($this->all_categories[$decodedkey])) {
            return $this->all_categories[$decodedkey]['name'];
        } else {
            foreach ($this->all_categories as $category) {
                if (isset ($category['subclasses'][$decodedkey]['name'])) {
                    return $category['subclasses'][$decodedkey]['name'];
                } else
                    if (is_array($category['subclasses'])) {
                        foreach ($category['subclasses'] as $c) {
                            if (isset ($c['subclasses'][$decodedkey]['name'])) {
                                return $c['subclasses'][$decodedkey]['name'];
                            }
                        }
                    }
            }
        }
    }

    //--------------------------------------------------------------------------
    /**
    * a generalized, function for retrieving the number of facets which are found within the search
    * 
    * @return int the number of instances of the property name
    * */
    //--------------------------------------------------------------------------
    public function getSearchPropertyCount($category_uri, $property_name_or_uri) {
        $count = 0;

        //no need to go further if empty
        if (empty ($this->search_results)) {
            return 0;
        }

        foreach ($this->search_results as $key => $value) {
            if ($property_name_or_uri == SONIA_RDF_CREATE_TYPE) {
                $new_a = $value[SONIA_RDF_CREATE_TYPE];

                foreach ($new_a as $k => $v) {
                    if ($v == $category_uri)
                        $count++;
                }
            } else {
                foreach ($value as $k => $v) {
                    if (isset ($v['p']) && $v['p'] == $property_name_or_uri && $v['o'] == $category_uri)
                        $count++;
                }
            }
        }
        return $count;
    }

    public function getComment($uri) {
        return $this->store->getComment($uri);
    }

    /**
    * Get the label (name for an URI)
    * */
    public function getLabel($uri) {
        //try to use the cache
        if (isset ($this->temp_space_hiercharchy[$uri])) {
            return $this->temp_space_hiercharchy[$uri]['name'];
        } else
            if (isset ($this->temp_facet_hiercharchy[$uri])) {
                return $this->temp_facet_hiercharchy[$uri]['name'];
            } else {
                return $this->store->getLabel($uri);
            }
    }

    //SEARCH HELPERS
    /**
     * Provides a standard interface for retrieving options and their
     * occurrences in a search
     * 
     * */
    public function getOptions($ns_predicate, $ns_subject, $count = false) {
        return $this->store->getAllLiteralsAndCount($ns_predicate, $ns_subject);
    }

    /**
     * function for getting metadata about all the data
     * */
    public function getBaseElementsAndCounts(){
        
       $uris['Classes'] = $this->store->getClassesAndCounts();
       //$uris['Properties'] = $this->store->getPropertiesAndCounts();
       //$uris['property_values']=  $this->store->getPropertyValuesAndCounts();
       return $uris;
    }
    
    public function getAllBaseElementsAndCounts(){
       $uris = array();
       $uris['Classes'] = $this->store->getClassesAndCounts(true);
       //$uris['Properties'] = $this->store->getPropertiesAndCounts();
       //$uris['property_values']=  $this->store->getPropertyValuesAndCounts();

       return $uris;
    }
    
    public function getAllCategoriesRecursive($uri = null) {
        //if ($uri == null) $uri = SONIA_RDF_DEFAULT_ROOT_CLASS_URI;
        $t = ($uri != null) ? true : false;
        return $this->store->getAllCategoriesRecursive($uri, 0, $t);
    }

    /**
     * given an IRI for a category, check that it exists
     * */
    public function classExists($key) {
        $decodedkey = SONIA_H :: decodeURI($key);

        //first try cache
        //
        if (isset ($this->all_categories[$decodedkey])) {
            return true;
        } else
            if (is_array($this->all_categories)) {
                foreach ($this->all_categories as $k2 => $category) {
                    if (isset ($category['subclasses'][$decodedkey]['label'])) {
                        return true;
                    } else
                        if (is_array($category['subclasses'])) {
                            foreach ($category['subclasses'] as $k3 => $c) {
                                if (isset ($c['subclasses'][$decodedkey]['label'])) {
                                    return true;
                                }
                            }
                        }
                }
            }
        // echo "\$decodedkey $decodedkey \$key =$key";
        //then try DB  
        return $this->store->statementExists($decodedkey, SONIA_RDF_CREATE_TYPE, SONIA_RDFS_CREATE_CLASS_TYPE);

    }

    /**
     * determine if a property is a file type
     * */
    public function isFileType($key) {
        return $this->store->statementExists($key, SONIA_RDF_CREATE_TYPE, SONIA_FILE_TYPE_URI);
    }

    //-------------------------------------------------------------------------- 
    /**
    * check to see if a displayed property was in the search
    * This would be useful, for example, if you want to highlight a displayed
    * property that was in the search.
    * 
    * @todo mixed name alternate
    * 
    * @param string $key the key number or id of the property
    * @param mixed $value, should the value be matched
    * @param string $name_alternate the string name of the property to match
    * */
    //--------------------------------------------------------------------------
    public function checkPropertyInSearch($key,$search_value,$name_alternate = null, $avalue = null) {
       
        $key = trim(SONIA_H :: encodeURI($key));
        
        //TODO on Pvalues only display 1
        foreach ($this->search_params as $k1 => $value) {
            if (is_array($value)) {
                foreach ($value as $k => $v) {
                    if (!is_array($v)){
                        $k = trim(SONIA_H :: encodeURI($k));
                         
                        if ($k == $key && $k1 != 'p_value'){ // && ($k >0 && $k != null)
                            return true;
                        }
                        else if ($k == $key  && $v == $search_value){ // && ($k >0 && $k != null)
                            return true;
                        }
                    } else{
                      foreach($v as $kk => $vv){
                        $kk = trim(SONIA_H :: encodeURI($kk));
                        $vv2 = trim(SONIA_H :: encodeURI($vv));
                        
                        if ($kk == $key && $k1 != 'p_value'){ // && ($k >0 && $k != null)
                            return true;
                        }
                        else if ($kk == $key && $vv == $search_value){ // && ($k >0 && $k != null)
                            return true;
                        } else if ($key ==$vv2 ){
                            return true;
                        }
                      }
                    }
                }
            } else
                if ($key == $name_alternate && $k1 == $name_alternate && $value != '' && $key != '') {
                    return true;
                }
            //TODO note if $value of term is in value of search
            else
                if (($avalue != null) && (stristr(trim($value), trim($avalue)))) {
                    return true;
                }
        }
        return false;
    }

    //--------------------------------------------------------------------------
    /**
     * 
     * Retrieves the number of objects having a certain facet in general 
     * This function is a wrapper from the internal store
     * 
     * USAGE:
     * 
     * @param string $property_name_or_id name or id of the property
     * 
     * @return int count
     * @see SONIAStoreAPI :: getPropertyCount()
     * @see query()
     * */
    //--------------------------------------------------------------------------
    public function getPropertyCount(& $facet_array) {
        foreach ($facet_array as $k => $v) {
            if (isset ($v['subclasses'])) {
                //not very memory efficient
                $tmp = $v['subclasses'];
                $this->getPropertyCount($tmp);
                $facet_array[$k]['subclasses'] = $tmp;
            } else {
                foreach ($facet_array as $key => $value) {
                    $facet_array[$key]['count'] = $this->store->getPropertyCount($key);
                }
            }
        }
    }

    /**
     * get a count of category AND its subclasses
     * */
    function getCategoryCountRecursive($key, $category) {
        if (empty ($category['subclasses']) || !is_array($category['subclasses'])) {
            $category['count'] = $this->getCategoryCount($key);
        } else {
            foreach ($category['subclasses'] as $k2 => $c2) {
                $category['subclasses'][$k2] = $this->getCategoryCountRecursive($k2, $c2);
            }
        }
        return $category;
    }

    /**
      * retrieve the label of a facet/property from the stored member variable
      * 
      * @param string iri the URI of which the label need be found
      * */
    public function getPropertyName($iri, $temp = null) {
        $decodedkey = SONIA_H :: decodeURI($iri);
        //lookin search results
        //dbg($this->search_results,'$this->search_results');
        if (is_array($this->search_results)) {
            foreach ($this->search_results as $key => $value) {
                foreach ($value as $k => $v) {
                    if ($v['p'] == $decodedkey && (isset ($v['p_l'])))
                        return $v['p_l'];
                    if ($v['p'] == $decodedkey && (isset ($v['p_l'][0])))
                        return $v['p_l'][0];
                }
            }
        }
        //or get from store
        return $this->getLabel($decodedkey);
    }

    /**
     * given a class, find all it's properties
     * 
     * @param array $properties, the set of properites
     * @param string sclass, the starting class to find properties for
     * */
    public function getAllProperties($sclass, $properties) {
        if ($sclass == null)
            $sclass = SONIA_DEFAULT_CATEGORY_START_ID;
        $count = 0;

        $properties = $this->convertAllPropertiesArray($properties);

        $known_properties = $this->store->getAllClassProperties($count, $sclass);

        $checks = SONIA_H :: chk($properties);
        //print_r($properties);
        //echo "\$checks $checks -------------------------------------000000000000";

        return $properties;
    }

    /**
    * provides 
    * 
    * @param array $temp_properties the list of properties
    * @param int break_custom - number of items to break the search into
    * @param array filter, a set of filters to apply, here MIN, MAX etc
    * */
    public function convertParameterLabelsToRanges($temp_properties, $break_custom, $filter) {
        //echo '$temp_properties'; print_r($temp_properties);

        //create a new temp array
        $array_temp = array ();
        $final_array = array ();

        foreach ($temp_properties as $key => $property) {
            $array_temp[$key] = $property['name'];
        }

        //sort the list of names alphabetically
        asort($array_temp);
        $total_size = count($array_temp);

        if ($total_size > $break_custom * 2) {
            $increment = $total_size / $break_custom;
            $count = 0;

            foreach ($array_temp as $key => $name) {
                if ($count % $break_custom == 0) {
                    $temp_count = 0;
                    $temp_properties[$key]['name'] = $temp_properties[$key]['name'];
                    $final_array[$key] = $temp_properties[$key];
                    $previous_key = $key;
                } else
                    if (($count +1) % $break_custom == 0) {
                        $final_array[$previous_key]['name'] = $final_array[$previous_key]['name'] . ' - ' . $temp_properties[$key]['name'];
                        $final_array[$previous_key]['count_in_search'] += $temp_properties[$key]['count_in_search'];
                        $final_array[$previous_key]['count'] += $temp_properties[$key]['count'];
                    } else {
                        $final_array[$previous_key]['count_in_search'] += $temp_properties[$key]['count_in_search'];
                        $final_array[$previous_key]['count'] += $temp_properties[$key]['count'];
                    }

                $count++;
            }
        } else //no increment can be performed
            {
            foreach ($array_temp as $key => $name) {
                $final_array[$key] = $temp_properties[$key];
            }
        }

        //break into certain number of sections
        return $final_array;
    }

    /**
     * 
     * */
    public function convertAllPropertiesArray($array) {
        $return_array = array ();

        foreach ($array as $key => $value) {
            $temp_array = $value['subclasses'];
            if (!empty ($temp_array)) {
                foreach ($temp_array as $k => $v) {
                    $k2 = key($v);
                    $v2 = array_values($v);
                    $return_array[$k2] = $v2[0];
                }
            }
        }

        return $return_array;
    }

    /**
    * Given the SONIA array of classes checks for the presence of a class in an array
    * recursively
    * 
    * @param string sclass the start class (its IRI/URI)
    * @param array $classes the set of classes in the form classes[classiri][name|subclasses][...]
    * */
    public function getAllSubclasses($sclass, $classes) {
        if ($sclass == null)
            $sclass = SONIA_DEFAULT_CATEGORY_START_ID;

        if (isset ($classes[$sclass])) {
            $return_array['aaaa']['subclasses'][$sclass] = $classes[$sclass];
            $return_array['aaaa']['subclasses'][$sclass]['class'] = $sclass;
            $return_array['aaaa']['subclasses'][$sclass]['name'] = (isset ($classes[$sclass]['name'])) ? $classes[$sclass]['name'] : $sclass;

            return $return_array;
        }

        foreach ($classes as $class) {
            if (isset ($class['subclasses'])) {
                if ($this->getAllSubclasses($sclass, $class['subclasses']) !== null) {
                    return $this->getAllSubclasses($sclass, $class['subclasses']);
                }
            }
        }
        return null;
    }

    /**
     * public function wrapper to use an IRI to find a category's superclasses from cache
     * 
     * @access public
     * @param string iri the iri to find a name for
     * @return array
     * @see SONIA_H :: _getSuperClassesRecursive()
     * */
    public function getSuperClassesRecursive($iri) {
        return SONIA_H :: _getSuperClassesRecursive($iri, $this->all_classes);
    }

    /**
    * function for retrieving the number of facets which are found within the search
    * */
    function getFacetSearchCount($facet_id) {
        $count = 0;
        if (empty ($this->search_results))
            return 0;
        foreach ($this->search_results as $key => $value) {
            foreach ($value as $k => $v) {
                if ($k == $facet_id)
                    $count++;
            }
        }
        return $count;
    }

    /**
    * function for retrieving the number of categories which are found within the search
    * */
    function getCategoryCounts($key, $category) {
        if (empty ($category['subclasses']) || !is_array($category['subclasses'])) {
            $category['count_in_search'] = ($category['count'] > 0) ? $this->_getCategoryInternalCount($key) : 0;
            $category['in_search'] = $this->categoryInSearch($key);
        } else {

            foreach ($category['subclasses'] as $k2 => $c2) {
                $category['subclasses'][$k2] = $this->getCategoryCounts($k2, $c2);
            }
        }
        return $category;
    }

    /**
    * Searches for the name of a category in cached values given an IRI
    * 
    * @param string iri the iri to find a name for
    * @param array classes a set of classes in a sonia standard associative array
    * @see getCategoryNameRecursive() 
    * */
    public function _getCategoryNameRecursive($iri, $classes) {
        foreach ($classes as $key => $class) {
            if ($key == $iri)
                return $class['name'];
            if (isset ($class['subclasses']) && is_array($class['subclasses']) && !empty ($class['subclasses'])) {
                $name = $this->_getCategoryNameRecursive($iri, $class['subclasses']);
                if ($name != null)
                    return $name;
            }
        }
        return null;
    }

    /**
    * @todo obsolete? handled in store?
    * @param array v result from SONIA store
    * */
    public function getValue($v) {
        if (isset ($v['literal'])) {
            if (!empty ($v['little_literal'])) {
                return strip_tags($v['little_literal']);
            } else
                if (!empty ($v['big_literal'])) {
                    return strip_tags($v['big_literal']);
                }
        } else {
            return $v['o'];
        }
    }

    /**
    * public function wrapper to use an IRI to find a category name from cache
    * 
    * @access public
    * @param string iri the iri to find a name for
    * @see _getCategoryNameRecursive()
    * */
    public function getCategoryNameRecursive($iri) {
        return $this->_getCategoryNameRecursive($iri, $this->all_classes);
    }

}
