<?php


 *  Copyright (C) 2006, 2007,2008  Jonathan Hendler
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 * The author can be contacted at: 
 *  - [hendlerman (at) yahoo DOT com]
 * 
 * @license GPLv2
 * @link    http://drupal.org/project/semantic_search
 */


/**
 * on install or settings update
 * go through all nodes and make sure they are synced and up to date
 * 
 * sync is one directional drupal->semantic_search
 * */
function _semantic_search_node_sync(&$form_values) {
  global $_semantic_search_search;
  if (empty ($_semantic_search_search))
    _SEMANTIC_SEARCH_INIT();
  
  //check that store is installed, auto installs for ARC
  $_semantic_search_search->sonia->store->installBackend('drupal_set_message');
  
  //erase arc_db data
  //TODO would be nice to do this more gracefully
  // but this is done because update node properties is tricky
  // since the for the store to know if a property is updated, it must
  // have the old value and the new value. Currently I do not track the old value
  if(isset ($form_values['NODE_RESET']) && ($form_values['NODE_RESET'] == 1)) {
     _semantic_search_node_index_del(); //$content_type
    $_semantic_search_search->sonia->store->truncateTables('drupal_set_message');
  }
   
  $total_node_count = _semantic_search_node_total_count();
  $batch_size = $form_values['SEMANTIC_SEARCH_IMPORT_BATCH_SIZE'];
   
  //get all cck types
  //if not active via settings
  $type_options = _content_type_info();
  foreach ($type_options['content types'] as $cck_type => $data) {
    //create, update, or delete types and fields
    $is_cck_type = true;
    $pref_name = 'display_' . $cck_type;
    $cck_uri = SONIA_CLASS_ID_URI . $cck_type;
    $total_nodes_completed =  _semantic_search_node_index_count($cck_type);
    
    //should default be 0?
    $semantic_search_set_active = variable_get($pref_name, '1');

    if (!$semantic_search_set_active) {
      //delete this class and all items
      $_semantic_search_search->sonia->deleteClass($cck_uri, true, true);
      _semantic_search_node_index_del($cck_type); //remove these nodes from the index
      watchdog('SEMANTIC_SEARCH',' deleted class ' . $cck_uri . ' and subclasses. ');

    } else if ($total_nodes_completed == 0) {
      //update class $cck_uri
      $class_label = $data['name'];
      $class_comment = $data['description'];
      //$first_param = $data['title_label'];
      _semantic_search_sync_cck_class($cck_uri, $class_label, $class_comment, array ());
      _semantic_search_taxonomy_create($cck_type);
    }
    
     
    
    if ($total_nodes_completed == 0) {
      foreach ($data['fields'] as $field_type => $field) {
        $property = SONIA_PROPERTY_ID_URI . $field_type;
  
        if (!$semantic_search_set_active) {
          //delete this property
          $_semantic_search_search->sonia->deleteProperty($property, $cck_uri);
          //this deletes all traces of CCK property, which mimicks drupal behaviour 
          //I think BAD... 
          //$_semantic_search_search->sonia->deleteProperty($cck_field_uri);
          watchdog('SEMANTIC_SEARCH',' property deleted');
        } else {
          //update this property
          $is_file_field = ($field['type'] == 'file');
          $label = $field['widget']['label'];
          $description = $field['widget']['description'];
          _semantic_search_sync_cck_field($property, $label, $description, $cck_uri, $is_file_field);
        }
      }
    }
   
    //do actual node sync
    $sql = "SELECT node.nid FROM node WHERE node.type = '$cck_type' ";
    $result = db_query($sql);
    $current_completed = 0;
  
    while ($nodet = db_fetch_object($result)) {
      if ( ($current_completed<$batch_size) && !_semantic_search_node_index_node_exists($nodet->nid)){  
        
        $node = node_load(array (
          'nid' => $nodet->nid
        ));
  
        $status = $node->status;
        $node_type = $node->type;
  
        //if it is a cck node is active in prefs and status= 1 
        if ($is_cck_type && $semantic_search_set_active && ($status == 1)) {
          
          semantic_search_nodeapi($node,'insert',true); //this handles taxonomy sync as well
        } else
          if ($status == 0 || !$semantic_search_set_active) {
            semantic_search_nodeapi($node, 'delete');
          }
        $current_completed++;
      }
    }
  }
}

/**
 * creates set of classes based on drupal taxonomy
 * */
function _semantic_search_taxonomy_create($node_type){
  global $_semantic_search_search;
  //get it's vocabularies 
  $vocabularies = taxonomy_get_vocabularies($node_type);
  
  //dbg($vocabularies,'sync index at 0 creating taxonomy _semantic_search_taxonomy_create()');
  
  foreach($vocabularies as $vid=>$vocabulary){
    $tree = taxonomy_get_tree($vocabulary->vid);
    $vid_uri = _semantic_search_cid_from_vid($vocabulary->vid);
    $v_uri = $_semantic_search_search->sonia->createClass($vocabulary->name, null, $vid_uri);
    $_semantic_search_search->sonia->createComment($v_uri, $vocabulary->description);
    watchdog('SEMANTIC_SEARCH',' class <strong>' . $vocabulary->name . '</strong> created from vocabulary.');        
    foreach($tree as $term){
      $tid_uri = _semantic_search_cid_from_tid($term->tid);
      $parents = array($vid_uri);
      foreach($term->parents as $parent){
         $parents[] =_semantic_search_cid_from_tid($parent);
      }
      $t_uri = $_semantic_search_search->sonia->createClass($term->name, $parents, $tid_uri);
      $_semantic_search_search->sonia->createComment($t_uri, $term->description);
      watchdog('SEMANTIC_SEARCH',' class <strong>' . $term->name . '</strong> created from term.');        
    }
  }
}

/**
 * todo, make this specific for a CCK type
 * */
function _semantic_search_node_index_count($content_type = null){
  if ($content_type === null)
    return db_result(db_query("SELECT COUNT(rdf_id) as count FROM {semantic_search_node_index}"));
  else
    return db_result(db_query("SELECT COUNT(rdf_id) as count FROM {semantic_search_node_index} WHERE content_type='%s'",$content_type));
}

/**
 * todo, make this specific for a CCK type
 * */
function _semantic_search_node_total_count($content_type = null){
  if ($content_type === null)
    return db_result(db_query("SELECT COUNT(nid) as count FROM {node}"));
  else
    return db_result(db_query("SELECT COUNT(nid) as count FROM {node} WHERE content_type='%s'",$content_type));
}
/**
 * */
function _semantic_search_node_index_add_node($nid,$rdf_id,$content_type,&$node){
  if( _semantic_search_node_index_node_exists($nid)){
    _semantic_search_node_index_del_node($nid);
  }
  db_query("INSERT INTO {semantic_search_node_index} (nid,rdf_id,content_type) VALUES (%d,'%s','%s') ", $nid,$rdf_id,$content_type);
}

/**
 * */
function _semantic_search_node_index_del_node($nid){
  db_query("DELETE FROM {semantic_search_node_index} WHERE nid = %d ", $nid);
}

/**
 * */
function _semantic_search_node_index_del($content_type = null){
   if ($content_type === null)
    db_query("TRUNCATE TABLE {semantic_search_node_index}");
  else
    db_query("DELETE FROM {semantic_search_node_index} WHERE content_type='%s'",$content_type);
}


/**
 * */
function _semantic_search_node_index_node_exists($nid){
   return db_result(db_query("SELECT nid FROM {semantic_search_node_index} WHERE nid= %d ", $nid));
}

/**
 * */
function _semantic_search_node_exists($nid){
   return db_result(db_query("SELECT nid FROM {node} WHERE nid= %d ", $nid));
}

/**
 * 
 * */
function _semantic_search_is_cck_type($node_type){
   return true;
   //return db_table_exists('content_type_'.$node_type);
}

/**
 * manage taxonomy parsing from nodes to classes
 * 
 * updated and refactored to handle tags and machine tags
 * @param array|object taxonomy
 * */
function _semantic_search_parse_taxonomy_to_classes(&$taxonomy,&$classes,&$vids,$a3) {
   if ($a3){
   /*    [1214] => stdClass Object
    (
        [tid] => 1214
        [vid] => 29
        [name] => Advice about Medicare
        [description] => 
        [weight] => 0
    )
        */
      foreach ($taxonomy as $tid => $tids) {
            if(!in_array($tids->vid,$vids)) $classes[$tids->vid] = _semantic_search_cid_from_vid($tids->vid);
            $vids[] = $tids->vid;
            //foreach($tids as $tid){
              //print '<br /> tid'.$tid;
              $classes[$tid] = _semantic_search_cid_from_tid($tid);
              //get each of these $tid's parents
              //although this creates reduntant data
              // this is done in place a
              // RDF store's lack of class inference
              if (SEMANTIC_SEARCH_TAXONOMY_IMPORT_INFERENCE){
                $parents = taxonomy_get_parents_all($tid);
                
                foreach($parents as $parent){
                  $classes[$parent->tid] = _semantic_search_cid_from_tid($parent->tid);
                }
              }
           // }
        }
    }
    else{
      foreach ($taxonomy as $vid => $tids) {
          if(!in_array($vid,$vids)) $classes[$vid] = _semantic_search_cid_from_vid($vid);
          $vids[] = $vid;
          foreach($tids as $tid){
            //print '<br /> tid'.$tid;
            $classes[$tid] = _semantic_search_cid_from_tid($tid);
            //get each of these $tid's parents
            //although this creates reduntant data
            // this is done in place a
            // RDF store's lack of class inference
            if (SEMANTIC_SEARCH_TAXONOMY_IMPORT_INFERENCE){
              $parents = taxonomy_get_parents_all($tid);
              
              foreach($parents as $parent){
                $classes[$parent->tid] = _semantic_search_cid_from_tid($parent->tid);
              }
            }
          }
        }
    }
} 

/**
 * standard drupal nodeapi hook
 * 
 * support for taxonomy, CCK (types, fields, and nodes), 
 * and (planned) arbitrary nodereference
 * 
 * @todo create true inheretence
 * @todo only add data on publish
 * 
 */
function semantic_search_nodeapi(& $node, $op, $a3 = NULL, $a4 = NULL) {
    if($op == 'load' || $op == 'validate' || $op == 'prepare' || $op == 'view' 
    || $op == 'alter' || $op == 'submit') return;
    
    
    //hack to allow 
    //editing of nodes with duplicate field values
    if ($op == 'update'){
      semantic_search_nodeapi($node,'delete'); 
      //dbg($node,$op);
      $op = 'insert';
    }      
    
    //dbg($node,$op);
    
    global $_semantic_search_search;

    $node_type = $node->type;
    $node_id = $node->nid;
    $node_status = $node->status;

    //internally always try to delete the node if it is unpublished
    if (!$node_status)
        $op = 'delete';
    
    $is_cck_type = _semantic_search_is_cck_type($node_type);
    
    
    $pref_name = 'display_' . $node_type;
    $pref_value = variable_get($pref_name, '1');
    
    //dbg('$is_cck_type '.$is_cck_type,'$node_type '.$node_type.', $pref_value='.$pref_value);

    // add this if pages are to tracked and searched in RDF as well
    //|| ($node_type == 'page')

    $activate_rdf_sync = ($is_cck_type) && !empty ($node_id) && $pref_value;
    //prevent SEMANTIC_SEARCH sync
    if (!$pref_value && $is_cck_type) {
        global $cck_notice_count;
        if (!isset ($cck_notice_count))
            $cck_notice_count = 0;

        $link = url('admin/settings/semantic_search', null, null, true);
        $title = "Activate the CCK type for SEMANTIC_SEARCH here";
        if (user_access('admin semantic_search')) {
            $cck_notice_count++;
            if ($op != '' && $cck_notice_count == 1) {
                //too annoying
                //drupal_set_message('SEMANTIC_SEARCH admin Notice: CCK type "'.$node_type
                //  .'" is not synced with SEMANTIC_SEARCH RDF. Activate <a href="'.$link.'" title="'.$title.'">here</a>');
            }
        }
        return;
    }

    if ($activate_rdf_sync || !$node_status) {
        if (empty ($_semantic_search_search))
            _SEMANTIC_SEARCH_INIT();
        //drupal_set_message( "id=$node_id  op=$op \$node_type = $node_type \$cck_type =$cck_type<br/> ");
        // drupal_set_message( <br/> ");
        // dbg($node ,"id=$node_id  op=$op \$node_type = $node_type \$cck_type =$cck_type");

        //TODO only add if status is published or "1"
        $id = $node->nid;
        $taxonomy = $node->taxonomy;

        //$uri            = $_semantic_search_search->makeNodeURI($id);
        $uri = _semantic_search_getCCKOBJECTIDFromNode($node);
        $label = SONIA_H :: prep_input($node->title);
        $primary_class = _semantic_search_getCCKIDFromNode($node);
        //$primary_class  = $_semantic_search_search->makeNodeURI($node_type,'admin/node/types/');
        //$taxonomy = $node->taxonomy; ///$_semantic_search_search->convertDrupalTaxonomyToSoniaCategories($node->taxonomy);
        ////$id_iri = $_semantic_search_search->sonia->createClass($label, $superclasses,$uri); // for taxonomies
        $classes = array (
            $primary_class
        );
         
        //add taxonomy and reset
        $vids = array();
        if (!empty ($taxonomy)) {
           _semantic_search_parse_taxonomy_to_classes($taxonomy,$classes,$vids,$a3);
        }  
        
       // dbg($classes,'$classes $a3='.$a3);
       // dbg($parents,' $parents and SEMANTIC_SEARCH_TAXONOMY_IMPORT_INFERENCE '.SEMANTIC_SEARCH_TAXONOMY_IMPORT_INFERENCE);
        //dbg($classes, ' $classes op= '.$op);
        
        switch ($op) {
            case 'insert' :
                // create instance of a class
                $parameters = _semantic_search_getNodeFieldData($node);
                //dbg($parameters,'INSERT $parameters '.$label);
                $rdf_id = $_semantic_search_search->sonia->createObject($label, 
                $classes, $parameters, $uri,true);
                _semantic_search_node_index_add_node($node_id,$rdf_id,$node->type,$node);
                $_semantic_search_search->sonia->io->clearCache();
                watchdog('SEMANTIC_SEARCH',' <strong>' . $label . '</strong> created');
                break;
            case 'update' :
                $parameters = _semantic_search_getNodeFieldData($node);
                
                $_semantic_search_search->sonia->updateObject($uri, $label, $classes, $parameters);

                
                $_semantic_search_search->sonia->io->clearCache();
                watchdog('SEMANTIC_SEARCH',' <strong> ' . $label . '</strong> updated');
                break;
            case 'delete' :
                $_semantic_search_search->sonia->deleteObject($uri);
                _semantic_search_node_index_del_node($node->nid);
                //do the same to coresponding data in the RDF DB
                $extra = ($node_status) ? ' deleted. ' : ' deleted from search because it was set to unpublished.';
                $_semantic_search_search->sonia->io->clearCache();
                watchdog('SEMANTIC_SEARCH','  <strong>' . $label . '</strong>  ' . $extra);
                break;
            case 'view' :
                break;
        }
    } //end rdf sync area
}

/**
 * convert a Drupal VID to a SEMANTIC_SEARCH class id
 * define('SEMANTIC_SEARCH_TID_PREFIX','tid'); 
define('SEMANTIC_SEARCH_VID_PREFIX','vid'); 
 * */
function _semantic_search_cid_from_vid($vid) {
    return SONIA_CLASS_ID_URI . SEMANTIC_SEARCH_VID_PREFIX . $vid;
}

/**
 * convert a Drupal TID to a SEMANTIC_SEARCH class id
 * */
function _semantic_search_cid_from_tid($tid) {
    return SONIA_CLASS_ID_URI . SEMANTIC_SEARCH_TID_PREFIX . $tid;
}

/**
 * Hook into drupal taxonomy
 * used here to sync with RDF on admin of taxonomy
 * */
function semantic_search_taxonomy($op, $type, $object = null) {
    
    global $_semantic_search_search;
    if (empty ($_semantic_search_search))
      _SEMANTIC_SEARCH_INIT();
      
    //get cck types associated
    //op form, update
    $actionable = ($op == 'delete') || ($op == 'create') || ($op == 'insert') || ($op == 'update');

//    if ($actionable) {
//        $node_types = $object['nodes'];
//        $is_cck_type = false;
//        foreach ($node_types as $node_type) {
//            //TODO check to see that it is enabled in adming/settings/semantic_search
//            $pref_name = 'display_' . $node_type;
//            variable_get($pref_name, '1');
//            $is_cck_type = $is_cck_type || variable_get($pref_name, '1');
//        }
//
//        $actionable = $actionable && $is_cck_type;
//    }

    if (($type == 'vocabulary') && $actionable) {
        $vid = $object['vid'];
        $vid_uri = _semantic_search_cid_from_vid($vid);
        $name = $object['name'];
        $description = $object['description'];
        
        switch ($op) {
            case 'create' :
                $v_uri = $_semantic_search_search->sonia->createClass($name, null, $vid_uri);
                $_semantic_search_search->sonia->createComment($v_uri, $description);
                $_semantic_search_search->sonia->io->clearCache();
                watchdog('SEMANTIC_SEARCH','class ' . $name . ' created.');
                break;
            case 'update' :
                $_semantic_search_search->sonia->updateClass($vid_uri, $name, null);
                $_semantic_search_search->sonia->updateComment($vid_uri, $description);
                $_semantic_search_search->sonia->io->clearCache();
                watchdog('SEMANTIC_SEARCH',' class ' . $name . ' updated.');
                break;
            case 'delete' :
                $v_uri = $_semantic_search_search->sonia->deleteClass($vid_uri);
                $_semantic_search_search->sonia->io->clearCache();
                watchdog('SEMANTIC_SEARCH',' class ' . $name . ' and subclasses deleted.');
                break;
        }
    } else
        if (($type == 'term') && $actionable) {
            $vid = $object['vid'];
            $tid = $object['tid'];
            $vid_uri = _semantic_search_cid_from_vid($vid);
            $tid_uri = _semantic_search_cid_from_tid($tid);
            $superclasses_ = $object['parent'];

            $superclasses = array ();

            if (!empty ($superclasses_)) {
                foreach ($superclasses_ as $superclass) {
                    $superclasses[] = _semantic_search_cid_from_tid($superclass);
                }
            }  
           
           $superclasses[] = $vid_uri;
           //hack fix for http://drupal.org/node/236126 
          //     $superclasses[] = "http://www.ahirc.org/namespaces/semantic_search/Class#ahirc_information_resource";
             

            //TODO
            //related

            $name = $object['name'];
            $description = $object['description'];

            switch ($op) {
                case 'insert' :
                    $t_uri = $_semantic_search_search->sonia->createClass($name, $superclasses, $tid_uri);
                    $_semantic_search_search->sonia->createComment($t_uri, $description);
                    $_semantic_search_search->sonia->io->clearCache();
                    watchdog('SEMANTIC_SEARCH',' class ' . $name . ' created.');
                    break;
                case 'update' :
                    $_semantic_search_search->sonia->updateClass($tid_uri, $name, $superclasses);
                    $_semantic_search_search->sonia->updateComment($tid_uri, $description);
                    $_semantic_search_search->sonia->io->clearCache();
                    watchdog('SEMANTIC_SEARCH','class ' . $name . ' updated.');
                    break;
                case 'delete' :
                    $v_uri = $_semantic_search_search->sonia->deleteClass($tid_uri);
                    $_semantic_search_search->sonia->io->clearCache();
                    watchdog('SEMANTIC_SEARCH',' class ' . $name . ' and subclasses deleted.');
                    break;
            }
        }
}

//seperate function for update, create, delete to be used from
// semantic_search taxonomy, and other syncs?

////////////////////////////////////////////////////////////////////////////////
// EXPERIMENTAL HOOKS
//function semantic_search_field_settings($op, $field)
//{
//  dbg($op,'$op semantic_search_field_settings()');
//  dbg($field,' $field semantic_search_field_settings()');
//}
//
//function semantic_search_field($op, &$node, $field, &$node_field, $teaser, $page)
//{
//  dbg($op,'$op semantic_search_field()');
//  dbg($field,' $field semantic_search_field()');
//}
//
//function semantic_search_widget($op, &$node, $field, &$node_field, $teaser, $page)
//{
//   dbg($op,'$op semantic_search_widget()');
//  dbg($field,' $field semantic_search_widget()');
//}
// END EXPERIMENTAL HOOKS
////////////////////////////////////////////////////////////////////////////////

/**
 * This hook into CCK synchronizes changes to fields for content types against
 * the RDF data (RDFS propertie es which belong to classes, and their labels)
 * 
 * @todo depends on op, kind of a hack, would work if internationalized
 * @todo should use http://drupal.org/node/82661
 * @todo label update/create 2nd on phase 2 of of creating a field
 * */
function semantic_search_form_alter($form_id, & $form) {
    
    if (isset ($_REQUEST['op'])) {
        $op = $_REQUEST['op'];
    } else {
        $op = null;
    }

    //drupal_set_message( "  \$form_id=$form_id  <br/> ".dbg($form,"\$op = $op"));
    //find when the content type is being modified
    //hack for drupal form messiness
    if ($form_id == '_content_admin_type_edit') {
        $label = _semantic_search_getLabelFromForm($form);
    }

    $is_cck_type = preg_match('/^content_/i', $form_id);
    if ($is_cck_type) {
        $cck_id = _semantic_search_getCCKIDFromForm($form, true);
        $pref_name = 'display_' . $cck_id;
        $pref_value = variable_get($pref_name, '1');
        if (!$pref_value) {
            $link = url('admin/settings/semantic_search', null, null, true);
            $title = "Activate the CCK type for SEMANTIC_SEARCH here";
            if ($op != '')
                drupal_set_message('SEMANTIC_SEARCH: CCK object is not synced with RDF. Activate <a href="' . $link . '" title="' . $title . '">here</a>');
            return;
        }
    }

    switch ($form_id) {
        case '_content_admin_type_edit' :
            if ($label != '') {
                //TODO differentiate create and update which both use this id
                // $label        = _semantic_search_getLabelFromForm($form); //SONIA_H :: prep_input ($form['label']['#default_value']);
                $title_label_property = SONIA_PROPERTY_ID_URI . 'title_label';
                $description = SONIA_H :: prep_input($form['description']['#default_value']);
                //id is a hash of names
                $uri = _semantic_search_getCCKIDFromForm($form);
                //echo $form['type_name']['value'];
                //$title_label = _semantic_search_getCCKTitleLabelFromForm($form);
                //if (!$this->sonia->propertyExists('title_label'))
                //{
                //  $title_label_property = $_semantic_search_search->sonia->createProperty($title_label,null,$title_label_property);
                //}

                $superclasses = array ();

                _semantic_search_sync_cck_class($uri, $label, $description, $superclasses);
            }
            // drupal_set_message( "  \$form_id=$form_id  <br/> ".dbg($form));
            break;
        case '_content_admin_field_add_new' :
        case '_content_admin_field' :
            if ($op == 'Save field settings' || $op == 'Create field') {
                $cck_uri = _semantic_search_getCCKIDFromForm($form);
                $label = _semantic_search_getLabelFromForm($form);
                $cck_field_uri = _semantic_search_getCCKFieldIDFromForm($form);
                $is_file_field = _semantic_search_checkIsFileType($form);
                $description = SONIA_H :: prep_input($form['widget']['description']['#default_value']);
                _semantic_search_sync_cck_field($cck_field_uri, $label, $description, $cck_uri, $is_file_field);
            }
            // drupal_set_message( "  \$form_id=$form_id  <br/> ".dbg($form));
            break;
        case '_content_admin_type_delete' :
            if ($op == 'delete' || $op == 'Delete') {
                //note: I don't know if this delete the whole field or a CCK nodes reference to it.
                // get class
                $cck_uri = _semantic_search_getCCKIDFromForm($form);
                //get all related nodes/classes and delete them too
                // param 2 delete subclass, param 3 delete related objects
                // deletes all subclasses and all related elements of content ...
                // consistant with CCK behaviour
                $_semantic_search_search->sonia->deleteClass($cck_uri, true, true);
                $_semantic_search_search->sonia->io->clearCache();
                watchdog('SEMANTIC_SEARCH',' deleted class ' . $cck_uri . ' and subclasses. ');
            }
            break;
        case '_content_admin_field_remove' :
            if ($op == 'Remove') {
                $cck_uri = _semantic_search_getCCKIDFromForm($form);
                $cck_field_uri = _semantic_search_getCCKFieldIDFromForm($form);
                //this won't delete all traces of the property in data
                $_semantic_search_search->sonia->deleteProperty($cck_field_uri, $cck_uri);
                //this deletes all traces of CCK property, which mimicks drupal behaviour 
                //I think BAD... 
                //$_semantic_search_search->sonia->deleteProperty($cck_field_uri);
                watchdog('SEMANTIC_SEARCH',' property deleted =' . $cck_field_uri);
            }
            break;
        case '_content_admin_field_add_existing' :
            if ($op == 'Add field') {
                $cck_uri = _semantic_search_getCCKIDFromForm($form);
                $cck_field_uri = _semantic_search_getCCKFieldIDFromForm($form);
                $_semantic_search_search->sonia->addPropertyToClass($cck_uri, $cck_field_uri);
                watchdog('SEMANTIC_SEARCH',' field added from existing set of Drupal fields');
            }
            break;
        default :

            break;

    }
}

//HELPERS

function convertDrupalName($label) {
    return str_replace(' ', '_', trim(strtolower($label)));
}

/**
 *  get the parameter names and data from a node
 * 
 * @todo may be different for images or irregular field types
 * */
function _semantic_search_getNodeFieldData(& $node) {
    global $_semantic_search_search;
    
    $parameters = array ();
    $count = 0;
    foreach ($node as $key => $value) {
        
        if (preg_match('/^field_/i', $key)) {
            if (is_array($value)) {
                $is_file_field = $_semantic_search_search->sonia->isFileType(SONIA_PROPERTY_ID_URI . $key);
                $is_file_field = false;
                //dbg($value,'$value with images '.$is_file_field.' '. SONIA_PROPERTY_ID_URI.$key);
                //if the parameter is a file or image
                if ($is_file_field) {
                    foreach ($value as $item) {

                        //
                        if ($item['remove'] == 1) {
                            $image_uri = $_semantic_search_search->sonia->store->getFileInfoByFilename($item['filename'], true);
                            $_semantic_search_search->sonia->deleteAll($image_uri);
                            drupal_set_mesage("SEMANTIC_SEARCH {$item['filename']} removed.");
                        } else {
                            $parameters['images'][$count]['filename'] = $item['filename'];
                            $parameters['images'][$count]['filesize'] = $item['filesize'];
                            $parameters['images'][$count]['filepath'] = $item['filepath'];
                            $parameters['images'][$count]['id'] = $item['fid'];
                            $parameters['images'][$count]['uri'] = SONIA_PROPERTY_ID_URI . $key; //dbg($parameters,' $parameters images');
                            $count++;
                        }
                    }
                } else {
                    foreach ($value as $v) {
                        //cck field type URL
                        if (isset($v['url'])){
                          $parameters[$count]['value'] =$v['url'];
                        }
                        else { 
                          $parameters[$count]['value'] = $v['value'];
                        }

                        $parameters[$count]['uri'] = SONIA_PROPERTY_ID_URI . $key;
                        $count++;
                    }
                }
            } else // also include blank values if updated so they will be deleted
                {
                if (isset($v['url'])){
                  $parameters[$count]['value'] =$v['url'];
                }
                else { 
                  $parameters[$count]['value'] = $v['value'];
                }
                $parameters[$count]['uri'] = SONIA_PROPERTY_ID_URI . $key;
                $count++;
            }
        }
    }
    
    return $parameters;
}

/**
 * synchronize data on a cck class
 * */
function _semantic_search_sync_cck_class($uri, $label, $description, $superclasses = array (), $properties = array ()) {
    global $_semantic_search_search;
    if (empty ($_semantic_search_search))
      _SEMANTIC_SEARCH_INIT();
    
    $exists = $_semantic_search_search->sonia->classExists($uri);
    //drupal_set_message( "  \$form_id=$form_id  <br/> ".dbg($form,"\$op = $op"));
    if ($exists) {
        $id_iri = $_semantic_search_search->sonia->updateClass($uri, $label, $superclasses);
        $_semantic_search_search->sonia->updateComment($uri, $description);
        //create object title property
        //
        //add property to class/cck
        //$_semantic_search_search->sonia->updateProperty($title_label_property,$title_label_property,array($uri)); 
        $_semantic_search_search->sonia->io->clearCache();
        watchdog('SEMANTIC_SEARCH',' class ' . $label . ' updated as ' . $uri);
    } else {
        //create class
        $id_iri = $_semantic_search_search->sonia->createClass($label, $superclasses, $uri);
        //create description property
        $_semantic_search_search->sonia->createComment($uri, $description);
        //update title field
        //$_semantic_search_search->sonia->addPropertyToClass($uri,$title_label_property);
        $_semantic_search_search->sonia->io->clearCache();
        watchdog('SEMANTIC_SEARCH',' class ' . $label . ' created as ' . $uri);
    }
}

/**
* synce field info
* */
function _semantic_search_sync_cck_field($cck_field_uri, $label, $description, $cck_uri, $is_file_field) {
    global $_semantic_search_search;
    if (empty ($_semantic_search_search))
      _SEMANTIC_SEARCH_INIT();
    //check that ID is not blank
     
    $temp_id = SONIA_H :: getIDFromURI($cck_field_uri);
    //see if property exists
    $exists = $_semantic_search_search->sonia->propertyExists($cck_field_uri);
   // dbg($cck_field_uri,$exists.' PING' );
    $class_array_one = array (
        $cck_uri
    );

    //drupal_set_message('$temp_id = '.$temp_id.dbg($form,"\$op = $op"));
    if ($temp_id != '') {
        if ($exists) {
            //update the property
            $_semantic_search_search->sonia->updateProperty($cck_field_uri, $label, $class_array_one);
            //update the  property values in class/cck
            $_semantic_search_search->sonia->io->clearCache();
            watchdog('SEMANTIC_SEARCH',' updated property. ' . $cck_field_uri);
        } else {
            //create the property

            $property1 = $_semantic_search_search->sonia->createProperty($label, $class_array_one, $cck_field_uri);
            //dbg(array($is_file_field,$property1),'$is_file_field - $property1');
            if ($is_file_field) {
                $_semantic_search_search->sonia->io->clearCache();
                $_semantic_search_search->sonia->setAsImageType($property1);
                $extra = ' (Property is a special file property.) ';
            } else {
                $extra = '';
            }
            //add property to class/cck
            $_semantic_search_search->sonia->addPropertyToClass($cck_uri, $cck_field_uri);
            $_semantic_search_search->sonia->io->clearCache();
            watchdog('SEMANTIC_SEARCH',' added new property. ' . $extra . $cck_field_uri);
        }
    }
}

/**
 * 
 * */
function _semantic_search_getLabelFromForm(& $form) {
    if (isset ($form['widget']['label']['#default_value']) && $form['widget']['label']['#default_value'] != '') {
        return SONIA_H :: prep_input($form['widget']['label']['#default_value']);
    } else
        if (isset ($form['label']['#default_value']) && $form['label']['#default_value'] != '') {
            return SONIA_H :: prep_input($form['label']['#default_value']);
        } else
            if (isset ($form['new']['label']['#default_value']) && $form['new']['label']['#default_value'] != '') {
                return SONIA_H :: prep_input($form['new']['label']['#default_value']);
            }

}

/**
 * get the unique URI for the CCK type
 * */
function _semantic_search_getCCKOBJECTIDFromNode(& $node, $return_id = false) {
    global $_semantic_search_search;
    if (!empty ($node->type)) {
        $id = $node->nid;
    }

    if ($return_id)
        return $id;
    return SONIA_OBJECT_ID_URI . $id;
}

function _semantic_search_checkIsFileType(& $form) {
    if (isset ($form['widget']['widget_type']['#default_value']) && $form['widget']['widget_type']['#default_value'] == 'file') {
        return true;
    }
    return false;
}

/**
 * get the unique URI for the CCK type
 * */
function _semantic_search_getCCKIDFromNode(& $node, $return_id = false) {
    global $_semantic_search_search;
    if (!empty ($node->type)) {
        $id = $node->type;
    }

    if ($return_id)
        return $id;
    return SONIA_CLASS_ID_URI . $id;
}

/**
 * get the unique URI for the CCK type
 * */
function _semantic_search_getCCKIDFromForm(& $form, $return_id = false) {
    global $_semantic_search_search;
    if (!empty ($form['type_name']['#value'])) {
        $id = SONIA_H :: prep_input($form['type_name']['#value']);

    } else
        if (!empty ($form['field']['type_name']['#value'])) {
            $id = SONIA_H :: prep_input($form['field']['type_name']['#value']);
        } else
            if (!empty ($form['existing']['type_name']['#value'])) {
                $id = SONIA_H :: prep_input($form['existing']['type_name']['#value']);
            } else
                if (!empty ($form['new']['type_name']['#value'])) {
                    $id = SONIA_H :: prep_input($form['new']['type_name']['#value']);
                }
    if ($return_id)
        return $id;
    return SONIA_CLASS_ID_URI . $id;
    //$_semantic_search_search->makeNodeURI($id,'admin/node/types/');
}

/**
 * get the unique URI for the CCK field
 * */
function _semantic_search_getCCKFieldIDFromForm(& $form) {
    global $_semantic_search_search;
    $cck_id = _semantic_search_getCCKIDFromForm($form, true);
    if (!empty ($form['field']['field_name']['#value'])) {
        $id = SONIA_H :: prep_input($form['field']['field_name']['#value']);
    } else
        if (!empty ($form['field_name']['#value'])) //HACK better way?
            {
            $id = SONIA_H :: prep_input($form['field_name']['#value']);
            //drupal_set_message( dbg($_REQUEST,"got $id")); 
        } else
            if (!empty ($_REQUEST['edit']['field_name'])) //HACK better way?
                {
                $id = SONIA_H :: prep_input($_REQUEST['edit']['field_name']);
                //drupal_set_message( dbg($_REQUEST,"got $id")); 
            }

    return SONIA_PROPERTY_ID_URI . $id;
    //$_semantic_search_search->makeNodeURI($id,'admin/node/types/'.$cck_id.'/fields/');
}

/**
 * get the unique URI for the CCK field
 * */
function _semantic_search_getCCKTitleLabelFromForm(& $form) {
    global $_semantic_search_search;
    $cck_id = _semantic_search_getCCKIDFromForm($form, true);
    if (!empty ($form['field']['title_label']['#default_value'])) {
        $id = SONIA_H :: prep_input($form['field']['title_label']['#default_value']);
    } else
        if (!empty ($form['title_label']['#value'])) //HACK better way?
            {
            $id = SONIA_H :: prep_input($form['title_label']['#default_value']);
        }

    //else if (!empty($_REQUEST['edit']['field_name'])) //HACK better way?
    //{
    //$id            = SONIA_H :: prep_input ($_REQUEST['edit']['field_name']);
    //}
    //SONIA_PROPERTY_ID_URI.
    return $id;
    //$_semantic_search_search->makeNodeURI($id,'admin/node/types/'.$cck_id.'/fields/');
}

/**
 * get CCK content types
 * */
function _semantic_search_get_content_types() {
  $options = array ();
  $count = 0;

  $type_info = _content_type_info();
  $types = $type_info['content types'];

  foreach ($types as $id => $info) {
    $options[$id] = $info['label'];
    $count++;
  }
  return $options;
}

