<?php
/*
 * Copyright (C) 2006, 2007,2008  Jonathan Hendler
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 * The author can be contacted at: 
 *  - [hendlerman (at) yahoo DOT com]
 * 
 * @license GPLv2
 * @link    http://drupal.org/project/semantic_search
 */
/**
 * The Following search class is a collection of functions used in the SEMANTIC_SEARCH module.
 * It provides the API for constructing search elements
 * 
 * 
 * @todo should be more configurable for data including the search terms
 * @author Jonathan Hendler 
 * @package SEMANTIC_SEARCH
 * 
 * */
class NinaSearch {
  /**
   * the SONIA api search tool
   * */
  public $sonia;

  /**
   * semantic_search search params
   * */
  public $search_params;

  /**
   * controls if users are allowed to create, update or delete data
   * */
  private $allowCrud;

  /**
   * The constructor initiates a SONIA object 
   * */
  function __construct() {

    //__nsp(' NinaSearch() start');
    //construct SONIA (see sonia.conf.inc for constants)
    $store_type = variable_get('semantic_search_rdf_store_choice','sonia_arc_local');
    $this->sonia = new SONIA($store_type, SONIA_FORM_ARRAY_PREFIX);

    $this->search_hash = $this->getSearchHash();
    // build search query
    $this->search_params = $this->sonia->getSearchParams();
    
    if ($this->doSearch()) {
      $this->search_results = $this->sonia->getSearchResults();
      $this->object_search_result_ids = $this->sonia->store->object_search_result_ids;
    }
    
  }

  private function getSearchHash() {
    //cache, at this leve needs to know if the page has been cached
    //  specific to pagination page
    $pgnation = $this->sonia->io->getPaginationBegin();

    if (empty ($this->sonia->sonia_search_array)) {
      $key = 'empty';
    } else {
      $key = array ();
      foreach ($this->sonia->sonia_search_array as $q) {
        $key[$q['predicate']] = $q['subject'] . $q['object'];
      }
    }
    return SO_CA :: gh($key, $pgnation, true);
  }

  /**
   * used to capture params from search params
   * 
   * note: 
   *   uri must be not encoded
   * */
  function searchedForURI($uri, $type) {
    
    if (is_array($this->search_params[$type])) {
      foreach ($this->search_params[$type] as $kur => $asset) {
        if (!is_array($asset)) {
          if ($kur == $uri)
            return true;
          if ($asset == $uri)
            return true;
        } else {
          foreach ($asset as $key => $value) {
            if ($key == $uri &&  ($type != 'facet')) {
              return true;
            }
            if ($value == $uri){
              return true;
            }
          }
        }
      }
    }
    return false;
  }

  /**
   * count the number of facets found in search
   * */
  function countInSearch($uri, $type,$predicate= null) {
    if (empty ($this->search_results) )
      return 0;
    $count = 0;
     
    if ($type == 'cat') {
    	
      return $this->sonia->store->getObjectsCountInSearch( $uri,SONIA_RDF_CREATE_TYPE, false, false);
         
    } else
      if ($type == 'facet') {
       return $this->sonia->store->getObjectsCountInSearch(null,$uri);  
          
      } else
        if ($type == 'p_value') {
          return $this->sonia->store->getObjectsCountInSearch($uri,$predicate);
        }

    return $count;
  }
  
  /**
   * 
   * */
  public function getAllBaseOptions(){
    $all_base_elements = $this->sonia->getBaseElementsAndCounts();
    $uris = array();
    $type_name = '';
    
    foreach($all_base_elements as $type => $data){
      $uris[]  = strtoupper($type);
      foreach($data as $uri =>$values) {
         $uri = SONIA_H :: encodeURI($uri);
         $uris[$uri]  = '-'.$values['label'] . '('.$values['count'].')';
      }
    }
    
    return $uris;
  }
  
  //---------------------------------------------------------------------------
  /**
   * The generalized function which allows those creating semantic_search.app.inc to build
   * custom search elements.
   * 
   * It's all about abstraction. 
   * 
   * 
   * @param array $form_element_options
   * $form_element_options['#query_type']  =  'class', 'property', 'object','p_value',fulltext
   
   * $form_element_options['uris']         = array of uris as top level elements
   * $form_element_options['orders']       = a general setting to the search results - [URI]=>wieght(int)
   
   * * $form_element_options['#type']        = 'checkbox','textfield','select', 'radio'
   * $form_element_options['#multiple_facet']    =  allow multiple facets to be searched on without overwriting
   * $form_element_options['#return_type'] =  'list_objects' 'list_properties' 'list_values'
   * $form_element_options['#title']       = $title title/label for the search item
   * $form_element_options['#min']         = (value integero or alphbet to do telescoping range searches)
   * $form_element_options['#max']         = (value integero or alphbet to do telescoping range searches)
   * $form_element_options['#description'] = string $description description of the field 
   * $form_element_options['#sort_order']  = asc | desc (refers to list, not results)
   * $form_element_options['#order_by']    = alpha | count (refres to list, not results)
   * $form_element_options['#return_counts']  = boolean - return numbers
   * $form_element_options['#grand_total']    = return total results (in select )
   * $form_element_options['#title_as_option']    = title should not display as a label, but as an defaul option (in selects)
   * $form_element_options['#rpc']    = will the functions use javascript to query the page
   * 
   * 
   * @param string $order_by eg. 'label ASC'
   * @param boolean $return_counts include count information, can be a performance hit
   * 
   * @see SONIA :: getCustomSearchParamsData()
   * @see SONIA :: getSearchParamsData()
   * 
   * @todo add support for multiple field search instead of all or one on fulltext
   * */
  //---------------------------------------------------------------------------
  public function getSearchFormElement($form_element_options) {
    $result_array = $form_element = array ();
    $prefix = '';

    //defaults
    if (!isset ($form_element_options['values']))
      $form_element_options['values'] = null;
    if (!isset ($form_element_options['#display_if_count_zero']))
      $form_element_options['#display_if_count_zero'] = true;
      
    if (!isset ($form_element_options['#title']))
      $form_element_options['#title'] = '';
    if (!isset ($form_element_options['#description']))
      $form_element_options['#description'] = '';
    if (!isset ($form_element_options['#return_counts']))
      $form_element_options['#return_counts'] = true;
    if (!isset ($form_element_options['#grand_total']))
      $form_element_options['#grand_total'] = true;
    if (!isset ($form_element_options['#order_by']))
      $form_element_options['#order_by'] = 'alpha';
    if (!isset ($form_element_options['#sort_order']))
      $form_element_options['#sort_order'] = 'asc';
    if (!isset ($form_element_options['#multiple_facet']))
      $form_element_options['#multiple_facet'] = false;
    if (!isset ($form_element_options['#title_as_option']))
      $form_element_options['#title_as_option'] = false;
    if (!isset ($form_element_options['#rpc']))
      $form_element_options['#rpc'] = false;
    if (!isset($form_element_options['#sort_sub_by_weight'])){
      $form_element_options['#sort_sub_by_weight'] = false;
    }
    
    /* TODO should become a switch of choices about how to sort
     * */
    $alphasort = $form_element_options['#order_by'];//(empty ($form_element_options['#order_by']) || $form_element_options['#order_by'] != 'num');

    
    
    if (!$form_element_options['#title_as_option']) {
      $form_element['#title'] = $form_element_options['#title'];
      $form_element['#description'] = $form_element_options['#description'];
    }
    
    /// support for Form API 
    if (isset($form_element_options['#prefix'])){
      $form_element['#prefix'] = $form_element_options['#prefix'];
    }
    
    if (isset($form_element_options['#suffix'])){
      $form_element['#suffix'] = $form_element_options['#suffix'];
    }
    
    if (isset($form_element_options['#id'])){
      $form_element['#id'] = $form_element_options['#id'];
    }
    //end Form API support
    
    
    $search_name = (isset($form_element_options['#search_name'])) ? '/'.$form_element_options['#search_name']: '';
    $multi_f = $form_element_options['#multiple_facet'];
    //check URIs
    if (!is_array($form_element_options['uris']))
      $form_element_options['uris'] = array (
        $form_element_options['uris']
      );
    $uris = $form_element_options['uris'];
    //what if a query involves custom classes or properties?
    //for now assuming queries and classes don't need both
    
    
    switch ($form_element_options['#query_type']) {
      case 'class' :
        //$form_element['#name'] = 'cat';
        $prefix = 'cat';
        break;
      case 'property' :
        $prefix = 'facet';
        break;
      case 'p_value' :
        //works like text search becauase it's by values
        $prefix = 'p_value';
        break;
      case 'object' :

        break;
      case 'fulltext' :
        //todo
        $rvalue = (isset ($uris[0])) ? SONIA_H :: encodeURI($uris[0]) : '';
        $qname = (!$multi_f) ? 'edit[qstring][' . $rvalue.']' : '[qstring][][' . $rvalue.']';
        if (isset($form_element_options['#title'])) $form_element[$qname]['#title'] = $form_element_options['#title'];
        if (isset($form_element_options['#id'])) $form_element[$qname]['#id'] = $form_element_options['#id'];
        
      //  $qname = SONIA_H :: decodeURI($qname);
        $form_element[$qname]['#description'] = $form_element_options['#description'];
        $form_element[$qname]['#type'] = 'textfield';
        $form_element[$qname]['#size'] = SEMANTIC_SEARCH_TEXT_INPUT_FORM_WIDTH;
        return $form_element;
        break;
      default :
        $message = $form_element_options['#query_type'] . ' is not a valid $query_type for getSearchFormElement()';
        $this->sonia->io->throwError($message);
        break;
    }

    //javascript?
    //form_element['#attributes'] = '';

    if ($form_element_options['#return_type'] == 'taxonomy_hack') {
      $result_array = $this->sonia->taxonomyForm($form_element_options['values']);
      
    } else {
      $result_array = $this->sonia->getCustomSearchParamsData($form_element_options['#query_type'], $uris, $form_element_options['#return_type'], $form_element_options['values'], $form_element_options['#order_by'], $form_element_options['#return_counts']);
    }
    if (empty($result_array)){
      $result_array = array();
    }
    
    switch ($form_element_options['#type']) {
      case 'select' :
        $array_prefix = null;
        if ($prefix == 'p_value') { //TODO maybe others
          $rvalue = SONIA_H :: encodeURI($uris[0]);
          $array_prefix = (!$multi_f) ? "p_value][$rvalue" : "p_value][][$rvalue";
          if (SEMANTIC_SEARCH_IS_DRUPAL_5)
            $array_prefix = "edit[$array_prefix]";
        }
        
        if ($prefix == 'facet') { //TODO maybe others
          //$rvalue = SONIA_H :: encodeURI($uris[0]);
          $array_prefix = (!$multi_f) ? "facet][" : "facet][][";
          if (SEMANTIC_SEARCH_IS_DRUPAL_5)
            $array_prefix = "edit[$array_prefix]";
        }

        $tarray = array ();
        $form_element['#type'] = $form_element_options['#type'];
        //if js
        if ($form_element_options['#rpc']) {
          $path = url('semantic_search_ajax/search'.$search_name, null, null, true);
          $base_url = $this->makeQueryURL();
          $html_id = md5(time() . $uris[0].print_r($form_element_options,1));
          //
          $url = $path . '?' . $base_url . '&';  
          $jscall = " setUrlPrefix('" . $this->makefullJSRPCURL($search_name) . "');  semantic_searchSearchSubmitSelect( '" . $html_id . "','" . $prefix . "'); ";
          $form_element['#attributes'] = array (
            'onChange' => $jscall,
            'id' => $html_id
          );
        }
        $total_total = 0;
        $total_inseach_total = 0;
        $tarray[0] = '';


        $asc = (empty ($form_element_options['#sort_order']) || $form_element_options['#sort_order'] != 'desc');

        if ($alphasort == 'num') {
          $count_array[0] = 800000;
        }
        
        $top_option = null;
        if (!is_array($result_array))$result_array = array($result_array);
        foreach ($result_array as $uri => $data) {
        	if (empty($data['label']) || trim($data['label']) == '') continue;
          $encuri = SONIA_H :: encodeURI($uri);
          if (isset($data['count']) && is_int($data['count'])) {
          	 $total_total += $data['count'];
          }
          $predicate = ($prefix == 'p_value'  )?$uris[0]:null;
          $predicate = ($prefix == 'facet'  )?$uri:null;
           
          //NOTE: $multi_f will not show results in search since if you only can search on 
          // on field, then all the others will reflect 0 and not encourage userse
          // to search elsewhere
          
          //TODO if this wasn't selected
          $not_active = false;
          $num_in_search = ($this->doSearch() && !empty ($this->search_results) && ($multi_f || $not_active || (isset($form_element_options['#show_in_search_count'])) && ($form_element_options['#show_in_search_count']) ) ) ?  $data['count_in_search'] : $data['count'];
          
           if (isset($data['label']) && isset($form_element_options['#filter_on_option_labels']) && function_exists($form_element_options['#filter_on_option_labels'])) {
           	 $function_name = (string) $form_element_options['#filter_on_option_labels'];
             $data['label'] = $function_name($data['label']);
        		} else if (isset($form_element_options['#filter_on_option_labels']) && !function_exists($form_element_options['#filter_on_option_labels'])){
        		  watchdog('SEMANTIC_SEARCH', $form_element_options['#filter_on_option_labels'] . ' is not a function.');
        	}
         
          $tarray[$encuri] = "{$data['label']} " . ' ' . "($num_in_search)";
          if (isset($form_element_options['#top_option']) && $form_element_options['#top_option'] == $data['label']){
            $top_option = $encuri;
          }
          
          if ($alphasort == 'num') {
            $count_array[$encuri] = $num_in_search;
          }
          
          if ($this->searchedForURI($uri, $prefix) && $num_in_search>0) {
            $form_element['#default_value'] = $encuri;
            if ($multi_f)
              $tarray[$encuri] .= ' ** ';
          } else { 
          
          }
        } //end foreach

        //sort
        if ($alphasort == 'alpha') {
          if ($asc)
            asort($tarray);
          else
            arsort($tarray);
        } else if ($alphasort == 'num')  { //number sort
          if ($asc)
            asort($count_array);
          else
            arsort($count_array);

          $new_array = array ();
          foreach ($count_array as $encuri => $_count) {
            $new_array[$encuri] = $tarray[$encuri];
          }
          $tarray = $new_array;
        }

        //then add first option
        if ($form_element_options['#title_as_option'] == true) {
          $display_count = ($this->doSearch() && !empty ($this->search_results)) ? $total_inseach_total : $total_total;

          $display_count = ($form_element_options['#grand_total'] == true) ? "($display_count)" : '';

          $tarray[0] = "{$form_element_options['#title']}  $display_count";
        } else {
          $tarray[0] = ' please select';
        }
        
        //re-situate top option
        if ($top_option !== null) {
           $new_array = array ();
           $new_array[0] = $tarray[0];
           $new_array[$top_option] =  $tarray[$top_option];
           //do the rest
           foreach ($tarray as $encuri => $_count) {
             if (!empty($encuri) && $encuri != $top_option){
              $new_array[$encuri] = $tarray[$encuri];
             }
           }
           
           $tarray = $new_array;
        }
        
        if ( $form_element_options['#uris_only']) {
          return $tarray;
        }
        
        
        
        $form_element['#options'] = $tarray;
        break;

      case 'checkbox' : //assumes to be a group
        $tarray = array ();
        $count = 1;
        if ($form_element_options['#title_as_option']) {
          $form_element['#title'] = $form_element_options['#title'];
          $form_element['#description'] = $form_element_options['#description'];
        }
        $form_element['#type'] = 'fieldset';
        foreach ($result_array as $uri => $data) {
          // a little switch
          if ($prefix == 'p_value') {
            $rvalue = SONIA_H :: encodeURI($uris[0]);
          } else {
            $rvalue = 1;
          }

          $encuri = SONIA_H :: encodeURI($uri);
          $dv = $this->searchedForURI($uri, $prefix);

          $form_element[$prefix . '][' . $encuri]['#title'] = "{$data['label']} ";
          $num_in_search = ($this->doSearch() && !empty ($this->search_results)) ?  $data['count_in_search'] : $data['count'];
          $form_element[$prefix . '][' . $encuri]['#title'] .= '<span class="countArea" >&nbsp;' . "($num_in_search)</span>";

          $form_element[$prefix . '][' . $encuri]['#description'] = $form_element_options['#description'];
          $form_element[$prefix . '][' . $encuri]['#type'] = $form_element_options['#type'];
          //$form_element[$uri]['#value'] = $uri;
          $form_element[$prefix . '][' . $encuri]['#default_value'] = $dv;
          $form_element[$prefix . '][' . $encuri]['#return_value'] = $rvalue;
          $count++;
        }
        break;

      case 'links' :

        $items = array ();
        if ($alphasort == 'num') {
          $count_array[0] = 800000;
        }

        if ($prefix == 'p_value' || $prefix == 'cat' ) {
          $rvalue = SONIA_H :: encodeURI($uris[0]);
        } else {
          $rvalue = '';
        }
        
        //HACK for AHIRC, maybe there is a better solution
        if ($form_element_options['#return_type'] == 'telescope'){
          foreach($result_array as $result_arraytemp){
            if (empty($result_arraytemp['label']) && !empty($result_arraytemp['children'])){
                $result_array = $result_arraytemp['children'];
                break;
            } 
          }
        }
        $multi_search = (!$multi_f) ? '' : '[]';
        $predicate = ($prefix == 'p_value'|| $prefix == 'facet' )?$uris[0]:null;
        
        foreach ($result_array as $uri => $data) {
          $title_temp = $data['label'];
          $encuri = SONIA_H :: encodeURI($uri);
          $dv = $this->searchedForURI($uri, $prefix);
          
          //TODO fix this for ARC
          $num_in_search = ($this->doSearch() && !empty ($this->search_results)) ? $data['count_in_search']: $data['count']; 
         // dbg($predicate,'$predicate');//$this->countInSearch($uri, $prefix,$predicate) 
          
         if($num_in_search > 0 || $form_element_options['#display_if_count_zero'] ){
              
            $show_counts =    ($form_element_options['#return_type'] != 'telescope') 
            || ($form_element_options['#return_type'] == 'telescope' && (!isset ($data['children']) || !is_array($data['children'])));
            
            $telescope_level_2 = (!$show_counts );
            
            $counts = ($show_counts) ?'<span class="countArea" >&nbsp;' . "($num_in_search)</span>":'';
            if ($alphasort == 'num') {
              $count_array[$encuri] = $num_in_search;
            }
  
            $desc = "for " . htmlentities($title_temp). ' in ' .$num_in_search . ' results ';
            
            //if js enabled 
            //$show_counts && !isset ($data['children'] ) &&
            if (!empty($encuri) &&  !$dv) {
            	
              $rpcpath = url('semantic_search_ajax/search'.$search_name, null, null, true);
              $base_url = $this->makeQueryURL();
              $rpcurl = $rpcpath . '?' . $base_url . '&' . SONIA_FORM_REQUEST_PREFIX . '[' . $prefix . ']' . $multi_search . '[' . $rvalue . ']=' . $encuri;
              
              $url =  '?' . $base_url . '&' . SONIA_FORM_REQUEST_PREFIX . '[' . $prefix . ']' . $multi_search . '[' . $rvalue . ']=' . $encuri;
              
              $rpccall = ($form_element_options['#rpc']) ? ' onclick="semantic_searchSearchSubmitLink(\'' . $rpcurl . '\')" ' : '';
              
              
              if ($show_counts){
                 $items[$encuri] = '<span class="semantic_searchSearchList" '.$rpccall.'>'; 
                 $items[$encuri] .= '<a class="semantic-search-link" title="search: ' . $desc . '" href="'.$url.'" >' ;
                 $items[$encuri] .= $title_temp . ' ' . $counts.'</a></span>';
              }
              else {
                 $items[$encuri] = '<span class="semantic_searchSearchListNoSearch"    >' . $title_temp . ' ' . $counts . '</span>';
              }
            }
            
   
            if ($dv && !empty($encuri)) {
              $items[$encuri] = '<span class="semantic_searchListSearched" >' . $items[$encuri] . '</span>';
            }
            
            if (!empty($encuri) && $form_element_options['#return_type'] == 'telescope' && isset($data['description'])){
              $items[$encuri] .= '<div class="link-description">'.$data['description'].'</div>';
            }
             
            //if this is a telescoping navigation, get the next layer
            if ($form_element_options['#return_type'] == 'telescope') {
              if (isset ($data['children']) && is_array($data['children'])) {
                $rpcpath = url('semantic_search_ajax/search', null, null, true);
                $base_url = $this->makeQueryURL();
                $titems = array ();
                $count_array2 = array ();
  
                foreach ($data['children'] as $uri_2 => $tdata) {
                  $title_temp2 = $tdata['label'];
                  //if the title is empty ignore it.
                  if ($title_temp2 == '') continue;
                  $desc2 = "for " . $title_temp .' in ' .$title_temp2;
                  $encuri2 = SONIA_H :: encodeURI($uri_2);
                  $dv2 = $this->searchedForURI($uri_2, $prefix);
                  $multi_search = (!$multi_f) ? '' : '[]';
                 // dbg($tdata,'$tdata[\'count_in_search\']');
                  $num_in_search2 = ($this->doSearch() && !empty ($this->search_results)) ? $tdata['count_in_search']: $tdata['count']; //$this->countInSearch($uri_2, $prefix) 
                  $counts2 = '<span class="countArea" >&nbsp;' . "($num_in_search2)</span>";
                  if($num_in_search2 > 0 || $form_element_options['#display_if_count_zero'] ){
                    if ($alphasort == 'num') {
                      $count_array2[$encuri2] = $num_in_search2;
                    }
                    $url2 = $path . '?' . $base_url . '&' . SONIA_FORM_REQUEST_PREFIX . '[' . $prefix . ']' . $multi_search . '[' . $rvalue . ']=' . $encuri2;
                    
                    $rpccall = ($form_element_options['#rpc']) ? ' onclick="semantic_searchSearchSubmitLink(\'' . $url2 . '\')" ' : '';
                    $url3 = '?' . $base_url . '&' . SONIA_FORM_REQUEST_PREFIX . '[' . $prefix . ']' . $multi_search . '[' . $rvalue . ']=' . $encuri2;
                    
                    $titems[$encuri2] = '<span class="semantic_searchSearchList"><a class="semantic-search-link" href="'.$url3.'" title="search: ' . $desc2. '" '.$rpccall.' >' . $title_temp2 . ' ' . $counts2 . '</a></span>';
                    
                    if ($dv2) {
                      $titems[$encuri2] = '<span class="semantic_searchListSearched" >' . $titems[$encuri2] . '</span>';
                    }
                  }
                }
                
                
                $sub = (isset($form_element_options['#split_sub_category_lists']) 
                && ($form_element_options['#split_sub_category_lists'] == true)) ? 'sub' : false;
                
                if ($sub === 'sub' && $form_element_options['#sort_sub_by_weight'] &&  isset($_GET['edit']['cat'])){
                  self :: sortItemsByTaxonomyWeight($titems,$data['children']);
                } else {
                  $this->sortItems($titems, $count_array2, $alphasort, $asc);
                }
                
                $items[$encuri] .= theme('item_list',$titems ,null,$sub);
              }
            }
         }
       }

        $this->sortItems($items, $count_array, $alphasort, $asc);
        $title = $form_element_options['#title'];
        $description = $form_element_options['#description'];
        
        if (!empty($title)){
          $form_element['#type'] = 'fieldset';
          $form_element['#title'] = $title;
          $form_element['#description'] = $description;
        }
        $form_element['sub']['#value'] = theme('item_list',$items ,null,true);
        break;

      case '' :
        break;

      default :
        $message = $form_element_options['#type'] . ' is not a supported $form_type in getSearchFormElement()';
        $this->sonia->io->throwError($message);
      
    }

    if ($array_prefix !== null) { //set up for multiple
      return array (
        $array_prefix => $form_element
      );
    }
    return $form_element;
  }
  
  /**
   * use the weights of Drupal's taxonomy to sort the items
   * 
   * @todo move weights into the RDF and sort there // current approach is innefficient
   * 
   * */
  private static function sortItemsByTaxonomyWeight(&$titems,$data,$type='links'){
    $new_array = array();
    $weight_array = array();
    
    // get weight
    foreach($data as $uri=>$val){
    	$weight_array[$uri] = db_result(db_query("SELECT weight FROM {term_data} WHERE name = '%s'",trim($val['label'])));
      
    }
    
    // sort by weight, lighter at top
    asort($weight_array);
    
    foreach($weight_array as $uri=>$weight){
    	$uri = SONIA_H :: encodeURI($uri);
      $new_array[$uri] = $titems[$uri];
    }
    
    $titems = $new_array;
  }

  /**
   * helper function to generate layers of links
   * */
  private function createHieararchicalLinks(& $items, & $count_array, $title, $description, $alphasort, $asc) {

    $form_element['#type'] = 'fieldset';
    $form_element['#title'] = $title;
    $form_element['#description'] = $description;
    $form_element['sub']['#value'] = theme_item_list($items);
    return $form_element;
  }

  /**
   * helper function to sort item list
   * */
  private function sortItems(& $items, & $count_array, $alphasort, $asc) {
    if ($alphasort == 'alpha') {
      if ($asc)
        asort($items);
      else
        arsort($items);
    } else if ($alphasort == 'num')  { //number sort
      if ($asc)
        asort($count_array);
      else
        arsort($count_array);

      $new_array = array ();
      foreach ($count_array as $encuri => $_count) {
        $new_array[$encuri] = $items[$encuri];
      }
      $items = $new_array;
    }
  }

  /**
   * returns absolute path to the node in drupal
   * */
  public static function makeNodeURL($id, $label = 'node', $query = null) {
    return url($label . '/' . $id, $query, null, true);
  }

  public function getClassProperties(& $count, $class) {
    return $this->sonia->getAllClassProperties($count, $class);
  }

  //////////////////////////////////////////////
  //UNFILTERED

  public function getLabel($object) {
    return $this->sonia->getLabel($object);
  }

  public function getClasses($object) {
    return $this->sonia->getClasses($object);
  }

  /////////////////////////////////////////////////

  public function passFormValues($form_values) {
    $this->sonia->io->setRequest($form_values);
  }

  public function dataSubmitted() {
    $this->sonia->_dataSubmitted();
  }

  /**
   * wrapper to expose private function signalling that a search has commenced
   * */
  public function doSearch() {
    return $this->sonia->io->doSearch();
  }

  /**
   * needed because drupals forms perform a redirect from POST to GET.
   * */
  public function makefullURLforRedirect() {
    $qstring = SONIA_URL_H :: makeFullURLFromSearch($this->search_params);
    return url(SEMANTIC_SEARCH_BASE_URL, $qstring, null, true);
  }
  
 
 /**
  * @param array data is the array from theme_semantic_search_navigation()
  * 
  * @see theme_semantic_search_navigation()
  * */
 public function getBreadCrumbURLs(&$data ){
  $jbcount = count($data['jellyBeans']);
  $return_array = array();
  
  foreach ($data['jellyBeans'] as $num => $jb) {
  	$allowed_urls = $jbcount +1;
    $exclude_list = array();
    $is_last = false;
    $aurl = '';
    
    if ($jbcount == ($num+1)){
    	 $is_last = true;
    	 $allowed_urls = $allowed_urls -1;
    } else {
      foreach($data['jellyBeans'] as $j2 => $jb2){
      	if ($j2 > $num){
      	   $exclude_list[$jb2[5]] = $jb2[4];
      	   $allowed_urls = $allowed_urls -1;
      	}
      }
      $return_array[] = $this->makeBreadCrumbURL($allowed_urls);
    }
  }
   return $return_array;
 }
  
  public function makeBreadCrumbURL($count){
    return SONIA_URL_H :: makeFullURLFromSearchWithExclude($this->search_params,$count);
  }
  
  public function makefullJSRPCURL($search_name = '') {
    $qstring = SONIA_URL_H :: makeFullURLFromSearch($this->search_params);
    return SEMANTIC_SEARCH_JSRPC_PATH.$search_name.'?'. $qstring;
  }

  /**
   * removes pagination, since it's assumed you need to start the pagination 
   * from the zero position
   * */
  public function makeQueryURL() {
    $qstring = SONIA_URL_H :: makeFullURLFromSearch($this->search_params);
    //remove the pagination
    $qstring = str_replace( '&' .  SEMANTIC_SEARCH_PAGINATION_SIGNAL . '='. SONIA_IO :: getPaginationBegin(),'',$qstring);
    return $qstring;
  }

  public function getSessionHash() {
    return $this->sonia->getSessionHash();
  }

  /**
   * 
   * 
   * @return boolean 
   */
  public function searchResultsFound() {
    return $this->doSearch() && is_array($this->search_results) && !empty ($this->search_results);
    
  }

  public function searchParamsActive() {
    return $this->doSearch() && is_array($this->search_params) && !empty ($this->search_params);
  }

  /**
   * get the data needed to create search result pagination elements in a theme
   * */
  public function searchPaginaton() {
    $data = array ();
    $data['base_url'] = $this->makefullURLforRedirect();
    $data['restart_url'] = $this->getRestartSearchLink();
    $data['do_search'] = $this->doSearch();
    $data['search_results_success'] = $this->searchResultsFound();
    $data['total'] = $this->sonia->store->object_search_result_count;
    //needed?
    $this->number_of_search_results = $data['total'];
    
 
    
    
    $data['begin'] = $this->sonia->io->getPaginationBegin();
    $data['end'] = ($data['total'] < (SEMANTIC_SEARCH_DISPLAY_PAGINATION + $data['begin'])) ? $data['total'] : (SEMANTIC_SEARCH_DISPLAY_PAGINATION + $data['begin']);

    $data['total_pages'] = (SEMANTIC_SEARCH_DISPLAY_PAGINATION > 0) ? ceil($data['total']/SEMANTIC_SEARCH_DISPLAY_PAGINATION ): 0;
    $data['this_page']   = ($data['begin']/SEMANTIC_SEARCH_DISPLAY_PAGINATION )+1;
    
    $data['page_links'] = array();
    
    for($i=1;$i<=$data['total_pages'];$i++){
      $data['page_links'][$i] = $data['base_url'] . '&' . SEMANTIC_SEARCH_PAGINATION_SIGNAL . '=' . (($i-1)*SEMANTIC_SEARCH_DISPLAY_PAGINATION);
    }

    if ($data['begin'] >= SEMANTIC_SEARCH_DISPLAY_PAGINATION || (($data['begin'] + 1) > $data['total'])) {
      $previous = (0 > ($data['begin'] - SEMANTIC_SEARCH_DISPLAY_PAGINATION)) ? 0 : $data['begin'] - SEMANTIC_SEARCH_DISPLAY_PAGINATION;
      $data['previous_link'] = $data['base_url'] . '&' . SEMANTIC_SEARCH_PAGINATION_SIGNAL . '=' . $previous;
    }

    if ($data['end'] < $data['total']) {
      $next = ($data['end'] < ($data['begin'] + SEMANTIC_SEARCH_DISPLAY_PAGINATION)) ? $data['end'] : $data['begin'] + SEMANTIC_SEARCH_DISPLAY_PAGINATION;
      $data['next_link'] = $data['base_url'] . '&' . SEMANTIC_SEARCH_PAGINATION_SIGNAL . '=' . $next;
    }

    return $data;
  }

  /**
   * JellyBeans are like breadcrumbs, but better.
   * 
   * The function provides text based feedback on what the user is searching for
   * The feedback is alos interactive - allowing them to add and remove sub-elements
   * of the  query.
   * 
   * 
   * */
  function jellyBean($not_rpc = true, $stand_alone_block = true, $extra = '') {
    //data to return
    $data = array ();
    $data['results_found'] = $this->searchResultsFound();
    $data['search_active'] = $this->searchParamsActive();
    if (!$this->searchParamsActive())
      return array ();

    $data['is_rpc'] = !$not_rpc;
    $data['url'] = NinaNav :: search_link($this->search_params);
    $data['jellyBeanMenu'] = theme("semantic_search_navigation_menu", $this->jellyBeanMenu($data['url']));
    $data['extra'] = $extra;
    $data['jellyBeans'] = array (); //o key, 1 value, 3 remove link

    $looking = false;
    $url = NinaNav :: search_link($this->search_params);
    $count = 0;

    if ($this->searchParamsActive()) {

      foreach ($this->search_params as $key => $value) {
        //ignore placholder
        if ($key === 0) {
          continue;
        }

        if (!is_array($value)) {
          //$html .= $value . ' $value ';
          if ($value !== '') {
            
            $data['jellyBeans'][] = $this->makeJellyBean($url, $key, $key, $value);
          } //end if
        } //end if
        else
          if (!empty ($value)) {
            foreach ($value as $k => $v) {
              if ($v != '') {
                if (!is_array($v)) {
                  $data['jellyBeans'][] = $this->makeJellyBean($url, $key, $k, $v);
                } else {
                  foreach ($v as $kk => $vv) {
                    $data['jellyBeans'][] = $this->makeJellyBean($url, $key, $kk, $vv);
                  } // end inner inner foreach
                }
              }
            } // end inner foreach
          }
      } //end foreach
    } //end if

    return $data;
  }

  /**
   * preps an array to be a jellybean
   * */
  private function makeJellyBean($url, $key, $k, $v) {
    $dkey = null;
    if ($key == 'p_value') {
      //HACK
      
      $nkey = $text = (stristr($k, 'http://')) ? $v : $k;
      $nvalue = ($nkey == $k) ? $v : $k;
      //(stristr($k,'http://')) ?$v:$k;

      if (!empty ($text)) {
        $dkey = SONIA_H :: encodeURI($nkey);
        $dvaluex = SONIA_H :: encodeURI($nvalue);
        $_a = $dkey;
        $_b = $dvaluex;
        $dremove = $url . '&remove=' . $dkey . '&vv=' . $dvaluex;
        $dvalue = rawurldecode($text);
        $dfacet = true;
        
      }
    } else
      if ($key == 'facet') {
        if ($v != '' && !is_array($v)) {
          $dkey = SONIA_H :: encodeURI($k);
          $vval = SONIA_H :: encodeURI($v);
          $dremove = $url . '&remove=' . $key . '&vv=' . $vval;
          $_a = $key;
          $_b = $vval;
          $dvalue = $this->sonia->getLabel($v);
          $dfacet = true;
        }
    } else
      if ($key == 'cat') {
        if ($v != '' && !is_array($v)) {
          $dkey = SONIA_H :: encodeURI($k);
          $vval = SONIA_H :: encodeURI($v);
          $dremove = $url . '&remove=' . $dkey . '&vv=' . $vval;
          $_a = $dkey;
          $_b = $vval;
          $dvalue = $this->sonia->getLabel($v);
          $dfacet = true;
        }
    } else
      if ($key == 'qstring') {
        if ($v != '' && !is_array($v)) {
          $dkey = SONIA_H :: encodeURI($k);
          $vval = SONIA_H :: encodeURI($v);
          $dremove = $url . '&remove=qstring&vn=' . $dkey . '&vv=' . $vval;
          $_a = $dkey;
          $_b = $vval;
          $dvalue = $v;
          $dfacet = false;
        }
      } else {
        $dkey = SONIA_H :: encodeURI($k);
        $dremove = $url . '&remove=' . $dkey;
        if ($this->sonia->classExists($k))
          $dvalue = rawurldecode($this->sonia->getLabel($k));
        if ($this->sonia->propertyExists($k))
          $dvalue = rawurldecode($this->sonia->getPropertyName($k));
        $dfacet = true;
        $_a = $dkey;
        $_b = '';
      }
    
    //if there is only one term left, removing it should restart the search
    if ($this->getSearchTermCount() == 1) $dremove = $this->getRestartSearchLink();
       
    if ($dkey != null) {
      _semantic_search_str_clean($dvalue);
      return array (
        $dkey,
        $dvalue,
        $dremove,
        $dfacet,
        $_a,
        $_b
      );
    }
  }

  /**
   * provide the number of search terms
   * */
  private function getSearchTermCount(){  
    return count($this->sonia->sonia_search_array);
  }


  public function getRestartSearchLink() {
    return NinaNav :: search_reset_link($this->sonia);
  }

  /**
   * */
  function jellyBeanMenu($url) {
    $data = array ();
    $data['url'] = $url;
    $data['restart'] = $this->getRestartSearchLink();

    $the_type = 'semantic_search_help_text';
    $data['help_text'] = variable_get($the_type, t('set in admin/settings/semantic_search'));

    $the_type = 'semantic_search_help_title';
    $data['help_title'] = variable_get($the_type, t('set in admin/settings/semantic_search'));

    return $data;
  }

  /**
   * parse search results into data for display
   * 
   * */
  function searchResults($not_rpc = true) {
    //__nsp(' semantic_search.search.inc searchResults() start');
    $rdata = array ();
    if (empty ($this->search_results))
      $this->search_results = array ();
     
    $count = 0;
    $total = 0;
    $invisible_key = 5;
    $no_count = 0;
    $yes_count = 0;

    //how many to display per page
    $rdata['displayed'] = $displayed = SEMANTIC_SEARCH_DISPLAY_PAGINATION;
    $rdata['base_url'] = $base_url = $this->makefullURLforRedirect();
    $rdata['begin'] = $begin = $this->sonia->io->getPaginationBegin();

    $rdata['not_rpc'] = $not_rpc;
    $rdata['total'] = count($this->search_results);
    $rdata['do_search'] = $this->doSearch();
    if (!$rdata['do_search'] && empty ($this->search_results)) {
      //__nsp(' semantic_search.search.inc searchResults() empty end');
      //print_r($this->search_results,'$this->search_results ev');
      return array ();
    }
    //search_results DATA STRUCTURE
    //TODO move to constants
    //search_results[0] ['#node_url'],['#result_number'],['#title'],
    //..[rows]// 0 name 1 value 2 searched 3 uri/key 4  hidden 
    //.. 5 type (document, image, url, email) 6 count(order number)

    $rdata['search_results'] = array ();

    $_SESSION['sonia_latest_search'] = $base_url . '&' . SEMANTIC_SEARCH_PAGINATION_SIGNAL . '=' . $begin;

    $the_type = 'semantic_search_display_preview_data';
    $display_all_paramter_counts = variable_get($the_type, '0');
    $cached_prefs = array ();
    
    foreach ($this->search_results as $key => $value) {
      $search_result = array ();

        $hidden_search_results = $main_search_results = array ();
        $inner_count = 0;

        if (true) {
          $nid = SONIA_H :: getIDFromURI($key);
          $url = url('node/' . $nid, null, null, true);

          $search_result['#node_url'] = $url;
          $search_result['#result_number'] = $count +1;

          $thtml = '';
          $hhtml = '';

          $param_count = 1;
          $param_display_data = array ();

          foreach ($value as $data) {
            $property = $data['p'];
            $object = $data['o'];
            $property_label = (isset ($data['p_l'])) ? $data['p_l'] : '';

            //the array of params for this result row
            $display = array ();

            //set up display order
            $fid = SONIA_H :: getIDFromURI($property);
            $pref_name2 = 'display_result_' . $fid;
            if (isset ($cached_prefs[$pref_name2])) {
              $display_visibly = $cached_prefs[$pref_name2];
            } else {
              $display_visibly = variable_get($pref_name2, '4');
              $cached_prefs[$pref_name2] = $display_visibly;
            }
            
            $is_file_type = !empty ($data['fname']) || strstr($object,SONIA_FILE_CLASS_URI);
            $display_order = 10 + $display_visibly;

            //create $display
            //place $display in sorting array 
            $display['name'] = _semantic_search_str_clean($property_label);
            $display['uri'] = SONIA_H :: encodeURI($property);
            if ($property_label != null) {
             
              $param_in_search = $display['searched'] = $this->sonia->checkPropertyInSearch($property, $object);
            } else
              $display['searched'] = false;

            $display['hidden'] = ($display_visibly == $invisible_key && !$display['searched']);
            $display['title'] = false;
            $display['image'] = false;

            if ($property == SONIA_RDFS_CREATE_LABEL) {
              //always first
              $search_result['#title'] = _semantic_search_str_clean($object);
              $display['title'] = true;

            } else
              if ($is_file_type) {
                $file_pref_name = 'display_image_' . $fid;
                $thidden = !variable_get($file_pref_name, '0');
                if (!$thidden) {
                  $display['hidden'] = $thidden;
                  $path = $data['fpath'];
                  $path = str_replace($data['fname'], rawurlencode($data['fname']), $path);
                   
                  if ($path != '') {
                    $image_src_url = '/' . $path;
                    //add description/title ?
                    $display['image'] = true;
                    $display['value'] = $image_src_url;
                    $param_display_data[$display_order][$param_count] = $display;

                  } else {
                    //this image is a phony should be deleted from the db
                    $display = array();
                  }
                } else {
                  $display = array();
                }
              } else
                if ($property == SONIA_RDF_CREATE_TYPE) {
                  //check to see if this class type is permitted to be displayed in search results
                  $content_type = SONIA_H :: getIDFromURI($object);
                  $pref_name = 'display_' . $content_type;
                  //$allowed = variable_get($pref_name, '1');
                  //debug
                  //if (!$allowed) dbg($pref_name,'$pref_name not allowed');

                } else
                  if ($property_label != null) {
                    $display['value'] = _semantic_search_str_clean($object);
                    if ($display['title']) {
                      //makes the search result show up first
                      $param_display_data[0][0] = $display;
                    } else
                      if ($param_in_search) {

                        // used by themers to highlight text optionally
                        if (isset ($this->search_params['qstring'])) {
                          foreach ($this->search_params['qstring'] as $term) {
                            if (!is_array($term)) {
                              $display['qstring'][] = $term;
                            } else {
                              foreach ($term as $tterm) {
                                $display['qstring'][] = $tterm;
                              }
                            }
                          }
                        }

                        $param_display_data[0][$param_count] = $display;
                      } else {
                        $param_display_data[$display_order][$param_count] = $display;
                      }
                  } //end property label has value
            $param_count++;
          } // end param foreach

          //complete and sort
          if (true) {
            // add to data array
            $search_result['rows'] = array ();

            ksort($param_display_data);
            foreach ($param_display_data as $pdata) {
              ksort($pdata);
              foreach ($pdata as $display) {
                $search_result['rows'][] = $display;
              }
            }

            $rdata['search_results'][] = $search_result;

          } else {
            $this->notAllowedResultMessage($count);
          }

        } else {
          //later this can be to retrieve items other than objects
          $this->badResultMessage();
        }
      $count++;
    } //end search results foreach
    //__nsp(' semantic_search.search.inc searchResults() end');
     
    return $rdata;
  }

  /***/
  private function notAllowedResultMessage($count) {
    $turl = url('admin/settings/semantic_search');
    if (user_access("administer semantic_search")) {
      drupal_set_message('Result ' . ($count +1) . ' has been <a href="' . $turl . '">shut off</a> either becaue it is not enabled or' .
      ' because it is not an allowed search result.');
    }

  }

  /***/
  private function badResultMessage() {
    if (user_access("administer semantic_search")) {
      if (!empty ($this->search_results))
        drupal_set_message('SEMANTIC_SEARCH ADMIN MESSAGE: bad result (non-object)');
    }
  }

  //HACK
  public function __destruct() {
    if (SEMANTIC_SEARCH_DEBUG && isset ($_SESSION['messages']['error'])) {
      //echo '<hr>'. $_SESSION['messages']['error'];
    }
  }

} //end class
