<?php

/*
* Copyright (C) 2007 Jonathan Hendler
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
* 
* The author can be contacted at: 
*  - [hendlerman (at) yahoo DOT com]
* 
* @license GPLv2
* @link    http://drupal.org/project/semantic_search
*/

/**
 * used for creating searches
 * */
function semantic_search_search_admin_form($search_name = null){
 _semantic_search_load_js_and_css();
 $form = array();
 
 
 $default_values = array();
 
 global $_semantic_search_search;
  if (empty ($_semantic_search_search))
    _SEMANTIC_SEARCH_INIT();
 //use cache?
 
 //get default values
 if (!empty($search_name)){
    $default_values = _semantic_search_get_search_form_vars($search_name);
   
 }//end empty search name
 
// //cache control for this search 
    $form['cache_settings'] = array (
      '#type' => 'fieldset',
      '#title' => 'Cache Settings',
      '#description' => t('Settings for indexing and caching.'),
      '#collapsible' => 1,
      '#collapsed' => 1
    );
//   
 
    
 $form['cache_settings']['sf_use_cache'.$search_name] = array (
    '#type' => 'checkbox',
    '#title' => t('Use Cache'), 
    '#description' => t('Cache can build organically with use, or you can build here.'), 
    '#return_value' => 1, 
   // '#default_value' => variable_get('sf_use_cache'.$search_name,0),
  );
  
 $form['main_search_settings'] = array (
    '#type' => 'fieldset',
    '#title' => 'Main Settings',
    '#description' => t('Main Settings for Creating or Modifying a Search'),
    '#collapsible' => 1,
    '#collapsed' => 1
  );
  
  $attributes = array();
  if (!empty($search_name) && $search_name != '') {
    $attributes = array('disabled'=>1);
    
    $form['main_search_settings']['search_id'] = array (
      '#type' => 'hidden',
      '#value' =>  $default_values['id'],
     );
    
  }
  
  $form['main_search_settings']['search_name'] = array (
    '#type' => 'textfield',
    '#title' => t('Search Name'), 
    '#description' => t('<strong>VERY IMPORTANT</strong> readable name like "animals", "birds", or "fish" .<br/>' .
      'Is used for the cache name, index name, rdf tables name etc.'),
    '#default_value' => $search_name,
    '#attributes' => $attributes,
     );
    
  $form['main_search_settings']['search_path'] = array (
    '#type' => 'textfield',
    '#title' => t('Search Path'), 
    '#description' => t('The url for users to access the search. Eg <code>search/myanimals</code>'),
    '#default_value' => $default_values['path'] );
  
  $options = array('default'=>'default','custom api'=>'custom api','syndicated'=>'syndicated',);
  
  $form['main_search_settings']['search_type'] = array (
    '#type' => 'select',
    '#title' => t('Search Type'), 
    '#options' => $options, 
    '#description' => t('<ul>' .
        '<li><strong>default</strong>: a default interface. Can be themed.</li>' .
        '<li><strong>custom api</strong>: creates an API available for RPC calls. </li>' .
        '<li><strong>syndicated</strong>: custom module for aggregating content dynamically.</li>' .
        '</ul>'),
    '#default_value' => $default_values['type']  );


   //TODO store type  
//    $options = array('arc_local'=>'default ARC local','arc_remote'=>'ARC remote','sesame_remote'=>'Sesame Remote',);
//    $form['main_search_settings']['search_store_type'] = array (
//    '#type' => 'select',
//    '#title' => t('Store Type'), 
//    '#options' => $options, 
//    '#description' => t('<ul>' .
//        '<li><strong>default ARC local</strong>: this is the only store available. Uses ARC locally</li>' .
//        '<li><strong>ARC remote</strong>: TODO: ARC on another server. </li>' .
//        '<li><strong>Sesame Remote</strong>: TODO Sesame on another server</li>' .
//        '</ul>'),
//    '#default_value' => $default_values['store_type']  );
    
   
    $form['main_search_settings']['search_title'] = array (
    '#type' => 'textfield',
    '#title' => t('Search Title'), 
    '#description' => t('The human-friendly, display name for the search. Like "My Search", or "Animals"'),
    '#default_value' => $default_values['title'] );
    
    $form['main_search_settings']['search_description'] = array (
      '#type' => 'textarea',
      '#rows' => 5,
      '#title' => t('Search Description'), 
      '#description' => t('Description of the search.'),
      '#default_value' => $default_values['description'] 
    );
  
  //node types/ cck types // users / relationships // node relation
  
  $form['default_search_settings'] = array (
    '#type' => 'fieldset',
    '#title' => 'Default Interface Settings',
    '#description' => t('Settings for "default" and "module" interface searches. '),
    '#collapsible' => 1,
    '#collapsed' => 1
  );
  
  // number of search results to display on a page
  $form['default_search_settings']['SF_DISPLAY_PAGINATION'.$search_name] = _semantic_search_form_element(
    'SF_DISPLAY_PAGINATION'.$search_name, 
    'Number of Results',
    'Number of search results to show per page.',
    10);
  
  $description_a =  (empty($search_name)) ?  t('Create the search form here, after the search has been created.'):  t('Create the search form here.');
  
  //get list of fields with search
  $form['default_search_settings']['search_form'] = array(  
    '#type' => 'fieldset',
    '#title' => 'The Search Form',
    '#description' => $description_a,
    '#collapsible' => 1,
    '#collapsed' => 1
  );
  
  if(!empty($search_name)){
   
    //list search fields
    $search_fields = _semantic_search_get_form_search_fields( $default_values['id']);
    //drupal_set_message( print_r($search_fields,1)); 
    
    $header = array( t('Facet Name'),t('Description'),'','');
    $data = array();
    
    $attr = array('title'=> t('Edit Search Facet of '). $search_name);
    $list = array();
    
    foreach($search_fields as $key=> $search_field){
      $data[$key][] = $title = t($search_field['title']);
      $data[$key][] =  t($search_field['description']);
      
      $fid = $search_field['id'];
      
      $link = url(SEMANTIC_SEARCH_ADMIN_HOME.'/searches/'.$search_name.'/edit/'.$fid,null,null,true);
      $data[$key][] = l(t('Edit'),$link,$attr) ;
      
      $link = url(SEMANTIC_SEARCH_ADMIN_HOME.'/searches/'.$search_name.'/delete',null,null,true);
      $data[$key][] = l(t('Delete'),$link,$attr);
      
    }
    
    $form['default_search_settings']['search_form']['edit_fields'] =  array(  
      '#value' => theme('table',$header,$data)
      );
    
    //link
    $attr = array('title'=> t('Add Search Facet to '). $search_name);
    $link = url(SEMANTIC_SEARCH_ADMIN_HOME.'/searches/'.$search_name.'/add',null,null,true);
    $form['default_search_settings']['search_form']['add_field'] = array(  
      '#value' => l(t('Add Facet'),$link,$attr) ,
    );
    
  }
  
 $form['import'] = array (
      '#type' => 'fieldset',
      '#title' => 'Import',
     // '#description' => t('Settings for indexing and caching.'),
      '#collapsible' => 1,
      '#collapsed' => 1
    ); 
      
 $form['export'] = array (
      '#type' => 'fieldset',
      '#title' => 'Export',
     // '#description' => t('Settings for indexing and caching.'),
      '#collapsible' => 1,
      '#collapsed' => 1
    );    
   
   
  //////////////////////////////////////////////////
  $word = ($search_name !== null) ? t('Update') : t('Create');
   
   $form['end']['submit'] = array (
    '#type' => 'submit',
    '#value'=>$word); 
    
   if($search_name !== null){
     $form['end']['delete'] = array (
      '#type' => 'submit',
      '#value'=> t('Delete')); 
   } 
  
  return $form;
}

/**
 * 
 * */
 function semantic_search_search_field_form($search_name, $field_id = null, $edit = false){
   global $_semantic_search_search;
   if (empty ($_semantic_search_search))
    _SEMANTIC_SEARCH_INIT();
    
    //select cck types for this search
   // get allowed fields
   //describe what is needed
   $description = t('These choices are based on what is selectedat the <a href="@a">search admin page</a>',array('@a'=>''));
   $warning = '';
   $warning = t('You must first select fields at the <a href="@a">search admin page</a>',array('@a'=>''));
   $default_values = $form = array();
   $search_data = _semantic_search_get_search_form_vars($search_name);
   
   $search_id = $search_data['id'];
      $form['field_settings'] = array (
      '#type' => 'fieldset',
      '#title' => 'Field Configuration',
      '#description' => t('Settings for an individual Facet to search on'),
    );
   
   if ($edit){
     //get default values
     $default_values = _semantic_search_get_form_search_field($field_id);
     
     $form['field_settings']['id'] = array (
      '#type' => 'hidden',
      '#value' => $field_id, 
     );
   }  
   
   
   $form['field_settings']['search_id'] = array (
    '#type' => 'hidden',
    '#value' => $search_id, 
   );
   
   
   //**posible URIs (based on CCK fields?)
   $form['field_settings']['title'] = array (
    '#type' => 'textfield',
    '#title' => t('Title'), 
    '#description' => t('In a select this can show as the first option, or a label. In other form elements, it is a label.'), 
     '#default_value' => (isset($default_values['title'])) ? $default_values['title']: '',
   );
   
   $form['field_settings']['description'] = array (
    '#type' => 'textarea',
    '#rows' => 3,
    '#title' => t('Description'), 
    '#description' => t('Same as the forms_api #description'), 
     '#default_value' => (isset($default_values['description'])) ? $default_values['description']: '',
   );
   
   $uri_options = $_semantic_search_search->getAllBaseOptions();  
   
   $form['field_settings']['uris'] = array (
    '#type' => 'select',
    '#options' => $uri_options,
    '#title' => t('URI'), 
    '#description' => t('In this list is the Node type, Taxonomy, or Node Field. For Semantic Search these are translated to RDF as Property, or RDF classes, or literals.'), 
     '#default_value' => (isset($default_values['uris'])) ? $default_values['uris']: '',
   );
   
   $options = array('select'=>'Select','links'=>'Links','checkbox'=>'Checkboxes',);
   
   $form['field_settings']['form_type'] = array (
    '#type' => 'select',
    '#options' => $options,
    '#title' => t('Form Field Type'), 
    '#description' => t('Standard HTML elements'), 
     '#default_value' => (isset($default_values['form_type'])) ? $default_values['form_type']: 0,
   );
   
  $options = array('p_value'=>'Property Value','class'=>'Class','property'=>'Property','fulltext'=>'Full Text',);
  $form['field_settings']['search_field_type'] = array (
    '#type' => 'select',
    '#options' => $options,
    '#title' => t('Search Field Type'), 
    '#description' => t('  '), 
     '#default_value' => (isset($default_values['search_field_type'])) ? $default_values['search_field_type']: 0,
   );
   
  $form_element_options_state['#type'] = 'select';
  $form_element_options_state['#query_type'] = 'p_value';
  $form_element_options_state['#multiple_facet'] = false;
  $form_element_options_state['#show_in_search_count'] = true;
   
   
//  uris  text  utf8_unicode_ci     No      Browse distinct values  Change  Drop  Primary   Unique  Index   Fulltext
//  form_type   varchar(32)   utf8_unicode_ci     No      Browse distinct values  Change  Drop  Primary   Unique  Index   Fulltext
//  search_field_type   varchar(32)   utf8_unicode_ci     No      Browse distinct values  Change  Drop  Primary   Unique  Index   Fulltext


//  top_option  int(1)    UNSIGNED  No      Browse distinct values  Change  Drop  Primary   Unique  Index   Fulltext
//   
  $form['field_settings']['title_as_option'] = array (
    '#type' => 'checkbox',
    '#title' => t('Title as option'), 
    '#description' => t(''), 
    '#return_value' => 1, 
    '#default_value' => (isset($default_values['title_as_option'])) ? $default_values['title_as_option']: 0,
  );
  
  $form['field_settings']['grand_total'] = array (
    '#type' => 'checkbox',
    '#title' => t('Display Grand Total with top Option'), 
    '#description' => t(''), 
    '#return_value' => 1, 
    '#default_value' => (isset($default_values['grand_total'])) ? $default_values['grand_total']: 0,
  ); 
  
  $form['field_settings']['multiple'] = array (
    '#type' => 'checkbox',
    '#title' => t('Multiple Facet'), 
    '#description' => t('Allow users to select multiple fields from this facet.'), 
    '#return_value' => 1, 
    '#default_value' => (isset($default_values['multiple'])) ? $default_values['multiple']: 1,
  ); 
  
  $word = ($edit) ? t('Update') : t('Create');
   
   $form['end']['submit'] = array (
    '#type' => 'submit',
    '#value'=>$word); 
    
   if($search_name !== null){
     $form['end']['delete'] = array (
      '#type' => 'submit',
      '#value'=> t('Delete')); 
   } 
   return $form;
 }
 
 function  _semantic_search_get_search_name($search_id){
   return db_result(db_query("SELECT name  FROM {semantic_search_searches} WHERE id=%d LIMIT 1",$search_id));
 }
 
 /**
 * 
 * */
 function semantic_search_search_field_form_submit($form_id, $form_values){
    $fid = _semantic_search_set_search_field_vars($form_values);
    $search_name  = _semantic_search_get_search_name($form_values['search_id']);
    return SEMANTIC_SEARCH_ADMIN_HOME.'/searches/'.$search_name.'/edit/'.$fid;
 }
 
 function semantic_search_search_field_form_validate($form_id, $form_values){
   $title = $form_values['title'];
   if (empty($title)){
     form_set_error('title', t('The field needs a name.'));
   }  
 }

/**
 * implementation of hook_submit
 * */
function semantic_search_search_admin_form_submit($form_id,$form_values){
  $op = $form_values['op'];
  $search_name = _semantic_search_set_search_form_vars($form_values, $op);
  
  
  if ($op == t('Create')){
    $number_of_results_var =   'SF_DISPLAY_PAGINATION';
  } else {
    $number_of_results_var =   'SF_DISPLAY_PAGINATION'.$search_name;
  }
  $number_of_results =  $form_values[$number_of_results_var];
 
  
  return SEMANTIC_SEARCH_ADMIN_HOME.'/searches/'.$search_name;
}


/**
 * implementation of hook_submit
 * */
function semantic_search_search_admin_form_validate($form_id,$form_values){
  $op = $form_values['op'];
  $search_name = $form_values['search_name'];
  $search_path = $form_values['search_path'];
  $search_title = $form_values['search_title'];
  
  if (empty($search_name)){
    form_set_error('search_name', t('The search needs a name.'));
  } else if ( $op == 'Create' && _semantic_search_get_search_form_vars($search_name) ){
    //check if name is taken
    form_set_error('search_name', t('The search name was taken.'));
  }
  
  if (empty($search_title)){
    form_set_error('search_name', t('The search needs a title.'));
  }
  
  if ($op == t('Create')){
    $number_of_results_var =   'SF_DISPLAY_PAGINATION';
  } else {
    $number_of_results_var =   'SF_DISPLAY_PAGINATION'.$search_name;
  }
  
  $number_of_results =  $form_values[$number_of_results_var];
  
  if (empty($number_of_results) || !is_numeric($number_of_results)){
     form_set_error($number_of_results_var, t('Search must return at least 1 result.'));
  }
  
  if (empty($search_path)){
    form_set_error('search_path', t('The search needs a path.'));
  } 
}


/**
 * 
 * */
 function _semantic_search_form_element($the_type,$title,$description = '',$default=''){
   $element = array();
   //$the_type = 'semantic_search_search_form_title';
   $element[$the_type] = array (
    '#type' => 'textfield',
    '#title' => t($title),
    '#description' => t($description),
    //'#return_value' => 1, 
  '#default_value' => variable_get($the_type,$default),);
  
   return $element;
 }

//TODO move to per config


/**
 * base URL should match waht is in settings.php
 * */
_semantic_search_def('_BASE_URL', 'http://ahirc');

/**
 * base URI for data specific to the site
 * */
_semantic_search_def('_BASE_NAMESPACE_URL', 'http://www.ahirc.org');

/***
 * prints debugging stuff to screen
 * (currently disabled)
 */
_semantic_search_def('SEMANTIC_SEARCH_DEBUG', true);

/**
 * Width of text search forms
 * */
_semantic_search_def('SEMANTIC_SEARCH_TEXT_INPUT_FORM_WIDTH', 25);

/**
 * number of search results to display on a page
 * */
_semantic_search_def('SEMANTIC_SEARCH_DISPLAY_PAGINATION', 10);

/**
 * display search
 * */
_semantic_search_def('SEMANTIC_SEARCH_DISPLAY_SEARCH_COUNTS', true);

/**
 * display totals for item 
 * */
_semantic_search_def('SEMANTIC_SEARCH_DISPLAY_TOTAL_COUNTS', true);

/**
 * display only one count and not both
 * */
_semantic_search_def('SEMANTIC_SEARCH_DISPLAY_SINGLE_COUNT', false);

define('SONIA_DEBUG', true);
/* location of an error log */
//define('SONIA_ERROR_LOG','/Temp/SONIAERROR.txt');
//define('SONIA_WRITE_ARC_LOG', true);
//define('SONIA_ARC_LOG_PATH', SONIA_LOCAL_ARC_TOOLS.'/logs/');

/**
 * datatabase that ARC uses
 * */
_semantic_search_def('ARC_DB_NAME', '');

/**
 * ARC DB username
 * */
_semantic_search_def('ARC_DB_USER', '');

/**
 * ARC DB password
 * */
_semantic_search_def('ARC_DB_PW', '');

/**
 * ARC DB host
 * */
_semantic_search_def('ARC_DB_HOST', 'localhost');

/**
 * ARC DB port... should be 3306 in most cases
 * */
_semantic_search_def('ARC_DB_PORT', 3306);

/**
 * table name prefix to use in case using the same DB which has non - arc tables
 * */
_semantic_search_def('ARC_DB_PREFIX_NAME', 'semantic_search');

/**
* simple function wrapper for defines
* */
function _semantic_search_def($title, $default = '') {
  if (defined($title))
    watchdog('semantic_search', "$title already defined in _semantic_search_def()", WATCHDOG_WARNING);
  define($title, variable_get($title, $default));
}

function _semantic_search_get_search_form_vars($search_name){
     return db_fetch_array(db_query("SELECT *  FROM {semantic_search_searches} WHERE name='%s' LIMIT 1",$search_name));
}


function _semantic_search_set_search_field_vars($form_values){
 $search_id = $form_values['search_id'];
 $fid= isset($form_values['id'])?$form_values['id']:null;
 $title = $form_values['title'];
 $description = $form_values['description'];
 $uris = $form_values['uris'];
 $form_type = $form_values['form_type'];
 $search_field_type = $form_values['search_field_type'];
 $title_as_option = $form_values['title_as_option'];
 $grand_total = $form_values['grand_total'];
 $multiple = $form_values['multiple'];
 
 if ($fid === null){  
    db_query("INSERT INTO {semantic_search_searches_fields} 
    (title, description, uris, form_type, search_field_type, title_as_option, grand_total, multiple, sid) 
     VALUES 
    ('%s','%s','%s','%s','%s',%d,%d,%d,%d) ",
    $title,$description,$uris,$form_type,$search_field_type, $title_as_option,$grand_total,$multiple,$search_id);
    $message = t('Created field @a for search @b',array('@b'=>$title));
    drupal_set_message('Semantic Search: '.$message);
    watchdog('Semantic Search',$message);
    return mysql_insert_id(); // not the drupal way, I know
  } else {
    //update
    db_query("UPDATE {semantic_search_searches_fields} SET  
    title='%s', description='%s', uris='%s', form_type='%s', search_field_type='%s',title_as_option=%d,grand_total=%d,multiple=%d  
    WHERE id=%d ",
    $title,$description,$uris,$form_type,$search_field_type, $title_as_option,$grand_total,$multiple,$fid);
    $message = t('Updated field @a for search @b',array('@b'=>$title));
    drupal_set_message('Semantic Search: '.$message);
    watchdog('Semantic Search',$message);  
    return $fid;
  }
}

function _semantic_search_get_search_field_vars($fid){
  if ($fid === null) return array();
  return db_fetch_array(db_query("SELECT * FROM  {semantic_search_searches_fields}  WHERE id=%d LIMIT 1",$fid));
}

function _semantic_search_get_all_search_field_vars($search_name){
  $results = db_query("SELECT b.* FROM  {semantic_search_searches} as a, {semantic_search_searches_fields} as b  WHERE a.name='%s' AND a.id=b.sid",$search_name);
  $return_array = array();
  while( $result = db_fetch_array($results)){
     $return_array[] = $result;
  }
  return  $return_array;
}

function _semantic_search_set_search_form_vars(&$form_values, $op){
  $search_name = $form_values['search_name'];
  $search_store_type = $form_values['search_store_type'];
  $search_type = $form_values['search_type'];
  $search_path = $form_values['search_path'];
  $search_title = $form_values['search_title'];
  $search_description = $form_values['search_description'];
  
  if ($op == 'Create'){
    db_query("INSERT INTO {semantic_search_searches} (name, type, store_type, path, title, description) VALUES ('%s','%s','%s','%s','%s','%s') ",$search_name,$search_type,$search_store_type,$search_path,$search_title, $search_description);
    $message = t('Created search @s',array('@s'=>$search_name));
    drupal_set_message('Semantic Search: '.$message);
    watchdog('Semantic Search',$message);
  } else {
    //update
    $search_id = $form_values['search_id'];
    //search name can not be changed
    db_query("UPDATE {semantic_search_searches} SET  type='%s', store_type='%s', path='%s', title='%s', description='%s' WHERE id=%d ",$search_type,$search_store_type,$search_path,$search_title, $search_description,$search_id);
    $message = t('Updated search @s',array('@s'=>$search_name));
    drupal_set_message('Semantic Search: '.$message);
    watchdog('Semantic Search',$message);
  }
 
 // node_settings   text  utf8_unicode_ci     No      Browse distinct values  Change  Drop  Primary   Unique  Index   Fulltext
 
  return $search_name;
}


