<?php

//DEFINES
/**
 * A script to be called directly which runs through the loading and running
 * of a few basic SONIA tests.
 * 
 * Cache functions are not yet tested as that requires Drupal, and these tests deal
 * less with performant issues than basic functionality
 * 
 * @license GPLv2 
 * @license 2006 KBCMS LLC
 * @author  Jonathan Hendler
 * @package SONIA
 * @version 0.3
 * */
class ST
{
  /**
   * total number of tests to be run
   * */
   public $test_count;
   
   /**
    * cleanup the database when finished
    * */
   public $do_cleanup;
   
   /**
    * */
   public $start_time;
   
   /***/
   public $end_time;
   
   
   /**
    * 
    * */
   public $total_success;
   
  /**
   * why have the functionality outsisw the builtin function?
   * so it can be tested outside the builtin function
   * */
  function __construct($do_cleanup = true)
  {
    $this->init();
    $this->do_cleanup = $do_cleanup;
    $this->base_uri = 'http://mydrupalsite.com/node/';
    $this->total_success = true;
   
  }
  
  function __destruct()
  {
    global $max_memory_usage;
    $memory = "Max Memory used: $max_memory_usage";
    $total_time = $this->end_time - $this->start_time;
    $content = 'Note: The get_memory_usage is emulated on some machines. On a P4HT 2.8 Ghz this was a 1 second delay.';
    printTestName("Time Elapsed: $total_time seconds. &nbsp;&nbsp;$memory", $content);
    if($this->do_cleanup) $this->testcleanup();
    
  }
  
  function runtests( )
  {
     global $tests;
     $this->start_time = this_microtime();
     
     
     foreach($this->tests as $test)
     {
       $this->{$test}();
     }
     
     $this->end_time = this_microtime();
  }
  
  
  //Load SONIA
  

  
  /**
   * create a class which needs an ID
   * then create a class which does not need an id
   * 
   * */
  function create_classes()
  {
    $success = false;
    $content = '';
    
    $this->createClassesAutoID();
    $this->createClassesWithID();
  
  }
  
  function create_superclasses()
  {
    $content = 'All Passed';
    $success = false;
    $base_uri = 'http://mydrupalsite.com/node/';
     for($i = 0; $i<10;$i++)
     {
        if ($i >0)
        {
          $this->sonia->store->setSubclass($base_uri.$i,$base_uri.($i-1));
          $content .= $base_uri.$i." set as subclass of ".$base_uri.($i-1)."<br/>";
        }
     }
     
     $superclasses = $this->sonia->store->getAllCategoriesRecursive($base_uri.'4',0,true);
     
     $depth =  ArrayDepth($superclasses);
     $success = ($depth == 10);
     if (!$success)
     { 
       $content = ' ';
       $this->total_success = false;
     }
     
     //now check to see that they are there
     printTest('Set Subclases ', 'Set ID based Classes as subclasses of eachother, tested this was true',$content,$success);
  
   }
  
  function delete_subclasses()
  {
    $content = 'All Passed';
    $success = false;
    $base_uri = 'http://mydrupalsite.com/node/';
    $superclasses = $this->sonia->store->getAllCategoriesRecursive( );
    dbg($superclasses,'$superclasses?');
    $this->sonia->deleteClass($base_uri.'4');
    $superclasses = $this->sonia->store->getAllCategoriesRecursive($base_uri.'0',0,true);
    
     $depth =  ArrayDepth($superclasses);
     $success = ($depth == 6);
     if (!$success)
     { 
       $content = 'should be 6  $depth ='.$depth;
       dbg($superclasses,'$superclasses?');
       $this->total_success = false;
     }
     
    printTest('Deleted Subclases ', 'Deleted Subclasses from ID 4 and below, tested this was true',$content,$success);
  }
  
  /**
   * update the labels , add and alter descriptions
   * add and alter tags 
   * **/
  function update_classes()
  {
    $content = 'All Passed';
    $success = true;
    $base_uri = 'http://mydrupalsite.com/node/';
    $original_class_label = 'My First Class';
    $class_one = $base_uri.'0';
    $class_two = $base_uri.'1';
    
    $label_1_before = $this->sonia->getLabel($class_one);
    if ($label_1_before != $original_class_label.' 0')
    {
      $success = false;
      $content = "$label_1_before != $original_class_label";
    }
    
    $label_2_before = $this->sonia->getLabel($class_two);
    if ($label_2_before != $original_class_label.' 1') 
    {
      $success = false;
      $content = "$label_2_before != $original_class_label";
    }
    
    $new_label_1 = "Second Class Name"; 
    $new_label_2 = "Second Class Name Updated too"; 
    
    $this->sonia->updateClass($class_one,$new_label_1);
    $this->sonia->updateClass($class_two,$new_label_2);
    
    
    $label_1_after = $this->sonia->getLabel($class_one);
    $label_2_after = $this->sonia->getLabel($class_two);
    
    if ($label_1_after != $new_label_1) 
    {
      $success = false;
      $content = "$label_1_after != $new_label_1";
    }
    
    if ($label_2_after != $new_label_2) 
    {
      $success = false;
      $content = "$label_2_after != $new_label_2";
    }
    
    $class_1_description = 'Very Very very nice. 150897 haf9upq84797-8q4-75camp(*^$^80  afa sdf af';
    
    //add comment
    $this->sonia->createComment($class_one,$class_1_description);
    
    //check comment
    $rdescription =  $this->sonia->getComment($class_one);
    
    if ($class_1_description != $rdescription) 
    {
      $success = false;
      $content = "$class_1_description != $rdescription";
    }
    
    $class_1_description = $class_1_description .' UPDATED';
    //update comment
    
    $this->sonia->updateComment($class_one,$class_1_description);
    //temp
    $this->sonia->createComment($class_one,$class_1_description);
    $rdescription =  $this->sonia->getComment($class_one);
    
    if ($class_1_description != $rdescription) 
    {
      $success = false;
      $content = " UPDATE FAILED: $class_1_description != $rdescription";
    }
    //delete comment    
    
    $this->sonia->deleteComment($class_one);
    
    $rdescription =  $this->sonia->getComment($class_one);
    
    if ('' != $rdescription) 
    {
      $success = false;
      $content = " Delete FAILED: null != $rdescription";
    }
    
    if (!$success)
     { 
       $this->total_success = false;
     }
    
    printTest('Updated classes ', 
      'Retrieved two classes Labels.  Updated two classes labels. Retrieved and tested these and comments.',$content,$success);
  }
  
  function class_properties()
  {
    $content = 'All Passed';
    $success = true;
    $base_uri = $this->base_uri;
    $original_class_label = 'My First Class';
    $class_one = $base_uri.'0';
    $class_two = $base_uri.'1';
    $class_three = $base_uri.'3';
    $count = 0;
    
    $property_label_one = ' First Clss for properties';
    $property_label_two = ' Second Clss for properties';
    //$this->base_uri
    $class_array_one = array($class_one,$class_two);
    $class_array_two = array($class_one);
    
    $property1 = $this->sonia->createProperty($property_label_one,$class_array_one );
    $property2 = $this->sonia->createProperty($property_label_two, $class_array_two);
    
    if (!$this->sonia->propertyExists($property2))
    { 
      $success = false;
      $content = "$property2 should exist!!";
    }
    //echo "<br> \$property1 $property1 \$property2 $property2 <br/>";
    
    $this->sonia->addPropertyToClass($class_two,$property2);
    
    $class1_properties = $this->sonia->getAllClassProperties($count, $class_one);
    //get all properties of each class
    $class1_properties = $class1_properties['rdfs:domain']['subclasses'];
     
    if (!array_key_exists($property1,$class1_properties) || !array_key_exists($property2,$class1_properties))
    { 
      $success = false;
      $content = "$property1 not found OR $property2 not found in class $class_one ";
    }
    
    $class2_properties = $this->sonia->getAllClassProperties($count, $class_two);
    //get all properties of each class
    $class2_properties = $class2_properties['rdfs:domain']['subclasses'];
     
    if (!array_key_exists($property1,$class2_properties) || !array_key_exists($property2,$class2_properties))
    { 
      $success = false;
      $content = "$property1 not found OR $property2 not found in class $class_two";
    }
    
    //remove property 1 from class 1
    $this->sonia->deletePropertyFromClass($class_one, $property1);
    
    $class1_properties = $this->sonia->getAllClassProperties($count, $class_one);
    //get all properties of each class
    $class1_properties = $class1_properties['rdfs:domain']['subclasses'];
    
    if (array_key_exists($property1,$class1_properties) || !array_key_exists($property2,$class1_properties))
    { 
      $success = false;
      $content = "$property1  *found* (and it should not be) OR $property2 not found. ";
      $content .= array_key_exists($property1,$class1_properties). ' should be false.';
      $content .= !array_key_exists($property2,$class1_properties). ' should be false.';
    }
    
    
    $classes_1 = $this->sonia->store->getAllClassesOfProperty($property1);
    $classes_1 = $classes_1['http://www.w3.org/2000/01/rdf-schema#Class']['subclasses'];
    $classes_2 = $this->sonia->store->getAllClassesOfProperty($property2);
    $classes_2 = $classes_2['http://www.w3.org/2000/01/rdf-schema#Class']['subclasses'];
    
    //get all classes of each property
    //check count
    if (array_key_exists($class_one,$classes_1) || !array_key_exists($class_two,$classes_1))
    { 
      $success = false;
      $content = "$class_one  *found* (and it should not be) OR $class_two not found. ";
    }
    
    if (!array_key_exists($class_one,$classes_2) || !array_key_exists($class_two,$classes_2))
    { 
      $success = false;
      dbg($classes_2,'$classes_2');
      $content = "$class_one  not found OR $class_two not found. 3";
    }
    
    
       
    
    //update label
    $new_label = 'This is the new label for property 1';
    $this->sonia->updateLabel($property1, $new_label);
    //$property_label_one_old = $property_label_one;
    $property_label_one_r = $this->sonia->getLabel($property1);
    //verify change was made
    if ( $property_label_one_r != $new_label )
    { 
      $success = false;
      $content = "label was not updated properly - should be \"$new_label\"";
    }
    
    //delete property entirely 
    $this->sonia->deleteProperty($property1);
    $class2_properties = $this->sonia->getAllClassProperties($count, $class_two);
    //get all properties of each class
    $class2_properties = $class2_properties['rdfs:domain']['subclasses'];
    
    if (array_key_exists($property1,$class2_properties) || !array_key_exists($property2,$class2_properties))
    { 
      $success = false;
      $content = "$property1  *found* (and it should not be) OR $property2 not found. ";
    }

    //create same as class
     if (!$success)
     { 
       $this->total_success = false;
     }
      printTest('CRUD Class Properities ', 
      'Properties are added, removed to classes, Searched for and deleted. ',$content,$success);
      
      //add some more properties for later tests
      for($i=0;$i<10;$i++)
      {
        $p{$i} = $this->sonia->createProperty("Label $i" );
        $this->sonia->addPropertyToClass($class_two,$p{$i});
      }
      $this->sonia->deletePropertyFromClass($class_two, $property2);
      
      $this->sonia->addPropertyToClass($class_one,$p{($i-1)});
    
  }//end function
  
  function crud_objects()
  {
    $count = 0;
    $content = 'All Passed';
    $success = true;
    $base_uri = $this->base_uri;
    $original_class_label = 'My First Class';
    $class_one = $base_uri.'0';
    $class_two = $base_uri.'1';
    $class_three = $base_uri.'2';
    
    $object_one_name = 'OBJECT ONE';
    $object_two_name = 'OBJECT TWO';
    $object_three_name = 'OBJECT THREE';
    $object_four_name = 'OBJECT FOUR';
    
    $classes_one = array($class_one);
    $classes_two = array($class_two,$class_three);
    $classes_three = array($class_one,$class_three);
    
    $class1_properties = $this->sonia->store->getAllClassPropertiesMultiple($count, $classes_one);
    $class2_properties = $this->sonia->store->getAllClassPropertiesMultiple($count, $classes_two);
    $class3_properties = $this->sonia->store->getAllClassPropertiesMultiple($count, $classes_three);
     
    $class1_properties = $class1_properties['rdfs:domain']['subclasses'];
    $class2_properties = $class2_properties['rdfs:domain']['subclasses'];
    $class3_properties = $class3_properties['rdfs:domain']['subclasses'];
   
    
    $parameters = array();
    srand(make_seed());
    
    $count = 0;
    foreach($class1_properties as $property => $label)
    {
      $randval = md5(rand(3,297824897025));
      $parameters[$count]['uri'] = $property;
      $parameters[$count]['value'] = $randval;
      $count++;
    }
    
    $object_one = $this->sonia->createObject(
        $object_one_name, //TODO not needed?
        $classes_one,
        $parameters
    );
   
    
    if ($object_one != SONIA_OBJECT_ID_URI.SONIA_AUTO_ID_PREFIX.'1')
    { 
      $success = false;
      $content = "ID was $object_one and should be ".SONIA_OBJECT_ID_URI.SONIA_AUTO_ID_PREFIX.'1';
    }
    
    //reset
    $parameters2 = array();
    
    foreach($class2_properties as $property => $label)
    {
      $randval = md5(rand(3,297824897025));
      $parameters2[$count]['uri'] = $property;
      $parameters2[$count]['value'] = $randval;
      $count++;
    }
     
   $object_two = $this->sonia->createObject(
        $object_two_name,  
        $classes_two,
        $parameters2
    );
   
    if ($object_two != SONIA_OBJECT_ID_URI.SONIA_AUTO_ID_PREFIX.'2')
    { 
      $success = false;
      $content .= "ID was $object_two and should be ".SONIA_OBJECT_ID_URI.SONIA_AUTO_ID_PREFIX.'2';
    }
    
   //in these cases provide the id
   //check for collision?
   $object_three_given = SONIA_OBJECT_ID_URI.'3';
   $object_three = $this->sonia->createObject(
        $object_three_name, //TODO not needed?
        $classes_one,
        $parameters,
        $object_three_given
    );
    
    if ($object_three_given != $object_three)
    { 
      $success = false;
      $content = "ID was $object_three_given and should be $object_three";
    }
    
    $object_four_given = SONIA_OBJECT_ID_URI.'4';;
    $object_four = $this->sonia->createObject(
        $object_four_name, //TODO not needed?
        $classes_two,
        $parameters2,
        $object_four_given
    );
   
    $sonia_search_array[] = array (
      'subject'=>null,
      'predicate'=> SONIA_INSTANCE,
      'object'=>$class_two,
      'literal_object'=>false,
      'category_query'=>true);  
    
    //this get's all objects of class 2
    $results = $this->sonia->query(0,$sonia_search_array,null,null,false,'','');
    $count = count($results);
    
    if ($count != 2)
    { 
      $success = false;
      $content .= "There should be 2 objects $class_two of but there were $count";
    }
    
    
    //this get's object 2
    $original_complete_object_two = $this->sonia->getObject($object_two);
    $count = count($original_complete_object_two);
    
    if (!($count > 1))
    { 
      $success = false;
      $content .= "There should be 1 objects based on the URI search, of ".
        htmlentities($object_two)." but there were $count";
    }
    
    
    $property_to_alter = SONIA_DEFAULT_MODEL_IRI.'/Property#id10';
    $property_to_remove = SONIA_DEFAULT_MODEL_IRI.'/Property#id12';
    $property_to_add = SONIA_DEFAULT_MODEL_IRI.'/Property#id99';
    $property_to_add_multiple_cardinality = SONIA_DEFAULT_MODEL_IRI.'/Property#id11';
    
    $property_value_of_altered_original = $this->sonia->getLiteralValueOfSearchResult($original_complete_object_two, $object_two, $property_to_alter);
    $property_value_of_dupe_original = $this->sonia->getLiteralValueOfSearchResult($original_complete_object_two, $object_two, $property_to_add_multiple_cardinality);
    
    $property_value_of_altered_original = $property_value_of_altered_original[0];
    $property_value_of_dupe_original = $property_value_of_dupe_original[0];
    
    //echo "\$property_value_of_altered_original {$property_value_of_altered_original[0]}";
    
    $property_value_of_altered_altered  = 'ALTERED VALUE';
    $p_duplicate_value = 'DUPLICATE VALUE';
    $p_new_p_value = 'NEW RANDOM PROPERTY ADDED';
    
    //alter a parameter value
    $this->sonia->updatePropertyOfObject(
      $object_two,
      $property_to_alter,
      $property_value_of_altered_original,
      $property_value_of_altered_altered
    );
    
    //create a duplicate paramater value (also see tags and relationship)
    $this->sonia->addPropertyToObject($object_two,$property_to_add_multiple_cardinality,$p_duplicate_value);
    
    //add a new paramter value
    $this->sonia->addPropertyToObject($object_two,$property_to_add,$p_new_p_value);
    
    //remove a property
    $this->sonia->removePropertyFromObject($object_two,$property_to_remove,null);
    
    //load just this object
    //test thtat the parameters have changed
    $updated_complete_object_two = $this->sonia->getObject($object_two);
    
    $new_altered_value = $this->sonia->getLiteralValueOfSearchResult($updated_complete_object_two, $object_two, $property_to_alter);
    if ($new_altered_value[0] != $property_value_of_altered_altered)
    { 
      $success = false;
      $content .= "\$new_altered_value $new_altered_value should be $property_value_of_altered_altered";
    }
    
    
    $new_property_value = $this->sonia->getLiteralValueOfSearchResult($updated_complete_object_two, $object_two, $property_to_add);
    if ($new_property_value[0] != $p_new_p_value)
    { 
      $success = false;
      $content .= "The new property of $property_to_add with value $p_new_p_value was not found ";
    }
    
    $results = $this->sonia->propertyValuesQuery($property_to_add_multiple_cardinality);
    $rcount = count($results);
    if ($rcount != 2 ||  $results[$p_duplicate_value]['count'] != 1 ||  $results[$p_duplicate_value]['objects'][0] != $object_two)
    { 
      $success = false;
      $content .= "$object_two should have 1 value of a property $property_to_add_multiple_cardinality";
      dbg($results,'$results from property vlues query in test $p_duplicate_value = '.$p_duplicate_value);
    }
    
    
    $duplicate_values = $this->sonia->getLiteralValueOfSearchResult($updated_complete_object_two, $object_two, $property_to_add_multiple_cardinality);
    if (count($duplicate_values) != 2)
    { 
      $success = false;
      $content .= "There should be 2 property values present for  $property_to_add_multiple_cardinality of but there were $count";
    }
    
    
    $removed_value = $this->sonia->getLiteralValueOfSearchResult($updated_complete_object_two, $object_two, $property_to_remove);
    if (!empty($removed_value))
    { 
      $success = false;
      $content .= "There should be not value for property $property_to_remove.";
    }
    
//    //update an aobjects association with certain classes
    $parameters = array();
    $this->sonia->updateObject(
        $object_two ,
        $label, 
        $classes_three,
        $parameters
    );       
    
    $updated_complete_object_two = $this->sonia->getObject($object_two);
    
    $belongs_to_class_two = false;
    $belongs_to_class_one = false;
    if (is_array($updated_complete_object_two))
    {
      foreach($updated_complete_object_two as $values)
      {
        if ($values['o'] == $class_two) $belongs_to_class_two = true;
        if ($values['o'] == $class_one) $belongs_to_class_one = true;
      }
    }
    
    if ($belongs_to_class_two || !$belongs_to_class_one)
    {
      $success = false;
      $content .= "object 2 should not be of class $class_two ";
    }
    
    $this->sonia->updateObject(
        $object_two ,
        $label, 
        $classes_two,
        $parameters
    );       
    
    $updated_complete_object_two = $this->sonia->getObject($object_two);
    
    $belongs_to_class_two = false;
    $belongs_to_class_one = false;
    
    if (is_array($updated_complete_object_two))
    {
      foreach($updated_complete_object_two as $values)
      {
        if ($values['o'] == $class_two) $belongs_to_class_two = true;
        if ($values['o'] == $class_one) $belongs_to_class_one = true;
      }
    }
    
    if (!$belongs_to_class_two || $belongs_to_class_one)
    {
      $success = false;
      $content .= "object 2 SHOULD be of class $class_two \$belongs_to_class_two= $belongs_to_class_two \$belongs_to_class_three= $belongs_to_class_three ";
      dbg($updated_complete_object_two,'$updated_complete_object_two');
    }
    
    
    $filename = 'sillywalk.jpg';
    
    //add image to object
    $image_uri = $this->sonia->createImage(
      $object_two,
      'SILLY IMAGE',
      'Description of Silly Image',
      'sillywalk.jpg',
      '200k (estimated data)',
      './'
    );
    
    $object_two_with_images = $this->sonia->getObjectWithFiles($object_two);
    
    if (!isset($object_two_with_images['images'][0]) || !file_exists('./'.$filename))
    {
      $success = false;
      $content .= "image should display";
      dbg($object_two_with_images,'$object_two_with_images');
    }
    else
    {
      $image_info = $object_two_with_images['images']['0'];
      $image = SONIA_H :: getFileInfo($image_info);
      echo '<br/>'.$image['label'].' :<img src="'.$image['path'].$image['filename'].'" title="'.$image['comment'].'" alt="'.$image['comment'].'" /></br>';
    
    }
    
    $image_info = $this->sonia->store->getFileInfoByFilename($filename);     
    $image = SONIA_H :: getFileInfo($image_info);
      echo '<br/>'.$image['label'].' :<img src="'.$image['path'].$image['filename'].'" title="'.$image['comment'].'" alt="'.$image['comment'].'" /></br>';
    
    

    $results = $this->sonia->store->getClassesAndCounts();
    
    dbg($results,'getClassesAndCounts() ' );
    
    
   $object_two = $this->sonia->createObject(
        $object_two_name,  
        array($class_one),
        $parameters2
    );
   
    $results = $this->sonia->store->getPropertiesAndCounts($class_one);
    dbg($results,'getPropertiesAndCounts($class_one) ' );
    
    $results = $this->sonia->store->getPropertiesAndCounts();
    dbg($results,'getPropertiesAndCounts() ' );
    $this->search_classes();
//    //get the objects classes
//    // (see below also)
//    $object_two_retrieved_classes = $this->sonia->getObjectsDirectClasses($object_two);
//    //test that the classes match
//    $all_classes_found = false;
//    foreach($object_two_retrieved_classes as $class)
//    {
//      if (!)
//    }
//    
//    if (!$all_classes_found)
//    { 
//      $success = false;
//      $content .= "$object_two should belog to the same classes as it was set to.";
//    }
//    
      
//    $this->sonia->store->debug_text = '';
//    dbg($updated_complete_object_two);
//    echo $this->sonia->store->debug_text;
    
    ////////////////////////////////////////////////////////////////////////////
    //delete an object by deleteting the class
    $this->sonia->deleteClass($class_two, $delete_subclasses = true, $delete_all_objects_related = true);
    $updated_complete_object_two = $this->sonia->getObject($object_two);
    
    $sonia_search_array = array();
    //search for that object
    $sonia_search_array[] = array (
      'subject'=>null,
      'predicate'=> SONIA_INSTANCE,
      'object'=>$class_two,
      'literal_object'=>false,
      'category_query'=>true);  
    
    //this get's all objects of class 2
    $results = $this->sonia->query(0,$sonia_search_array,null,null,false,'','');
    $count = count($results);
    
    $belongs_to_class_two = false;
    if (is_array($updated_complete_object_two))
    {
      foreach($updated_complete_object_two as $values)
      {
        if ($values['p'] == $class_two) $belongs_to_class_two = true;
      }
    }
    
//    echo 'count($results) '.count($results).' count($updated_complete_object_two) 
//         '.count($updated_complete_object_two);
    if ($count != 0 || $belongs_to_class_two)
    { 
      $success = false;
      $content .= "There should be NO objects $class_two of but there were $count";
     // dbg($updated_complete_object_two,'$updated_complete_object_two');
      //dbg($results,'$results');
    }
    ////////////////////////////////////////////////////////////////////////////
    // Tags and relationships 
    //create aribtrary relationships (mutliple cardinality)
    // literal :has tag
    // resource :similarTo    
    
      
    ////////////////////////////////////////////////////////////////////////////
    ////LATER create object 2 with checking parameters for
    // type of data
    // which fields are required
    // owner
    
     if (!$success)
     { 
       $this->total_success = false;
     }
     
      printTest('CRUD Objects', 
      'Objects are created, modified and deleted. Their properties added, changed',$content,$success);
  }
  
  function search_classes()
  {
    $content = 'All Passed';
    $success = true;
    $base_uri = $this->base_uri;
    $original_class_label = 'My First Class';
    $class_one = $base_uri.'0';
    $class_two = $base_uri.'1';
    $class_three = $base_uri.'3';
    
    //get all classes and their labels flat
    //get count of related objects
    //$result = $this->sonia->getAllClassesFlat();
    //$all_classes_count = count($result);
    
    //dbg($result,'$result $all_classes_count= '.$all_classes_count);
    $result = $this->sonia->getAllCategoriesRecursive();
    if (empty($result) || $all_classes_count != 21)
    { 
      $success = false;
      $content = "getAllCategoriesRecursive() failed \$all_classes_count = $all_classes_count";
     // dbg($result,'$result = $this->sonia->getAllCategoriesRecursive();');
      
    }
    
    //test format
    $result = $this->sonia->classQuery(null);
    $all_classes_count = count($result);
    
    
    
    $result = $this->sonia->propertyQuery();
    $all_property_count = count($result);
    //dbg($result,'$result $all_property_count= '.$all_property_count);
    //get all classes and their labels in hiearchy
    //$result = $this->sonia->getAllCategoriesRecursive();
    
    //give a class get objects and object labels
    
    //give a class and get properies and labels
    
    
    //give an object and get properties
    
    //give and object and get classes and their labels
    
    
    //give a property label and
    
    
    //give a property ID and get classes which have it
    
    //give a property ID and get objects which have it
    
    
    //get all classes belonging to an object
    
    
    
        $test_query = 
'PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
PREFIX owl: <http://www.w3.org/2002/07/owl#>
PREFIX foaf: <http://xmlns.com/foaf/0.1/>
PREFIX semantic_search: <http://semanticsearch.org/ns/semantic_search#>
PREFIX sonia: <http://semanticsearch.org/ns/sonia#>
PREFIX soniap: <http://semanticsearch.org/ns/sonia_test/Property#>
PREFIX carsf: <http://semanticsearch.org/ns/car-spacefinder#>
SELECT  DISTINCT ?s ?p  ?o 
WHERE {
?s ?p ?o .


}';

//?s <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> ?type .
//?type <http://www.w3.org/2000/01/rdf-schema#label> ?t_label .

//?p <http://www.w3.org/2000/01/rdf-schema#label> ?p_label .
//?s <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> ?type .
//?type <http://www.w3.org/2000/01/rdf-schema#label> ?t_label .

 
 //?p_label ?o ?t_label ?type
// ?p rdfs:label ?p_label .
//?s rdf:type ?type .
//?type rdfs:label ?t_label .
   
     $results = $this->sonia->store->query($test_query, false, false);
     // print_r($results);
    
//    if (array_key_exists($property1,$class2_properties) || !array_key_exists($property2,$class2_properties))
//    { 
//      $success = false;
//      $content = "$property1  *found* (and it should not be) OR $property2 not found. ";
//    }
        
    //get object, and it's params and labels by $uri
    //get object, and it's params and labels by main label

      
      
     if (!$success)
     { 
       $this->total_success = false;
     }
     printTest('Search tests complete ', 'Searches classes, objects and properties',$content,$success);
  }
  
  function add_properties_to_superclass()
  {
    $content = 'All Passed';
    $success = true;
    $base_uri = $this->base_uri;
    $original_class_label = 'My First Class';
    $class_one = $base_uri.'0';
    $class_two = $base_uri.'1';
  }
  
  function  test_subclass_properties()
  {
    $content = 'All Passed';
    $success = true;
    $base_uri = $this->base_uri;
    $original_class_label = 'My First Class';
    $class_one = $base_uri.'0';
    $class_two = $base_uri.'1';
  }
  
  
  /**
   * create classes with IDs provided, like drupal's Node ID
   * */
   function createClassesWithID()
   {
      $success = false;
      $content = '';
      $class_amount = 10;
      
      $base_uri = 'http://mydrupalsite.com/node/';
      for($i = 0; $i<10;$i++)
      {
        $uri = $this->sonia->createClass('My First Class '.$i,null,$base_uri.$i);
        $content .= "Created $uri <br/>";
      }
      
      $exists = $this->sonia->classExists($uri) ;
      
      //measure success by if the URIs match
      $success = (($base_uri.($i-1) == $uri) && (SONIA_H :: is_IRI($uri)) && $exists);
      if (!$success) $content .= "ERROR: $base_uri".($i-1)." == $uri ?";
      $id1 = SONIA_H :: getIDFromURI($uri);
      
      $class_with_given_id_with_underscore = $base_uri.'funny_name_is';
      $uri = $this->sonia->createClass('My First Class',array(),$class_with_given_id_with_underscore);
      $exists = $this->sonia->classExists($class_with_given_id_with_underscore) ;
      
      if (!$exists)
      { 
        $success = false;
        $content = "ID was $class_with_given_id_with_underscore was not created";
      }
      
      
       if (!$success)
       { 
         $this->total_success = false;
       }
      printTest('Creating '.$class_amount.' ID included classes ', 'Provides an ID in the form of a URI ',$content,$success);
      //test to make sure duplicates are not allowed?
   }
  
  /**
   * create classes that require an autoID
   * */
  function createClassesAutoID()
  {
    
    ///
    $success = false;
    $content = '';
    $uri1 = $this->sonia->createClass('My First Class',null);
    $id1 = SONIA_H :: getIDFromURI($uri1);
    $content = "created class $uri1 with id <strong>$id1</strong>";
    $success = SONIA_H :: is_IRI($uri1) && ($id1 == SONIA_AUTO_ID_PREFIX.'1') ; 
    if (!$success) $content = "Retrieved id ".$id1." from class $uri1";
    printTest('Creating Standard Class 1', 'Requires the generation of an ID ',$content,$success);
    
    
    //////////////////////////////////////////////////
    //create 10 more
    $created_classes = 10;
    
    for($i=0;$i<$created_classes;$i++)
    {
      $turi = $this->sonia->createClass('CLASS '.$i,null);
    }
    $tid = SONIA_H :: getIDFromURI($turi);
    $success = SONIA_H :: is_IRI($turi) && ($tid == SONIA_AUTO_ID_PREFIX.'11') ; 
    $content = "created class $turi with id <strong>$tid</strong>";
    if (!$success) $content = "Retrieved id ".$tid." from class $turi";
     if (!$success)
     { 
       $this->total_success = false;
     }
    printTest('Created '.$created_classes.' classes with Genderated IDs', 'Requires the generation of an ID ',$content,$success);
    ////////////////////////////////////////////
  }
  
  
  //destroy ARC DB
  
  
  
///////////////////////////////////////////////////////////////////////////////  
  /// these functions shouldn't change
  
  /**
   * bootstrap order is very important :)
   * */
  function init()
  { 
    global $tests;
    
    $this->tests = $tests;
    $this->test_count = count($tests);
    
    printTestName('Starting Sonia Tests', 'Beggining '.$this->test_count.' tests of Sonia.
      <br />PHP Version '. phpversion());
    
    
    require SONIA_LOCATION.'/sonia.php';
    //init sonia 
    
    $this->sonia = new SONIA(
      SONIA_BACKEND_TYPE,
      SONIA_FORM_ARRAY_PREFIX,
      'drupal',
      null,
      null,
      true //create a database
    );
    
    $port = ARC_DB_PORT;
    if (!empty( $port ))
    {
      $port = ':' . $port;
    }
    
    
    $this->dbconn = new SONIA_TEST_MYSQL_DB(ARC_DB_NAME,ARC_DB_PW,ARC_DB_USER,ARC_DB_HOST.$port);
    $sql = 'SELECT * FROM `'.SONIA_DB_PREFIX_NAME.'_id2val`';
    $result = $this->dbconn->get_array($sql);
    $content = 'Passed uneventfully.';
    $success = (!empty($result[0]['id']));
    if (!$success) $content = 'Test may have failed because the tables already exist. Change SONIA_TEST_CLEANUP in your config file';
    printTest('Creating DB', 'Creates an Arc Database',$content,$success);
    
  }
  
    //Create ARC DB
  function create_database()
  {
    if (!class_exists('SONIAARCStore'))
    { 
      set_sonia_configs('sonia_arc_local','drupal'); 
    }
    
    $this->store       = new SONIAARCStore(null);
    //basic check
    $success = $this->store  instanceof SONIAStoreAPI;
    $content = '';
    
    printTest('Creating Store Object', 'Creates a SONIA store object.',$content,$success);

    $content = 'Passed uneventfully.';
    $table_names = $this->store->installBackend('sprintf');
    //print_r($table_names);
    $success = (!empty($table_names))? true  : false;
    
     
  }
  
  
  /**
   * @param string truncate or drop
   * */
  function testcleanup($action = 'DROP')
  {
     //`".SONIA_DB_PREFIX_NAME."_store_var`, 
     //remove databases
     $content = '';
     $sql = "$action TABLE `".SONIA_DB_PREFIX_NAME."_id2val`, `".SONIA_DB_PREFIX_NAME."_triple`, `".SONIA_DB_PREFIX_NAME."_triple_dup`;";
     $success = $this->dbconn->query($sql);
     if (!$success)
     { 
       $this->total_success = false;
       $content = print_r($success,1);
       $content .= mysql_error();
     }
     printTest('Database Cleaned', ' The database has removed traces of the test ',$sql,$success);
     printTest('ALL TESTS STATUS', ' The status of all tests',$content, $this->total_success);
  }

} //end class
?>